

#include "accion.h"
#include "tablero.h"


// Constructor por defecto
accion::accion ()
  : _sgtServicio ( false ),
    _encarcelar ( false ),
    _avanzarA ( -1 ),
    _avanzar ( 0 ),
    _pagar ( 0 ),
    _sgtEstacion ( false ),
    _tarjetaCarcel ( false ),
    _pagarCasa ( 0 ),
    _pagarHotel ( 0 ),
    _pagarCadaJugador ( 0 ),
    _multiplicarPago ( 1 ),
    _cobrarSalida ( true ),
    _multiPagoDado ( 0 ),
    _sgtPropiedad ( false )
{
  // No se hace nada!
}

// constructor explcito
accion::accion ( int pagar, int avanzar, int avanzarA, int pagarCadaJugador, 
		 int pagarCasa, int pagarHotel, int multiPago, bool sgtServicio, bool encarcelar,
		 bool sgtEstacion, bool tarjetaCarcel, bool cobrarSalida, int multiPagoDado, 
		 bool sgtPropiedad )

  : _sgtServicio ( sgtServicio ),
    _encarcelar ( encarcelar ),
    _avanzarA ( avanzarA ),
    _avanzar ( avanzar ),
    _pagar ( pagar ),
    _sgtEstacion ( sgtEstacion ),
    _tarjetaCarcel ( tarjetaCarcel ),
    _pagarCasa ( pagarCasa ),
    _pagarHotel ( pagarHotel ),
    _pagarCadaJugador ( pagarCadaJugador ),
    _multiplicarPago ( multiPago ),
    _cobrarSalida ( cobrarSalida ),
    _multiPagoDado ( multiPagoDado ),
    _sgtPropiedad ( sgtPropiedad )
{
  // No se hace nada!
}
 
// Copy constructor
accion::accion (const accion& right)
{
  _sgtServicio		= right._sgtServicio;  
  _encarcelar		= right._encarcelar;
  _avanzarA		= right._avanzarA;
  _avanzar		= right._avanzar;
  _pagar		= right._pagar;
  _sgtEstacion		= right._sgtEstacion;
  _tarjetaCarcel	= right._tarjetaCarcel;
  _pagarCasa		= right._pagarCasa;
  _pagarHotel		= right._pagarHotel;
  _pagarCadaJugador	= right._pagarCadaJugador;
  _multiplicarPago	= right._multiplicarPago;
  _cobrarSalida		= right._cobrarSalida;
  _multiPagoDado	= right._multiPagoDado;
  _sgtPropiedad		= right._sgtPropiedad;
}

// Assignment operator
accion& accion::operator= (const accion& right)
{  
  _sgtServicio		= right._sgtServicio;  
  _encarcelar		= right._encarcelar;
  _avanzarA		= right._avanzarA;
  _avanzar		= right._avanzar;
  _pagar		= right._pagar;
  _sgtEstacion		= right._sgtEstacion;
  _tarjetaCarcel	= right._tarjetaCarcel;
  _pagarCasa		= right._pagarCasa;
  _pagarHotel		= right._pagarHotel;
  _pagarCadaJugador	= right._pagarCadaJugador;
  _multiplicarPago	= right._multiplicarPago;
  _cobrarSalida		= right._cobrarSalida;
  _multiPagoDado	= right._multiPagoDado;
  _sgtPropiedad		= right._sgtPropiedad;
  return ( *this );
}

// Destructor
accion::~accion ()
{
  // No se hace nada!
}



/**
 * Mtodo que ejecuta una instancia de la clase accin y hace que los efectos le afecten a un jugador
 *
 * Este mtodo comprobar los valores de todos los atributos de la clase, y decidir que accin debe realizar.
 * En primer lugar se comprobar si debe multiplicar el pago por algn valor, y en caso afirmativo modificar
 * el valor necesario de la instancia jugador. A continuacin seguir ejecutando el resto de las acciones.
 *
 *  IN
 *     - j: jugador sobre el que recae la accin
 */
int accion::ejecutar ( jugador *j )
{
  int ok = 0;
  int accionTarjeta = 0; 

  //Si un jugador debe multiplicar su siguiente pago por un nmero dado, se modifica su atributo _multiPago
  if ( get_multiplicarPago () > 1 )
    {
      cout << "Se multiplicar la cantidad que se abone por " << get_multiplicarPago () << endl;
      j->set_multiPago ( get_multiplicarPago () );
      if ( get_multiPagoDado() > 0 )
	cerr << "Aviso: Existe un error en la definicin de la tarjeta (se ha identificado que se debe multiplicar el pago por una cantidad fija y por la cantidad obtenida de lanzar los dados. Por defecto se elige la primera opcin. Revise la definicin de las tarjetas." << endl;
    }
  //Si debe multiplicarlo por el valor que determinen los dados, se lanzarn los dados y luego se modificar _multiPago de jugador
  else if ( get_multiPagoDado() > 0 )
    {
      cout << "Pulsa 'intro' para lanzar el dado para ver por que cantidad se debe multiplicar su pago.";
      getchar();
      bool tiradaDoble;
      int dado1, dado2;      
      int total = j->tirarDado ( tiradaDoble, dado1, dado2 );
      cout << "DADO 1 = " << dado1 << "  DADO 2 = " << dado2 << "  TOTAL = " << total << endl;
      int multi = get_multiPagoDado ();
      j->set_multiPago ( multi * total );      
    }
  //Accion para salir libre de la crcel. Se aadir a la lista de tarjetas del jugador
  if ( get_tarjetaCarcel() == 1 )
    {
      accionTarjeta = 1;
    }
  //Mover a la siguiente estacin al jugador
  if ( get_sgtEstacion() )
    {      
      int sgt = tablero::buscarSiguienteCasillaTipoInt ( j, "estacion" );
      if ( sgt != -1 )
	{
	  if ( get_cobrarSalida() == 1 )
	    tablero::pagarSalario ( j, sgt );      
	  ok = tablero::moverAAccion ( j, sgt );
	}
      else
	cout << "No hay estaciones en el juego" << endl;
    }
  //Mover a la siguiente utilidad o servicio
  if ( get_sgtServicio() )
    {      
      int sgt = tablero::buscarSiguienteCasillaTipoInt ( j, "servicio" );
      if ( sgt != -1 )
	{
	  if ( get_cobrarSalida() == 1 )
	    tablero::pagarSalario ( j, sgt );
	  ok = tablero::moverAAccion ( j, sgt );
	}
      else
	cout << "No hay casillas de servicio en el juego" << endl;
    }
  //Mover a la siguiente calle
  if ( get_sgtPropiedad() )
    {      
      int sgt = tablero::buscarSiguienteCasillaTipoInt ( j, "calle" );
      if ( sgt != -1 )
	{
	  if ( get_cobrarSalida() == 1 )
	    tablero::pagarSalario ( j, sgt );
	  ok = tablero::moverAAccion ( j, sgt );
	}
      else
	cout << "No hay calles en el juego" << endl;
    }
  //Avanzar a la casilla dada por el atributo _avanzarA
  if ( get_avanzarA() >= 0 )
    {      
      if ( get_cobrarSalida() == 1 )
	tablero::pagarSalario ( j, get_avanzarA() );      
      ok = tablero::moverAAccion ( j, get_avanzarA() );      
    }
  //Encarcelar al jugador j
  if ( get_encarcelar() )
    {
      tablero::encarcelar ( j );
    }
  //Avanzar el nmero de casillas dado por el atributo _avanzar
  if ( get_avanzar() != 0 )
    {
      ok = tablero::mover ( j, get_avanzar(), false, get_cobrarSalida () );
    }
  //Pagar la cantidad indicada en _pagar (Si es negativo, representa cobrar)
  if ( get_pagar() != 0 )
    {
      if ( get_pagar () > 0 )
	cout << "Se descontarn de tu saldo " << get_pagar() << endl;
      else
	cout << "Enhorabuena, tu saldo ha sido incrementado " << 0 - get_pagar() << endl;     
      long saldo = j->pagar ( get_pagar() );
      if ( saldo >= 0 )
	cout << "Dispones de " << saldo << " en dinero" << endl;
      else
	cout << "No has podido pagar, no tienes suficiente dinero" << endl;
    }
  //Pagar la cantidad indicada por cada casa construida
  if ( get_pagarCasa() > 0 )
    {      
      int totalCasas = j->get_casasConstruidas ();
      if ( totalCasas > 0 )
	{
	  long pagoCasa = get_pagarCasa();
	  long pago = pagoCasa * totalCasas;
	  cout << "Debes pagar " << pago << " por tus " << totalCasas << " casas" << endl;
	  if ( j->get_dinero_total() < pago )
	    ok = -1;
	  else 
	    ok = 0;
	  long saldo = j->pagar ( pago );
	   if ( saldo >= 0 )
	     cout << "Dispones de " << saldo << " en dinero despus de pagar tus casas" << endl;
	   else
	     cout << "No has podido pagar los impuestos por las casas, no tienes suficiente dinero" << endl;
	}
      else
	{
	  cout << "No tienes que pagar nada por tus casas ya que no tienes ninguna" << endl;
	}  
    }
  //pagar la cantidad indicada por cada hotel construido
  if ( get_pagarHotel() > 0 )
    {      
      int totalHoteles = j->get_hotelesConstruidos ();
      if ( totalHoteles > 0 )
	{
	  long pagoHotel = get_pagarHotel();
	  long pago = pagoHotel * totalHoteles;
	  cout << "Debes pagar " << pago << " por tus " << totalHoteles << " hoteles" << endl;
	  if ( j->get_dinero_total() < pago )
	    ok = -1;
	  else 
	    ok = 0;
	  long saldo = j->pagar ( pago );
	   if ( saldo >= 0 )
	     cout << "Dispones de " << saldo << " en dinero despus de pagar tus hoteles" << endl;
	   else
	     cout << "No has podido pagar los impuestos por los hoteles, no tienes suficiente dinero" << endl;
	}
      else
	{
	  cout << "No tienes que pagar nada por tus hoteles ya que no tienes ninguno" << endl;
	}      
    }
  //Pagar ( o cobrar ) a todos los dems jugadores.
  if ( get_pagarCadaJugador () != 0 )
    tablero::pagarATodos ( j, get_pagarCadaJugador () );
  
  j->set_multiPago ( 1 ); 
  return accionTarjeta;
}
