/**
 *  CLASE ACCION:
 *  Clase encargada de representar todas las acciones que pueden darse en una tarjeta del
 *  monopoly. Esta clase soporta tarjetas de los tipos:
 *     - Avanzar a siguiente casilla de utilidad (servicio), calle o estacin.
 *     - Avanzar x casillas, o hasta una casilla dada.
 *     - Pagar por casas y hoteles construidos.
 *     - Pagar / cobrar a todos los jugadores, a uno en concreto o a la banca.
 *     - Mover el jugador a una casilla y hacer que pague X veces su valor.
 *  La idea de esta clase es que ejecute todas las acciones posibles una detrs de otra,
 *  cubriendo todas las posibles necesidades que se han identificado en las tarjetas del
 *  juego Monopoly.
 *
 *  Requiere:
 *     - clase jugador
 *     - clase string (STL)
 *
 *  Autor: Javier Aragn Zabalegui (PFC para la uc3m: street Master's
 *  Contacto: 100039831@alumnos.uc3m.es
 *  version 1.0
 */


#ifndef __ACCION__
#define __ACCION__

#include <string>
using namespace std;

#include "jugador.h"


class accion {

 private:

  //Atributos
  bool _sgtServicio;
  bool _encarcelar;
  int _avanzarA;
  int _avanzar;
  int _pagar;              //Positivo pagar, negativo cobrar.
  bool _sgtEstacion;
  bool _tarjetaCarcel;
  int _pagarCasa;
  int _pagarHotel;
  int _pagarCadaJugador;
  int _multiplicarPago;  
  bool _cobrarSalida;
  int _multiPagoDado;
  bool _sgtPropiedad;

 public:

  // Constructores
  accion ();  
  accion ( int pagar, int avanzar, int avanzarA, int pagarCadaJugador, 
	   int pagarCasa, int pagarHotel, int multiPago, bool sgtServicio, bool encarcelar,
	   bool sgtEstacion, bool tarjetaCarcel, bool cobrarSalida, int multiPagoDado, 
	   bool sgtPropiedad );
  
  // Copy constructor
  accion ( const accion& right );

  // Operador de asignacin
  accion& operator= (const accion& right);

  // Destructor
  ~accion ();

  // Get operators
  bool get_sgtServicio () const;
  bool get_encarcelar () const;
  int get_avanzarA () const;
  int get_avanzar () const;
  int get_pagar () const;
  bool get_sgtEstacion () const;
  bool get_tarjetaCarcel () const;
  int get_pagarCasa () const;
  int get_pagarHotel () const;
  int get_pagarCadaJugador () const;
  int get_multiplicarPago () const;
  bool get_cobrarSalida () const;
  int get_multiPagoDado () const;
  bool get_sgtPropiedad () const;

  //Set operators
  void set_sgtServicio ( bool sgtServicio );
  void set_encarcelar ( bool encarcelar );
  void set_avanzarA ( int avanzarA );
  void set_avanzar ( int avanzar );
  void set_pagar ( int pagar );
  void set_sgtEstacion ( bool sgtEstacion );
  void set_tarjetaCarcel ( bool tarjetaCarcel );
  void set_pagarCasa ( int pagarCasa );
  void set_pagarHotel ( int pagarHotel );
  void set_pagarCadaJugador ( int pagarCadaJugador );
  void set_multiplicarPago ( int multiplicarPago );
  void set_cobrarSalida ( bool cobrarSalida );
  void set_multiPagoDado ( int multiPagoDado );
  void set_sgtPropiedad ( bool sgtPropiedad );

  //Mtodos

  /**
   * Mtodo que ejecuta una instancia de la clase accin y hace que los efectos recaigan sobre un jugador   
   * Este mtodo comprobar los valores de todos los atributos de la clase, y decidir que accin debe realizar.
   * En primer lugar se comprobar si debe multiplicar el pago por algn valor, y en caso afirmativo modificar
   * el valor necesario de la instancia jugador. A continuacin seguir ejecutando el resto de las acciones. 
   */
  int ejecutar ( jugador *p );
  
};

/**
 * OPERADORES GET
 */
inline bool accion::get_sgtServicio () const
{
  return _sgtServicio;
}

inline bool accion::get_encarcelar () const
{
  return _encarcelar;
}

inline int accion::get_avanzarA () const
{
  return _avanzarA;
}

inline int accion::get_avanzar () const
{
  return _avanzar;
}

inline int accion::get_pagar () const
{
  return _pagar;
}

inline bool accion::get_sgtEstacion () const
{
  return _sgtEstacion;
}

inline bool accion::get_tarjetaCarcel () const
{
  return _tarjetaCarcel;
}

inline int accion::get_pagarCasa () const
{
  return _pagarCasa;
}

inline int accion::get_pagarHotel () const
{
  return _pagarHotel;
}

inline int accion::get_pagarCadaJugador () const
{
  return _pagarCadaJugador;
}

inline int accion::get_multiplicarPago () const
{
  return _multiplicarPago;
}

inline bool accion::get_cobrarSalida () const
{
  return _cobrarSalida;
}

inline int accion::get_multiPagoDado () const
{
  return _multiPagoDado;
}

inline bool accion::get_sgtPropiedad () const
{
  return _sgtPropiedad;
}

/**
 * OPERADORES SET
 */
inline void accion::set_sgtServicio ( bool sgtServicio )
{
  _sgtServicio = sgtServicio;
}

inline void accion::set_encarcelar ( bool encarcelar )
{
  _encarcelar = encarcelar;
}

inline void accion::set_avanzarA ( int avanzarA )
{
  _avanzarA = avanzarA;
}

inline void accion::set_avanzar ( int avanzar )
{
  _avanzar = avanzar;
}

inline void accion::set_pagar ( int pagar )
{
  _pagar = pagar;
}

inline void accion::set_sgtEstacion ( bool sgtEstacion )
{
  _sgtEstacion = sgtEstacion;
}

inline void accion::set_tarjetaCarcel ( bool tarjetaCarcel )
{
  _tarjetaCarcel = tarjetaCarcel;
}

inline void accion::set_pagarCasa ( int pagarCasa )
{
  _pagarCasa = pagarCasa;
}

inline void accion::set_pagarHotel ( int pagarHotel )
{
  _pagarHotel = pagarHotel;
}

inline void accion::set_pagarCadaJugador ( int pagarCadaJugador )
{
  _pagarCadaJugador = pagarCadaJugador;
}

inline void accion::set_multiplicarPago ( int multiplicarPago )
{
  _multiplicarPago = multiplicarPago;
}

inline void accion::set_cobrarSalida ( bool cobrarSalida )
{
  _cobrarSalida = cobrarSalida;
}

inline void accion::set_sgtPropiedad ( bool sgtPropiedad )
{
  _sgtPropiedad = sgtPropiedad;
}  

inline void accion::set_multiPagoDado ( int multiPagoDado )
{
  _multiPagoDado = multiPagoDado;
}

#endif
