/**
 *  CLASE BANCA
 *  Clase encargada de gestionar los contenidos de la banca de street master's.
 *  La banca es una clase derivada de la clase propietario, por lo que podrá poseer las
 *  diferentes propiedades del juego.
 *  La banca llevará un control sobre las casas y hoteles que se han construido, ya que
 *  existe una limintación en su número. 
 *
 *  Requiere:
 *     - clase propietario ( jerarquia )
 *     - clase string (STL)
 *     - clase list (STL)
 *
 *  Autor: Javier Aragón Zabalegui (PFC para la uc3m: street Master's)
 *  Contacto: 100039831@alumnos.uc3m.es
 *  version 1.0
 */

#ifndef __BANCA__
#define __BANCA__

#include "propietario.h"
#include <list>
#include <string>
using namespace std;

class banca : public propietario {

 private: 
  
  //Atributos
  int _casasDisponibles;
  int _hotelesDisponibles;

 public:   
  //Constructores
  banca ();
  banca ( int casas, int hoteles );
  
  //Copy constructor
  banca (const banca& right);
  
  //sobrecarga de operadores
  banca& banca::operator= (const banca& right);
  friend ostream& operator<< ( ostream &os, banca* b );

  //Destructor
  ~banca ();
  
  //Mtodos GET
  int get_casasDisponibles () const;
  int get_hotelesDisponibles () const;
  
  //Mtodos SET
  void set_casasDisponibles ( int numCasas );
  void set_hotelesDisponibles ( int numHoteles );
};

//Mtodos GET
inline int banca::get_casasDisponibles () const
{
  return ( _casasDisponibles ); 
}

inline int banca::get_hotelesDisponibles() const
{
  return _hotelesDisponibles;
}

//Mtodos SET
inline void banca::set_hotelesDisponibles ( int numHoteles )
{
  _hotelesDisponibles = numHoteles;
}

inline void banca::set_casasDisponibles ( int numCasas )
{
  _casasDisponibles = numCasas;
}

#endif
