
#include "cadenas.h"


/**********FUNCIONES TILES PARA EL TRATAMIENTO DE CADENAS**************/

/**
 * Duplica una cadena de caracteres (s). Realiza la reserva de memoria 
 */
char * dupstr (char *s)
{
	char *r;

	r = new char[strlen(s) + 1];
	strcpy (r, s);
	return (r);
}

/** Elimina espacios del principio y el final de un STRING (de tipo C). Devuelve un
 *  puntero a la cadena. 
 * IN: Cadena sin limpiar
 * OUT: Cadena sin espacios 
 */
char *stripwhite (char *string)
{
  register char *s, *t;
  
  for (s = string; whitespace (*s); s++);    
  if (*s == 0)
    return (s);
  t = s + strlen (s) - 1;
  while (t > s && whitespace (*t))
    t--;
  *++t = '\0';
  return s;
}

/** Elimina espacios del principio y el final de un STRING. Devuelve la cadena
 *  sin espacios
 *  IN: Cadena sin limpiar
 *  OUT: Cadena sin espacios 
 */
string stripwhite_st (string linea)
{
  string new_word;
  /*1 - Eliminar espacios al inicio */
  int i = 0;
  int j;
  while ( ( ( linea[i] == ' ' ) or ( linea[i] == '\t' ) ) && ( i <= linea.size () ) )
    i++;
  /*2 - Eliminar espacios al final */	
  j = linea.size () - 1;
  while ( ( linea[j] == ' ' ) or ( linea[j] == '\t' ) )
    j--;
  
  for ( i; i <= j; i++ )
    new_word += linea[i];	
  return new_word;
}

/**
 * Metodo que busca la siguiente palabra de una linea, tomando como delimitadores
 * el caracter pasado como delimintador  
 */
char *buscar_palabra ( char *linea, char separador, int numero )
{
  char *aux;
  int i = 0, k = 0;
  aux = dupstr(linea);
  linea = stripwhite ( linea );
  
  while ( k++ < numero )
    {
      while ( linea[i] != separador )
	i++;
      
      i++;
    }
  
  int j = i;
  while ( ( linea[j] != separador ) && ( j <= strlen (linea) ) ) 
    j++;
  aux[j]='\0';  
  return ( aux + i );
}

/**
 * Dentro de un string buscar el substring que est comprendido entre 'pos' y el 'delimitador' 
 */
string obtener_substring_delimitado_st ( string linea, string delimitador, string::size_type pos, string::size_type &pos_despues )
{
  string saux;
  pos_despues = linea.find ( delimitador, pos );
  if ( pos == string::npos ) 
    {
      cerr << "Error de formato. Revise la siguiente linea:" << endl << linea << endl;
      return "@*{}";
    }	
  else
    saux = linea.substr ( pos, (pos_despues++ - pos) );
  return saux;
}

/** 
 * Dentro de un string buscar el substring que est comprendido entre 'pos' y el 'delimitador' 
 * El resultado debe ser un valor de tipo entero 
 */
int obtener_substring_delimitado_int ( string linea, string delimitador, string::size_type pos, string::size_type &pos_despues )
{	
  string saux = obtener_substring_delimitado_st ( linea, delimitador , pos, pos_despues);
  return ( atoi ( saux.c_str () ) );
}

/** 
 * Dentro de un string buscar el substring que est comprendido entre 'pos' y el 'delimitador' 
 * El resultado debe ser un valor de tipo entero largo (long) 
 */
int obtener_substring_delimitado_long ( string linea, string delimitador, string::size_type pos, string::size_type &pos_despues )
{	
  string saux = obtener_substring_delimitado_st ( linea, delimitador , pos, pos_despues);
  return ( atol ( saux.c_str () ) );
}

/** 
 * Busca y devuelve palabras completas dentro de un string, delimitado por dos valores
 *  nmericos, que le indican cual es el inicio donde debe empezar a buscar, y la longitud
 *  mxima del total de las palabras.
 */
string buscarPalabraCompleta ( string palabra, int inicio, int max )
{
  int i = max;
  
  if ( palabra.size() > max )
    {
      while ( ( palabra[i] != ' ' ) && ( inicio + i >= inicio ) )
	i--;
      if ( i > inicio )
	return palabra.substr( inicio, i - inicio );    
      else
	return palabra.substr ( inicio, inicio + i);
    }
  else
    return palabra.substr( inicio, inicio + palabra.size() );

}

/** 
 * Dada una cadenas de caracteres y una longitud mxima, se realizar una disposicin de la
 * cadena de caracteres de manera que quede centrada dentro de ese tamao. Tambin se debe 
 * indicar el mximo de lneas en las que se puede distribuir la cadena en caso de que no
 * entre en una, y el separador que se utilizar al inicio y final de la linea (opcional).
 *	-IN:    original:	Cadena original
 *		ancho:		Ancho de la linea en la que se centrar
 *		maxLineas:	Mximo de lineas en las que se puede repartir la palabra.
 *	-OUT:	string:		Nueva cadena
 */
string centrarCadena ( string original, int ancho, int maxLineas, string separador )
{
  string aux;
  ancho = ancho - 2 * separador.size(); //Quito 2 unidades de longitud para los separadores
  /* Si entra en una linea */
  int lineaActual = 1;
  int posInicial = 0;
  while ( lineaActual <= maxLineas )
    {
      aux += separador;
      string linea = buscarPalabraCompleta ( original, posInicial, ancho + posInicial );
      posInicial += linea.size();
      if ( posInicial < original.size() )
	posInicial++;
      if ( linea.size() <= ancho )
	{
	  int numEspaciosLaterales = ( (ancho + 2) - linea.size() ) / 2;
	  int anchoLinea;
	  for ( anchoLinea = 1; anchoLinea < numEspaciosLaterales; anchoLinea++ )
	    aux += " ";
	  aux += linea;
	  anchoLinea += linea.size();
	  for ( int i = anchoLinea; i <= ancho; i++)
	    aux += " ";
	}
      aux += separador + "\n";
      lineaActual++;
    }
  return aux;
}

/**
 * Mtodo que centra una cadena de caracteres en un ancho delimintado. Si no entrar en el
 * ancho indicado, se podrn usar el nmero de lineas que indica maxLineas
 */
string centrarCadena ( string original, int ancho, int maxLineas )
{
  string aux;
  /* Si entra en una linea */
  int lineaActual = 1;
  int posInicial = 0;
  while ( lineaActual <= maxLineas )
    {
      string linea = buscarPalabraCompleta ( original, posInicial, ancho + posInicial );      
      posInicial += linea.size();
      if ( linea.size() <= ancho )
	{
	  int numEspaciosLaterales = ( (ancho + 2) - linea.size() ) / 2;
	  int anchoLinea;
	  for ( anchoLinea = 1; anchoLinea < numEspaciosLaterales; anchoLinea++ )
	    aux += " ";
	  aux += linea;
	  anchoLinea += linea.size();
	  for ( int i = anchoLinea; i <= ancho; i++)
	    aux += " ";
	}
      aux += "\n";
      lineaActual++;
    }
  return aux;
}

/**
 * Mtodo que recoge una cadena de caracteres y la abrevia cogiendo los n primeros caracteres
 * o los n caracteres que siguen a un espacio.
 *	Ejemplos:
 *		Azul Marino --2carac--> AM
 *		Azul ---------2carac--> Az
 */
string abreviar ( string original, int max )
{
  string aux;
  int numCaracteres = 1;
  int ultimoEncontrado = 0;
  aux = original[0];
  
  //Aadir tantos caracteres como indique max (sin pasarse de lo permitido)
  while ( ( numCaracteres < max ) && ( original.size() - ultimoEncontrado > 0) )
    {
      string::size_type pos = 0;
      pos = original.find (" ",0);
      //Si se encuentra algn espacio
      if ( pos != string::npos )
	{
	  if ( original[ pos ] == ' ' )
	    ++pos;
	  aux += original[ pos ];
	  ultimoEncontrado = pos;
	}
      //Si no hay espacio, sigo aadiendo desde el ltimo.
      else
	{	
	  aux += original[ 1 ];
	  ultimoEncontrado++;
	}
      //Volver entrar con el resto de la cadena
      original = original.substr ( ultimoEncontrado, original.size() - ultimoEncontrado );
      ultimoEncontrado = 0;
      numCaracteres ++;
    }

  //Aado espacios al principio hasta completar el tamao pedido en MAX
  string spaces = "";
  for ( int i = 0; i + aux.size() < max; i++ )
    spaces += " ";
  spaces += aux;
  aux = spaces;

  //Devuelve el valor
  return ( aux );
}
