

#include "calle.h"

/**
 *  Constructor por defecto
 */
calle::calle ()
  : propiedad (),
    _alquilerBase ( 0 ),
    _alquilerUnaCasa ( 0 ),
    _alquilerDosCasas ( 0 ),
    _alquilerTresCasas ( 0 ),
    _alquilerCuatroCasas ( 0 ),
    _alquilerHotel ( 0 ),
    _permisoConstruccion ( false ),
    _precioCasa ( 0 ),
    _precioHotel ( 0 ),
    _numCasas ( 0 ),
    _numHoteles ( 0 ),
    _banca ( NULL )
{
  // No se hace nada!
}

/** 
 * Constructor explcito
 */
calle::calle ( int numero, string nombre, string nombreCorto, long precio, long hipoteca, 
	       bool hipotecado, bool hipotecable, long alquilerBase, long alquilerUnaCasa, 
	       long alquilerDosCasas, long alquilerTresCasas, long alquilerCuatroCasas, 
	       long alquilerHotel, bool permisoConstruccion, int precioCasa, int precioHotel,
	       int numCasas, int numHoteles, banca *b, propietario *p )
  : propiedad ( numero, nombre, nombreCorto, precio, hipoteca, hipotecado, hipotecable, p ),    
    _alquilerBase ( alquilerBase ),
    _alquilerUnaCasa ( alquilerUnaCasa ),
    _alquilerDosCasas ( alquilerDosCasas ),
    _alquilerTresCasas ( alquilerTresCasas ),
    _alquilerCuatroCasas ( alquilerCuatroCasas ),
    _alquilerHotel ( alquilerHotel ),
    _permisoConstruccion ( permisoConstruccion ),
    _precioCasa ( precioCasa ),
    _precioHotel ( precioHotel ),
    _numCasas ( numCasas ),
    _numHoteles ( numHoteles ),
    _banca ( b )
{
  // No se hace nada!
}
 
/**
 * Copy constructor
 */
calle::calle (const calle& right)
{
  propiedad calle(right);			
  _alquilerBase		= right.get_alquilerBase ();
  _alquilerUnaCasa	= right.get_alquilerUnaCasa ();
  _alquilerDosCasas	= right.get_alquilerDosCasas ();
  _alquilerTresCasas	= right.get_alquilerTresCasas ();
  _alquilerCuatroCasas	= right.get_alquilerCuatroCasas ();
  _alquilerHotel	= right.get_alquilerHotel ();
  _permisoConstruccion	= right.get_permisoConstruccion ();  
  _perteneceMonopolio	= right.get_perteneceMonopolio ();
  _precioCasa		= right.get_precioCasa ();
  _precioHotel		= right.get_precioHotel ();
  _numCasas             = right.get_numCasas ();
  _numHoteles           = right.get_numHoteles ();
  _banca                = right.get_banca ();
}

/**
 * Opreador de asignacin
 */
calle& calle::operator= (const calle& right)
{  
  propiedad::operator= ( right );
  _alquilerBase		= right.get_alquilerBase ();
  _alquilerUnaCasa	= right.get_alquilerUnaCasa ();
  _alquilerDosCasas	= right.get_alquilerDosCasas ();
  _alquilerTresCasas	= right.get_alquilerTresCasas ();
  _alquilerCuatroCasas	= right.get_alquilerCuatroCasas ();
  _alquilerHotel	= right.get_alquilerHotel ();
  _permisoConstruccion	= right.get_permisoConstruccion ();
  _perteneceMonopolio	= right.get_perteneceMonopolio ();
  _precioCasa		= right.get_precioCasa ();
  _precioHotel		= right.get_precioHotel ();
  _numCasas             = right.get_numCasas ();
  _numHoteles           = right.get_numHoteles ();
  _banca                = right.get_banca ();
  return ( *this );
}

/**
 * Operador menor
 */
bool calle::operator< ( calle compara )
{
  if ( get_numero() < compara.get_numero() )
    return true;
  else
    return false;
}

/**
 * Operador igual
 */
bool calle::operator== ( calle compara )
{
  if ( get_numero() == compara.get_numero() )
    return true;
  else
    return false;
}

/**
 * Operador distinto
 */
bool calle::operator!= ( calle compara )
{
  if ( get_numero() != compara.get_numero() )
    return true;
  else
    return false;
}

/**
 * Destructor
 */
calle::~calle ()
{
  // No se hace nada!
}

/**
 *  POLIMORFISMOS
 */


/**
 * Muestra el color de su grupo de propiedades
 */
string calle::ver_monopolio()
{ 
  cout << "Color: " <<  _perteneceMonopolio->get_color();
  cout << "numProp: " << _perteneceMonopolio->get_numPropiedades();
  return ( _perteneceMonopolio->get_color() );
}

/**
 * Mtodo que devuelve en una cadena de caracteres toda la informacin necesaria, con un
 * formato preparado para mostrarse por pantalle, toda la informacin de una calle, que 
 * incluye:
 *    -Nombre
 *    -Estado (hipotecado, construido, libre ...)
 *    -Casas y hoteles construidos (si procede)
 *    -Propietario
 *    -Nmero de casilla
 *    -Informacin sobre los precios de alquiler
 *    -Precio de compra e hipoteca
 * Representa la tarjeta de propiedad de una calle.
 */
string calle::mostrar ()
{  
  const int ancho = 42;
  string aux =  " __________________________________________\n";
  aux += "|                                          |\n";
  aux += "|                                          |\n";
  aux += centrarCadena ( get_nombre(), ancho + 2, 2, "|" );

  // Muestra el propietario si no se trata de la banca. Debe ser un jugador.
  if ( jugador *j = dynamic_cast <jugador *> ( get_esPropiedad() ) )
    {  
      if ( j != NULL )
	{
	  aux += "|                                          |";
	  if ( get_hipotecado() )
	    {
	      aux += "             PROPIETARIO: ";
	      aux += j->get_nombre() + " (" + j->get_color() + ")    HIPOTECADO\n";
	    }
	  else
	    {
	      aux += "             PROPIETARIO: ";
	      aux += j->get_nombre() + " (" + j->get_color() + ")\n";
	    }
	}
      else
	aux += "|                                          |\n"; 
    }
  else
    aux += "|                                          |\n";

  // Numero de hoteles construidos (maximo 1). Se indica con la palabra 'Hotel'
  if ( get_numHoteles() > 0 )
    {
      aux += "|                                          |"; 
      aux += "             EDIFICACIONES: Hotel\n";
    }
  // Numero de casas construidas
  else if (get_numCasas() > 0 )
    {
      char numCasas[10];
      sprintf ( numCasas, "%d", get_numCasas () );      
      aux += "|                                          |";
      aux += "             EDIFICACIONES: ";
      aux += string (numCasas) + " casa(s) \n";
    }
  else
    aux += "|                                          |\n";

  // Alquileres
  char precioAlpha[30];
  sprintf ( precioAlpha, "%d", get_alquilerBase () );
  string aux2 = "Alquiler base = " + string ( precioAlpha );
  aux += centrarCadena ( aux2, ancho + 2, 1, "|" );
  
  sprintf ( precioAlpha, "%d", get_alquilerUnaCasa () );
  aux2 = "Alquiler 1 casa = " + string ( precioAlpha );
  aux += centrarCadena ( aux2, ancho + 2, 1, "|" );

  sprintf ( precioAlpha, "%d", get_alquilerDosCasas () );
  aux2 = "Alquiler 2 casas = " + string ( precioAlpha );
  aux += centrarCadena ( aux2, ancho + 2, 1, "|" );

  sprintf ( precioAlpha, "%d", get_alquilerTresCasas () );
  aux2 = "Alquiler 3 casas = " + string ( precioAlpha );
  aux += centrarCadena ( aux2, ancho + 2, 1, "|" );

  sprintf ( precioAlpha, "%d", get_alquilerCuatroCasas () );
  aux2 = "Alquiler 4 casas = " + string ( precioAlpha );
  aux += centrarCadena ( aux2, ancho + 2, 1, "|" );

  sprintf ( precioAlpha, "%d", get_alquilerHotel () );
  aux2 = "Alquiler hotel = " + string ( precioAlpha );
  aux += centrarCadena ( aux2, ancho + 2, 1, "|" );
 
  // Precio de la calle
  sprintf ( precioAlpha, "%d", get_precio () );
  aux2 = "Precio = " + string ( precioAlpha );
  aux += centrarCadena ( aux2, ancho + 2, 1, "|" );
 
  // Hipoteca
  sprintf ( precioAlpha, "%d", get_hipoteca () );
  aux2 = "Hipoteca = " + string ( precioAlpha );
  aux += centrarCadena ( aux2, ancho + 2, 1, "|" );

  // Precio de construir una casa
  sprintf ( precioAlpha, "%d", get_precioCasa () );
  aux2 = "Precio por casa = " + string ( precioAlpha );
  aux += centrarCadena ( aux2, ancho + 2, 1, "|" );
  
  // Precio de construir un hotel
  sprintf ( precioAlpha, "%d", get_precioHotel () );
  aux2 = "Precio por hotel = " + string ( precioAlpha );
  aux += centrarCadena ( aux2, ancho + 2, 1, "|" );
 
  aux += "|                                          |\n";
  aux += "|                                          |\n";

  // Numero de casilla
  char numAlpha[10];
  sprintf ( numAlpha, "%d", get_numero() );
  if ( get_numero() >= 10 )
    aux += "|                                       " + string ( numAlpha )  + " |\n";
  else
    aux += "|                                        " + string ( numAlpha )  + " |\n";

  // Monopolio
  aux += centrarCadena ( verColor(), ancho + 2, 1, "|" );
  aux += "|__________________________________________|\n";

  return aux;
}

/**
 * Mtodo que dibuja la fila indicada de la casilla correspondiente (en las filas de arriba y abajo del tablero).
 * Se indicar la fila que se tiene que dibujar y el ancho con el que se tiene que pintar. Como resultado se
 * devolver una cadena de caracteres.
 */
string calle::dibujarCasillaHorizontal ( int fila, int anchoCasilla )
{
  string sFila;  
  const int MaxCasillasAncho = 13;
  
  switch ( fila )
    {
    // Dibuja el inicio del rectangulo de la casilla
    case 0:
      sFila = " ";				//Primer caracter
      for ( int i = 1; i < anchoCasilla ; i++ )
	sFila += "_";
      break;
      
    // Muestra el numero de casilla y el monopolio al que pertenece.
    case 1:	
      {
	// Numero de casilla
	char numAlpha[10];
	sprintf( numAlpha, "%d", get_numero() );
	sFila = string ( numAlpha );       
	while ( sFila.size() <= ( anchoCasilla - 4 ) )
	  sFila += " ";	
	//Monopolio al que pertence
	monopolio *mp = get_perteneceMonopolio();
	string aux = mp->get_colorAbreviado ();
	sFila += aux + "|";	
	break;
      }

    // Muestra el nombre corto de la calle, las edificaciones
    case 2:
      // Nombre corto
      sFila = get_nombreCorto();
      
      // Edificios
      if ( get_numHoteles() != 0 )
	sFila += "(H)";
      else if ( get_numCasas() != 0 )
	{
	  char numCasasAlpha[10];
	  sprintf ( numCasasAlpha, "%d", get_numCasas() );
	  sFila += "(" + string (numCasasAlpha) + ")";
	}
      else if ( banca *b = dynamic_cast <banca *> ( get_esPropiedad() ) )	
	sFila +="(L)";
      else
	sFila +="(C)";     

      while ( sFila.size() <= ( anchoCasilla - 2 ) )
	sFila += " ";      
      sFila += "|";
      break;

    // Muestra los tres primeros jugadores
    case 3:
      {
	int i = 0;
	string aux = "";
	int numJugadores = estaJugador.size ();
	// Recorre los jugadores hasta el tercero que esta en la casilla
	while ( ( numJugadores > 0 ) && ( i < numJugadores ) && ( i < 3 ) )
	  {
	    string color = estaJugador[i]->get_color();
	    string colorAbreviado = color.substr(0,2);
	    // Convierte a mayusculas el color
	    transform (colorAbreviado.begin(), colorAbreviado.end(), colorAbreviado.begin(), (int(*)(int))toupper);
	    aux += colorAbreviado;	    
	    i++;
	    if ( i < 3 ) 
	      aux += " ";
	  }
	while (aux.size() < anchoCasilla - 1 )
	  aux += " ";
	sFila += aux + '|';
	break;
      }

    // Muestra los ultimos jugadores en la casilla (los que hay por encima de tres)
    case 4:      
      {
	int i = 3;
	string aux = "";	
	int numJugadores = estaJugador.size ();
	// Recorre la lista de jugadores empezando en el tercero.
	while ( ( numJugadores > 3 ) && ( i < numJugadores ) && ( i < 6 ) )
	  {
	    string color = estaJugador[i]->get_color();
	    string colorAbreviado = color.substr(0,2);
	    // Convertir a mayusculas
	    transform (colorAbreviado.begin(), colorAbreviado.end(), colorAbreviado.begin(), (int(*)(int))toupper);
	    aux += colorAbreviado;	    
	    i++;
	    if ( i < 6 ) 
	      aux += " ";	
	  }	
	while (aux.size() < anchoCasilla - 1 )
	  aux += " ";
	sFila += aux + '|';
	break;
      }

    case 5:
      while ( sFila.size() <= ( anchoCasilla - 2 ) )
	sFila += "_";	 	
      sFila += "|";
      break;    
    }
  
  return sFila;
}


/**
 * Mtodo que dibuja la fila indicada de la casilla correspondiente (en las columnas derecha e izquierda del tablero).
 * Se indicar la fila que se tiene que dibujar y el ancho con el que se tiene que pintar. Como resultado se
 * devolver una cadena de caracteres.
 */
string calle::dibujarCasillaVertical ( int fila, int anchoCasilla )
{
  string sFila;  
  int const MaxCasillasAncho = 13;
  int const MaxCasillasAlto = 8;
  
  sFila = "|";
  switch ( fila )
    {  
  
    // Dibuja el numero de casilla, el monopolio al que pertenece y los tres primeros jugadores
    case 0:
      {	
	string aux = "";

	// Numero de casilla
	char numAlpha[30];
	sprintf( numAlpha, "%d", get_numero() );
	aux += string ( numAlpha );
	
	// Monopolio
	monopolio *mp = get_perteneceMonopolio();
	string monoP = mp->get_colorAbreviado ();	
	aux += " " + monoP;
	if ( get_numero() < 10 )
	  aux +=  "  ";
	else 
	  aux += " ";
	int i = 0;
	int numJugadores = estaJugador.size ();
	// Recorre los jugadores hasta el tercero o el ultimo que haya en la casilla.
	while ( ( numJugadores > 0 ) && ( i < numJugadores ) && ( i < 3 ) )
	  {
	    string color = estaJugador[i]->get_color();
	    string colorAbreviado = color.substr(0,2);
	    // Convertir a mayusculas
	    transform (colorAbreviado.begin(), colorAbreviado.end(), colorAbreviado.begin(), (int(*)(int))toupper);
	    aux += colorAbreviado;	        
	    i++;
	    if ( i < 3 ) 
	      aux += " ";	
	  }
	while (aux.size() < anchoCasilla - 1 )
	  aux += " ";
	sFila += aux + '|';      
	break;
      }
    
    // Muestra el nombre corto, las edificaciones y los jugadores a partir del tercero
    case 1:
      {
	string aux = "";
	// Nombre corto
	aux += get_nombreCorto();

	// Edificaciones
	if ( get_numHoteles() != 0 )
	  aux += "(H)";
	else if ( get_numCasas() != 0 )
	  {
	    char numCasasAlpha[10];
	    sprintf ( numCasasAlpha, "%d", get_numCasas() );
	    aux += "(" + string (numCasasAlpha) + ")";
	  }
	// Si no esta edificado se indica si esta Comprado o Libre
	else if ( banca *b = dynamic_cast <banca *> ( get_esPropiedad() ) )
	  //else if ( get_esPropiedad() == NULL )
	  aux +="(L)";
	else
	  aux +="(C)";

	while ( aux.size() < 6 ) aux += " ";
	int i = 3;
	int numJugadores = estaJugador.size ();

	//Recorre los jugadores a partir del tercero
	while ( ( numJugadores > 3 ) && ( i < numJugadores ) && ( i < 6 ) )
	  {
	    string color = estaJugador[i]->get_color();
	    aux += color.substr(0,2);
	    i++;
	    if ( i < 6 ) 
	      aux += " ";	
	  }	
	while (aux.size() < anchoCasilla - 1 )
	  aux += " ";
	sFila += aux + '|';
	break;
      }
      
    case 2:      
      while ( sFila.size() <= ( anchoCasilla - 1) )
	sFila += "_";      
      sFila += "|";
      break;  
      
      return sFila;
    }
}

/**
 * Mtodo que devuelve una cadena de caracteres que representa como es la casilla en su 
 * imgen en el tablero, indicando aquellos datos de inters para un jugador que caiga 
 * sobre ella:
 *  -Precio
 *  -Nombre
 *  -Monopolio
 */
string calle::dibujarCasillaEnTablero ()
{
  const int anchoDibujado = 27;
  monopolio* mp = get_perteneceMonopolio ();
  string aux;
  aux = " ___________________________\n";
  aux += centrarCadena ( mp -> get_color(), anchoDibujado + 2, 1, "|" );  
  aux +="|___________________________|\n";
  aux +="|                           |\n";
  
  // Linea del nombre
  string linea = "|";
  string nombre = get_nombre ();
  aux += centrarCadena ( nombre, anchoDibujado + 2, 2, "|" );

  // Informacin sobre la edificacin en la calle
  if ( get_numHoteles() == 1 )
    aux +="|        (1 hotel)          |\n";
  else if ( get_numCasas() == 1 )
    aux +="|         (1 casa)          |\n";
  else if ( get_numCasas() > 1 )
    {
      char numCasasAlpha[10];
      sprintf ( numCasasAlpha, "%d", get_numCasas() );
      aux +="|        (" + string ( numCasasAlpha ) + " casas)          |\n";
    }
  else if ( get_permisoConstruccion () )
    aux +="|       grupo completo      |\n";
  else    
    aux +="|                           |\n";
  aux +="|                           |\n";

  // Precio de la propiedad 
  string precio = "Precio ";
  char precioAlpha[30];
  sprintf( precioAlpha, "%d", get_precio() );
  precio += precioAlpha;  
  aux += centrarCadena ( precio, anchoDibujado + 2, 1, "|" );

  // Nmero de casilla 
  int numero = get_numero();
  aux +="|                         ";
  if ( numero < 10 )
    aux += " ";
  char numAlpha[10];
  sprintf( numAlpha, "%d", numero );
  aux += numAlpha;
  aux += "|\n";

  // Base del rectngulo 
  aux += "|___________________________|\n";

  return aux;
}

/**
 * Devuelve la informacin de la casilla en modo texto para mostrar por pantalla la informacin
 * en forma de tabla. 
 */
string calle::leyendaCasilla ( int num, int nombreCorto, int tipo, int nombre, int estado )
{
  string aux;
  
  // Numero de casilla
  int numeroCasilla = get_numero();
  char numAlpha[30];
  sprintf (numAlpha, "%d", numeroCasilla );
  string numero = numAlpha;
  aux = numero.substr ( 0, num );
  for ( int i = numero.size(); i<=num; i++ )
    aux += " ";

  // Nombre corto
  string sNombreCorto = get_nombreCorto();
  aux += sNombreCorto.substr ( 0, nombreCorto );
  for ( int i = sNombreCorto.size (); i<=nombreCorto; i++ )
    aux += " ";

  // Monopolio al que pertenece
  monopolio* mp = get_perteneceMonopolio();  
  string sTipo = mp->get_colorAbreviado();
  aux += sTipo.substr ( 0, tipo );
  for ( int i = sTipo.size (); i<=tipo; i++ )
    aux += " ";
  
  // Nombre completo
  string sNombre = get_nombre();
  aux += sNombre.substr ( 0, nombre );
  for ( int i = sNombre.size (); i<=nombre; i++ )
    aux += " ";

  // Estado actual de la calle ( edificaciones o comprada / libre )
  string sEstado;
  if ( get_numHoteles() != 0 )
    sEstado += "H";
  else if ( get_numCasas() != 0 )
    {
      char numCasasAlpha[10];
      sprintf ( numCasasAlpha, "%d", get_numCasas() );
      sEstado += string (numCasasAlpha) ;
    }
  // Si pertenece a la banca, esta libre, si no, esta comprado (C)
  else if ( banca *b = dynamic_cast <banca *> ( get_esPropiedad() ) )    
    sEstado += "L";
  else
    sEstado += "C";       
  aux += sEstado;

  for ( int i = sEstado.size (); i<=estado; i++ )
    aux += " ";

  return aux;
}

/**
 * Devuelve el importe a pagar en funcin de las construcciones realizadas sobre la calle.
 * Se debe tener en cuenta que si se ha completado el monopolio (permiso de construccion = true )
 * el alquiler base se duplica.
 */
long calle::calcularAlquiler ( jugador *j )
{
  long precio = 0;
  // hotel
  if ( get_numHoteles() == 1 )
    precio = get_alquilerHotel ();
  // resto de posiblidades
  else if ( get_permisoConstruccion () )
    switch ( get_numCasas () )
      {
      // Si no tiene casas pero tiene el monopolio completo, el alquiler se duplica
      case 0:
	precio = 2 * get_alquilerBase ();
	break;
      case 1:
	precio = get_alquilerUnaCasa ();
	break;
      case 2:
	precio = get_alquilerDosCasas ();
	break;
      case 3:
	precio = get_alquilerTresCasas ();
	  break;
      case 4:
	precio = get_alquilerCuatroCasas ();
	break;
      default:
	precio = 0;
      }
  else
    precio = get_alquilerBase();
  return precio;
}

/**
 * Se adjudica la calle al propietario dado como parmetro. Puede ser un jugador o la banca.
 * En ambos casos es necesario comprobar si se completa el monopolio para cambiar el estado
 * de la calle, otorgando o denegando el permiso de construccin segn el caso.
 * (Recuerda: El permiso de construccin se otorga al completar un monopolio)
 */
int calle::adjudicarA ( propietario *j )
{
  if ( jugador *jug = dynamic_cast <jugador *> ( j ) )
    {
      monopolio *mp = get_perteneceMonopolio ();
      jug->invertir ( get_precio() );
      jug->set_posee ( this );   
      set_esPropiedad ( jug );
      if ( mp->monopolioCompleto () )     
	mp->permitirConstruir ();
    }
  else
    {
      j->set_posee ( this );   
      set_esPropiedad ( j );
    }
  return 0;
}

/**
 * Devuelve la informacin rpida sobre una calle en una sla lnea.
 * til sobre todo para hacer listados de propiedades o casillas.
 */
string calle::infoString ()
{
  string aux = "";

  // Numero de casilla
  char numAlpha[30];
  sprintf ( numAlpha, "%d", get_numero() );

  // Nombre de la calle
  aux += "(" + string ( numAlpha ) + ") " + get_nombre();  
  for ( int i = aux.size() ; i <= 35; i++ )
    aux += " ";
  
  // Monopolio
  aux += "GRUPO: " + verColor();
  for ( int i = aux.size() ; i <= 55; i++ )
    aux += " ";
  
  // Construcciones
  aux += "CONSTRUIDO: ";
  if ( get_numHoteles() > 0 )
    {
      aux += "Hotel ";
    }  
  else if ( get_numCasas() > 0 )
    {
      char numAlpha[10];
      sprintf ( numAlpha, "%d", get_numCasas() );
      aux += string ( numAlpha ) + " casas";
    }
  else if ( get_permisoConstruccion() )
    aux += "(Sin construir)";

  // Hipotecada (S/N)
  else if ( get_hipotecado() )
    aux += "(HIPOTECADA)";
  else
    aux += "(No tiene permiso para construir)";
    
  return ( aux );
  
}

/**
 * Polimorfismo de la clase propiedad. Este mtodo es muy importante, ya que a diferencia 
 * del caso de las propiedades que no se construyen, en este caso hay que comprobar que se
 * pueda hipotecar verificando que no existan construcciones en su grupo de color.
 * Adems al hipotecar una calle, se debe denegar este permiso del resto de las calles del
 * monopolio.
 */
int calle::hipotecar ()
{
  int ok = 0;
  if ( jugador *j = dynamic_cast <jugador *> ( get_esPropiedad() ) )
    {
      if ( get_hipotecable() )
	{
	  ok = 0;
	  //Indico que no se puede hipotecar otra vez hasta que se deshipoteque
	  set_hipotecable ( false );
	  set_hipotecado ( true );
	  //Se paga al jugador la cantidad estipulada
	  int cantidadHipoteca = get_hipoteca ();	  
	  j->pagar ( 0 - cantidadHipoteca );

	  //Se deniegan los permisos de construccion de todo el monopolio si los tenian
	  if ( get_permisoConstruccion () )
	    {
	      monopolio *mp = get_perteneceMonopolio();
	      mp->prohibirConstruir ();
	    }
	}
      else
	{
	  ok = -1;
	}
    }
  else
    ok = -1;
  return ok;
}

/**
 * Proceso inverso al explicado en el mtodo anterior. Se debe volver a dar un permiso de
 * construccion al deshipotecar una calle que forme un monopolio completo.
 */
int calle::deshipotecar ()
{
  int ok = 0;
  if ( jugador *j = dynamic_cast <jugador *> ( get_esPropiedad() ) )
    { 
      long cantidadHipoteca = get_deshipoteca ();  
      if ( get_hipotecado() )
	if ( j->get_dinero_total() >= cantidadHipoteca )
	  {
	    ok = 0;
	    //Indico que se podr volver a hipotecar y que no est hipotecado
	    set_hipotecable ( true );
	    set_hipotecado ( false );
	    //Se cobra al jugador la cantidad estipulada (hipoteca + 10% de inters)
	    j->pagar ( cantidadHipoteca );
	    monopolio *mp = get_perteneceMonopolio ();
	    // Si se completa el monopolio, se otorgan permisos de construccion
	    if ( mp->monopolioCompleto () )
	      mp->permitirConstruir ();
	  }
	else
	  ok = -2;
      else
	{
	  ok = -1;
	}
    }
  else 
    ok = -1;
  return ok;
}

/**
 * Comprueba que una calle se pueda ofertar en una negociacin. Si esta construida no se podr
 * ofrecer.
 */
bool calle::esOfertable ()
{
  monopolio *mp = get_perteneceMonopolio ();
  if ( mp->maxConstruido() > 0 )
    return false;
  return true;
}

/**
 * Devuelve una calle a su estado inicial. Se supone que no tiene ni casas ni hoteles.
 * Se le asignar como propietario el que se indique como parmetro
 */
void calle::liberar ( propietario *p )
{
  set_hipotecado ( false );
  set_hipotecable ( true );
  set_permisoConstruccion ( false );
  _esPropiedad = p;
  return;
}

/**
 * Devuelve los datos necesarios para salvar la calle en una partida guardada
 */
string calle::guardar ()
{
  // ETIQUETA
  string aux = "PROPIEDAD\n";
  char numAlpha[30];
  
  // Numero
  sprintf ( numAlpha, "%d", get_numero() );
  aux += "NUMERO= " + string ( numAlpha );
  aux += "\n";
  
  //Estado hipotecado
  sprintf ( numAlpha, "%d", get_hipotecado() );
  aux += "HIPOTECADO = " + string ( numAlpha );
  aux += "\n";
  sprintf ( numAlpha, "%d", get_hipotecable () );
  aux += "HIPOTECABLE= " + string ( numAlpha );
  aux += "\n";

  //Permiso de construccion
  sprintf ( numAlpha, "%d", get_permisoConstruccion () );
  aux += "PERMISO CONSTRUCCION= " + string ( numAlpha );
  aux += "\n";

  // Edificaciones
  sprintf ( numAlpha, "%d", get_numCasas () );
  aux += "CASAS= " + string ( numAlpha );
  aux += "\n";
  sprintf ( numAlpha, "%d", get_numHoteles () );
  aux += "HOTELES= " + string ( numAlpha );
  aux += ";\n";
  return aux; 
}

/**
 * Comprueba que una calle es construible. Para ello se debe cumplir que no haya llegado
 * al lmite de las edificaciones permitidas ( 1 hotel ), no incumpla la norma de 
 * construcciones escalonadas ( mxima diferencia entre la propiedad ms construida y la
 * menos construida debe ser 1 ) y adems tenga el permiso de construccin (true).
 */
bool calle::esConstruible ()
{
  bool construir;
  monopolio *mp = get_perteneceMonopolio ();

  // Si no tiene permiso de construccion, o esta hipotecado devuelve false
  if ( ! get_permisoConstruccion () )
    construir = false;
  else if ( get_hipotecado () )
    construir = false;

  // Si la diferencia entre el maximo construido menos el minimo construido es 0, se puede construir
  else if ( mp->maxConstruido() - mp->minConstruido() == 0 )
    construir = true;

  // Si esta no es la calle mas construida tambien se permite (construccion escalonada)
  else if ( !mp->esCalleMasConstruida ( this ) )
    construir = true;
  else
    construir = false;
  return construir;   
}

/**
 * Una calle puede vender edificios siempre y cuando no incumpla la regla de construcciones
 * escalonadas (ver mtodo construir)
 */
bool calle::puedeVenderEdificio ()
{
  bool construir;
  monopolio *mp = get_perteneceMonopolio (); 

  // Puede vender si todas las calles estan construidas al mismo nivel
  if ( mp->maxConstruido() - mp->minConstruido() == 0 )
    construir = true;

  // Puede vender si esta calle es la mas construida
  else if ( mp->esCalleMasConstruida ( this ) )
    construir = true;
  else
    construir = false;
  return construir;   
}

/**
 * Aade una casa u hotel a la calle. Debe ser construible para poder hacerlo (ver mtodo esConstruible) y
 * la banca debe tener edificios para construir. Si no se cumple no se construir.
 */
int calle::construir ()
{
  int ok;
  // Si el propietario es un jugador y la calle es construible se procede a construir
  if ( jugador *j = dynamic_cast <jugador *> ( get_esPropiedad() ) )
    {      
      banca *b = get_banca();
      if ( esConstruible () )
	{
	  // Si hay menos de 4 casas se construye otra casa
	  if ( ( get_numCasas() < 4 ) && ( get_numHoteles() == 0 ) )
	    {
	      if ( j->get_dinero_total () >= get_precioCasa() )
		// Si quedan casas en la banca
		if ( b->get_casasDisponibles() > 0 )
		  {
		    // Se aade una casa a la calle
		    int casas = get_numCasas();	  
		    set_numCasas ( casas + 1 );
		    
		    // Se decrementa las casas disponibles en la banca
		    int casasDisponibles = b->get_casasDisponibles ();
		    b->set_casasDisponibles ( casasDisponibles - 1 );

		    // Se descuenta al jugador la cantidad (se aade al capital)
		    j->invertir ( get_precioCasa() );
		    monopolio *mp = get_perteneceMonopolio();

		    // Se marca como no hipotecable porque tiene una casa construida
		    mp->hipotecable ( false );
		    cout << "Enhorabuena, su nueva casa ha costado " << get_precioCasa() << endl;
		    ok = 0;
		  }
		else
		  { 
		    cout << "No quedan casas en la banca" << endl;
		    ok = -1;
		  }
	      else
		{
		  cout << "No dispone de dinero para construir una casa" << endl;
		  ok = -1;	      
		}
	    }
	  else 
	    // Si ya hay 4 casas y ningun hotel se construye un hotel
	    if ( get_numHoteles() == 0 )
	      {
		if ( j->get_dinero_total () >= get_precioHotel() )
		  // Si hay hoteles disponibles en la banca
		  if ( b->get_hotelesDisponibles() > 0 )
		    {
		      // Se construye el hotel
		      set_numHoteles ( 1 );
		      
		      // Se decrementan los disponibles en la banca
		      int hotelesDisponibles = b->get_hotelesDisponibles ();
		      b->set_hotelesDisponibles ( hotelesDisponibles - 1 );
		      
		      //Se devuelven las casas a la banca
		      int casasDisponibles = b->get_casasDisponibles ();
		      b->set_casasDisponibles ( casasDisponibles + 4 );
		      set_numCasas ( 0 );
		      
		      //Se indica que no se puede hipotecar ninguna propiedad del monopolio
		      monopolio *mp = get_perteneceMonopolio();
		      mp->hipotecable ( false );
		      
		      // El jugador paga
		      j->invertir ( get_precioHotel() );
		      cout << "Enhorabuena, su nuevo hotel ha costado " << get_precioHotel() << endl;
		      ok = 0;
		    }
		  else 
		    {
		      cout << "No quedan hoteles en la banca" << endl;
		      ok = -1;
		    }
		else
		  {
		    cout << "No dispone de dinero para construir un hotel" << endl;
		    ok = -1;
		  }
	      }
	    else ok = -1;
	}
      else ok = -1;
    }
  else
    ok = -1;
  return ok;
}

/**
 * Mtodo que construye sin realizar comprobaciones de ningun tipo, Lo que se indique deber ser correcto
 */
int calle::construirSinComprobar ()
{
  int ok;
  if ( jugador *j = dynamic_cast <jugador *> ( get_esPropiedad() ) )
    {      
      banca *b = get_banca();
      //Construye casas si no se ha llegado al mximo de 4 casas
      if ( ( get_numCasas() < 4 ) && ( get_numHoteles() == 0 ) )
	{	 
	  if ( b->get_casasDisponibles() > 0 )
	    {
	      int casas = get_numCasas();	  
	      set_numCasas ( casas + 1 );
	      int casasDisponibles = b->get_casasDisponibles ();
	      b->set_casasDisponibles ( casasDisponibles - 1 );
	      j->invertir ( get_precioCasa() );
	      monopolio *mp = get_perteneceMonopolio();
	      mp->hipotecable ( false );
	      ok = 0;
	    }
	  else	    		
	      ok = -1;	     
	}
      else 
	if ( get_numHoteles() == 0 )
	  {
	    if ( b->get_hotelesDisponibles() > 0 )
	      {
		set_numHoteles ( 1 );
		int hotelesDisponibles = b->get_hotelesDisponibles ();
		b->set_hotelesDisponibles ( hotelesDisponibles - 1 );
		int casasDisponibles = b->get_casasDisponibles ();
		b->set_casasDisponibles ( casasDisponibles + 4 );
		set_numCasas ( 0 );
		monopolio *mp = get_perteneceMonopolio();
		mp->hipotecable ( false );
		j->invertir ( get_precioHotel() );
		ok = 0;
	      }
	    else 	
	      ok = -1;	   
	  }
	else ok = -1;
    }
  else 
    ok = -1;
  
  return ok;
}

/**
 * Se comprobar que la calle pueda vender edificios (mtodo puedeVenderEdificio) y si es posible vender
 * una edificacin devolvindolo a la banca. En caso de que se quiera vender un hotel, se debe consultar
 * que la banca tenga 4 casas para construir en su lugar.
 */
int calle::vender ()
{  
  int ok;
  if ( jugador *j = dynamic_cast <jugador *> ( get_esPropiedad() ) )
    {
      banca *b = get_banca();
      if ( puedeVenderEdificio () )
	{
	  // Si hay un hotel construido debe haber 4 casas para restablecer el estado anterior
	  if ( get_numHoteles() > 0 )
	    {
	      // Si hay casas para restablecer el estado de 4 casas
	      if ( b->get_casasDisponibles() >= 4 )
		{
		  // Devolver el hotel
		  set_numHoteles ( 0 );
		  int hotelesDisponibles = b->get_hotelesDisponibles ();
		  b->set_hotelesDisponibles ( hotelesDisponibles + 1 );

		  //Construir las 4 casas
		  int casasDisponibles = b->get_casasDisponibles ();
		  b->set_casasDisponibles ( casasDisponibles - 4 );
		  set_numCasas ( 4 );

		  //Recuperar parte de la inversin de la venta
		  int venta =  get_precioVentaHotel();
		  long dinero = j->recuperarInversion ( venta, get_precioHotel () );
		  cout << "Has vendido un hotel en " << get_nombre() << " por " << venta << endl;
		  cout << "Ahora tienes " << j->get_dinero_total() << endl;
		  ok = 0;
		}
	      else
		{
		  cout << "No quedan casas en la banca para restablecer el hotel" << endl;
		  ok = -1;
		}
	    }
	  // Si solo hay casas construidas se vende y se elimina de la calle
	  else if ( get_numCasas() > 0 )
	    {  
	      //Eliminar la casa de la calle
	      int casas = get_numCasas();	  
	      set_numCasas ( casas - 1 );
	      
	      //Devolver la calle a la banca
	      int casasDisponibles = b->get_casasDisponibles ();
	      b->set_casasDisponibles ( casasDisponibles + 1 );
	      int venta =  get_precioCasa();

	      //Recupera la inversion realizada
	      long dinero = j->recuperarInversion ( venta, get_precioCasa() );
	      monopolio *mp = get_perteneceMonopolio ();
	      
	      //Indicar si las calles son hipotecables (si no quedan casas)
	      if ( ! mp->existeConstruido () )
		mp->hipotecable ( true );
	      cout << "Has vendido una casa en " << get_nombre() << " por " << venta << endl;
	      cout << "Ahora tienes " << j->get_dinero_total() << endl;
	      ok = 0;
	    }
	  
	} 
      else ok = -1;  
    }
  else ok = -1;
  return ok;
}

/**
 * Mtodo que elimina una edificacin sin realizar ninguna comprobacin previa
 */
int calle::venderSinComprobar ()
{
  int ok;
  if ( jugador *j = dynamic_cast <jugador *> ( get_esPropiedad() ) )
    {
      banca *b = get_banca();
      if ( get_numHoteles() > 0 )
	{       
	  set_numHoteles ( 0 );
	  int hotelesDisponibles = b->get_hotelesDisponibles ();
	  b->set_hotelesDisponibles ( hotelesDisponibles + 1 );
	  int casasDisponibles = b->get_casasDisponibles ();
	  b->set_casasDisponibles ( casasDisponibles - 4 );
	  set_numCasas ( 4 );
	  int venta =  get_precioVentaHotel();
	  long dinero = j->recuperarInversion ( venta, get_precioHotel () );      
	  ok = 0;
	}     
      else if ( get_numCasas() > 0 )
	{  
	  int casas = get_numCasas();	  
	  set_numCasas ( casas - 1 );
	  int casasDisponibles = b->get_casasDisponibles ();
	  b->set_casasDisponibles ( casasDisponibles + 1 );
	  int venta =  get_precioCasa();
	  long dinero = j->recuperarInversion ( venta, get_precioCasa() );
	  monopolio *mp = get_perteneceMonopolio ();
	  if ( ! mp->existeConstruido () )
	    mp->hipotecable ( true );      
	  ok = 0;
	}   
    }
  return ok;
}
