/***********************************************************************************
 *  CLASE CALLE
 *   Clase que representa una de las casillas ms comunes del juego
 *   street master's, que hereda de la clase propiedad los atributos y
 *   mtodos, ya que se trata de una de las casillas del juego que 
 *   pueden ser compradas, vendidas y alquiladas por los jugadores, que
 *   es la caracterstica comun de la clase propiedad.
 *   La clase calle adems, tiene los atributos y mtodos necesarios para
 *   representar la construccin de casas y el alquiler de la calle 
 *   cuando otro jugador caiga en ella. 
 *   Al igual que el resto de casillas, tiene sus propios mtodos para
 *   dibujarla en el tablero, ya que las caractersticas de cada una no
 *   son iguales. 
 *
 *  Requiere:
 *     - clase propiedad ( jerarquia )
 *     - clase string (STL)
 *     - clase list (STL)
 *
 *  Autor: Javier Aragn Zabalegui (PFC para la uc3m: street Master's)
 *  Contacto: 100039831@alumnos.uc3m.es
 *  version 1.0
 **********************************************************************************/

#ifndef __CALLE__
#define __CALLE__

#include "cadenas.h"
#include "propiedad.h"
#include <string>
using namespace std;


class calle : public propiedad {

 private:

  long _alquilerBase;                           //Indica el alquiler con 0 casas construidas
  long _alquilerUnaCasa;                        //Indica el alquiler con 1 casa construida
  long _alquilerDosCasas;                       //Indica el alquiler con 2 casas construidas
  long _alquilerTresCasas;                      //Indica el alquiler con 3 casas construidas
  long _alquilerCuatroCasas;                    //Indica el alquiler con 4 casas construidas
  long _alquilerHotel;                          //Indica el alquiler con 1 hotel construido
  bool _permisoConstruccion;                    //Indica si la casa puede ser construida o no. 
                                                //Si no est construida, y es 'true' su alquiler base se duplicar
  int _precioCasa;                              //Indica el precio de construir una casa
  int _precioHotel;                             //Indica el precio de construir un hotel
  int _numCasas;                                //Indica el numero de casas construidas
  int _numHoteles;                              //Indica el nmero de hoteles construidos

  //Asociaciones
  banca *_banca;                                //Banca con la que negocia la compra / venta de casas
 public:

  // Constructores
  calle ();  
  calle ( int numero, string nombre, string nombreCorto, long precio, long hipoteca, 
	  bool hipotecado, bool hipotecable, long alquilerBase, long alquilerUnaCasa, 
	  long alquilerDosCasas, long alquilerTresCasas, long alquilerCuatroCasas, 
	  long alquilerHotel, bool permisoConstruccion, int precioCasa, int precioHotel,
	  int numCasas, int numHoteles, banca *b, propietario *p );
  
  // Copy constructor
  calle (const calle& right);

  // Sobrecarga de operadores  
  calle& operator= (const calle& right);
  bool operator< ( calle compara );
  bool operator!= ( calle compara );
  bool operator== ( calle compara );
  friend ostream& operator<< ( ostream& os, calle *c );

  // Destructor
  ~calle ();

  // Mtodos GET
  long get_alquilerBase () const;
  long get_alquilerUnaCasa () const;
  long get_alquilerDosCasas () const;
  long get_alquilerTresCasas () const;
  long get_alquilerCuatroCasas () const;
  long get_alquilerHotel () const;
  bool get_permisoConstruccion () const; 
  int get_precioCasa () const;
  int get_precioVentaCasa () const;
  int get_precioHotel () const;
  int get_precioVentaHotel () const;
  int get_numCasas () const;
  int get_numHoteles () const;
  banca *get_banca () const;

  // Mtodos SET
  void set_alquilerBase ( long alquilerBase );
  void set_alquilerUnaCasa ( long alquilerUnaCasa );
  void set_alquilerDosCasas ( long alquilerDosCasas );
  void set_alquilerTresCasas ( long alquilerTresCasas );
  void set_alquilerCuatroCasas ( long alquilerCuatroCasas );
  void set_alquilerHotel ( long alquilerHotel );
  void set_permisoConstruccion ( bool permisoConstruccion );
  void set_precioCasa ( int precioCasa );  
  void set_precioHotel ( int precioHotel );
  void set_numCasas ( int numCasas );
  void set_numHoteles ( int numHoteles );
  void set_banca ( banca *b );
 

  // MTODOS

  // Polimorfismos  

  /**
   * Muestra el color de su grupo de propiedades
   */
  string ver_monopolio();

  /**
   * Mtodo que devuelve en una cadena de caracteres toda la informacin necesaria, con un
   * formato preparado para mostrarse por pantalle, toda la informacin de una calle, que 
   * incluye:
   *    -Nombre
   *    -Estado (hipotecado, construido, libre ...)
   *    -Casas y hoteles construidos (si procede)
   *    -Propietario
   *    -Nmero de casilla
   *    -Informacin sobre los precios de alquiler
   *    -Precio de compra e hipoteca
   * Representa la tarjeta de propiedad de una calle.
   */
  string mostrar();

  /**
   * Mtodo que dibuja la fila indicada de la casilla correspondiente (en las filas de arriba y abajo del tablero).
   * Se indicar la fila que se tiene que dibujar y el ancho con el que se tiene que pintar. Como resultado se
   * devolver una cadena de caracteres.
   */
  string dibujarCasillaHorizontal ( int fila, int anchoCasilla );

  
  /**
   * Mtodo que dibuja la fila indicada de la casilla correspondiente (en las columnas derecha e izquierda del tablero).
   * Se indicar la fila que se tiene que dibujar y el ancho con el que se tiene que pintar. Como resultado se
   * devolver una cadena de caracteres.
   */
  string dibujarCasillaVertical ( int fila, int anchoCasilla );

  /**
   * Mtodo que devuelve una cadena de caracteres que representa como es la casilla en su 
   * imgen en el tablero, indicando aquellos datos de inters para un jugador que caiga 
   * sobre ella:
   *  -Precio
   *  -Nombre
   *  -Monopolio
   */
  string dibujarCasillaEnTablero ();

  /**
   * Devuelve la informacin de la casilla en modo texto para mostrar por pantalla la informacin
   * en forma de tabla. 
   */
  string leyendaCasilla ( int num, int nombreCorto, int tipo, int nombre, int estado );  

  /**
   * Devuelve el importe a pagar en funcin de las construcciones realizadas sobre la calle.
   * Se debe tener en cuenta que si se ha completado el monopolio (permiso de construccion = true )
   * el alquiler base se duplica.
   */
  long calcularAlquiler ( jugador* j );

  /**
   * Se adjudica la calle al propietario dado como parmetro. Puede ser un jugador o la banca.
   * En ambos casos es necesario comprobar si se completa el monopolio para cambiar el estado
   * de la calle, otorgando o denegando el permiso de construccin segn el caso.
   * (Recuerda: El permiso de construccin se otorga al completar un monopolio)
   */
  int adjudicarA ( propietario* j );

  /**
   * Devuelve la informacin rpida sobre una calle en una sla lnea.
   * til sobre todo para hacer listados de propiedades o casillas.
   */
  string infoString ();

  /**
   * Polimorfismo de la clase propiedad. Este mtodo es muy importante, ya que a diferencia 
   * del caso de las propiedades que no se construyen, en este caso hay que comprobar que se
   * pueda hipotecar verificando que no existan construcciones en su grupo de color.
   * Adems al hipotecar una calle, se debe denegar este permiso del resto de las calles del
   * monopolio.
   */
  int hipotecar ();

  /**
   * Proceso inverso al explicado en el mtodo anterior. Se debe volver a dar un permiso de
   * construccion al deshipotecar una calle que forme un monopolio completo.
   */
  int deshipotecar ();

  /**
   * Comprueba que una calle se pueda ofertar en una negociacin. Si esta construida no se podr
   * ofrecer.
   */
  bool esOfertable ();

  /**
   * Devuelve una calle a su estado inicial. Se supone que no tiene ni casas ni hoteles.
   * Se le asignar como propietario el que se indique como parmetro
   */
  void liberar ( propietario *p );
  
  /**
   * Devuelve los datos necesarios para salvar la calle en una partida guardada
   */
  string guardar();

  //Otros mtodos:
  
  /**
   * Comprueba que una calle es construible. Para ello se debe cumplir que no haya llegado
   * al lmite de las edificaciones permitidas ( 1 hotel ), no incumpla la norma de 
   * construcciones escalonadas ( mxima diferencia entre la propiedad ms construida y la
   * menos construida debe ser 1 ) y adems tenga el permiso de construccin (true).
   */
  bool esConstruible ();
  
  /**
   * Una calle puede vender edificios siempre y cuando no incumpla la regla de construcciones
   * escalonadas (ver mtodo construir)
   */
  bool puedeVenderEdificio ();

  /**
   * Aade una casa u hotel a la calle. Debe ser construible para poder hacerlo (ver mtodo esConstruible) y
   * la banca debe tener edificios para construir. Si no se cumple no se construir.
   */
  int construir ();

  /**
   * Mtodo que construye sin realizar comprobaciones de ningun tipo, Lo que se indique deber ser correcto
   */
  int construirSinComprobar ();
  
  /**
   * Se comprobar que la calle pueda vender edificios (mtodo puedeVenderEdificio) y si es posible vender
   * una edificacin devolvindolo a la banca. En caso de que se quiera vender un hotel, se debe consultar
   * que la banca tenga 4 casas para construir en su lugar.
   */
  int vender (); 

  /**
   * Mtodo que elimina una edificacin sin realizar ninguna comprobacin previa
   */
  int venderSinComprobar ();

};

inline long calle::get_alquilerBase () const
{
  return _alquilerBase;
}

inline long calle::get_alquilerUnaCasa () const
{
  return _alquilerUnaCasa;
}

inline long calle::get_alquilerDosCasas () const
{
  return _alquilerDosCasas;
}

inline long calle::get_alquilerTresCasas () const
{
  return _alquilerTresCasas;
}

inline long calle::get_alquilerCuatroCasas () const
{
  return _alquilerCuatroCasas;
}

inline long calle::get_alquilerHotel () const
{
  return _alquilerHotel;
}

inline bool calle::get_permisoConstruccion () const
{
  return _permisoConstruccion;
}

inline int calle::get_precioCasa () const
{
  return _precioCasa;
}
 
inline int calle::get_precioVentaCasa () const
{
  return _precioCasa / 2;
}

inline int calle::get_precioHotel () const
{
  return _precioHotel;
}

inline int calle::get_precioVentaHotel () const
{
  return _precioHotel / 2;
}

inline int calle::get_numCasas () const
{
  return _numCasas;
}

inline int calle::get_numHoteles () const
{
  return _numHoteles;
}

inline banca *calle::get_banca () const
{
  return _banca;
}

//Set operators

inline void calle::set_alquilerBase ( long alquilerBase )
{
  _alquilerBase = alquilerBase;
}

inline void calle::set_alquilerUnaCasa ( long alquilerUnaCasa )
{
  _alquilerUnaCasa = alquilerUnaCasa;
}

inline void calle::set_alquilerDosCasas ( long alquilerDosCasas )
{
  _alquilerDosCasas = alquilerDosCasas;
}

inline void calle::set_alquilerTresCasas ( long alquilerTresCasas )
{
  _alquilerTresCasas = alquilerTresCasas;
}

inline void calle::set_alquilerCuatroCasas ( long alquilerCuatroCasas )
{
  _alquilerCuatroCasas = alquilerCuatroCasas;
}

inline void calle::set_alquilerHotel ( long alquilerHotel )
{
  _alquilerHotel = alquilerHotel;
}

inline void calle::set_permisoConstruccion ( bool permisoConstruccion )
{
  _permisoConstruccion = permisoConstruccion;
}

inline void calle::set_precioCasa ( int precioCasa )
{
  _precioCasa = precioCasa;
}

inline void calle::set_precioHotel ( int precioHotel )
{
  _precioHotel = precioHotel;
}

inline void calle::set_numCasas ( int numCasas )
{
  _numCasas = numCasas;
}

inline void calle::set_numHoteles ( int numHoteles )
{
  _numHoteles = numHoteles;
}

inline void calle::set_banca ( banca *b )
{
  _banca = b;
}

#endif
