/***********************************************************************************
 *  CLASE CARCEL
 *  Clase que representa la casilla de la crcel en el juego street master's.
 *  Esta clase debe conocer la informacin necesaria para mantener encarcelado a un
 *  jugador y liberarlo en el momento oportuno.
 *  
 *  Hereda de casilla los atributos nombre y nmero y los mtodos comunes de todas 
 *  las casillas.
 *
 *  Requiere:
 *     - clase casilla ( jerarquia )
 *     - clase string (STL)
 *
 *  Autor: Javier Aragn Zabalegui (PFC para la uc3m: street Master's)
 *  Contacto: 100039831@alumnos.uc3m.es
 *  version 1.0
 **********************************************************************************/

#ifndef __CARCEL__
#define __CARCEL__

#include "casilla.h"

#include <string>
using namespace std;

class carcel : public casilla {

 private:

  // Atributos
  int _maxTurnos;              //Nmero mximo de turnos que un jugador puede estar encarcelado
  long _multa;                 //Multa que puede pagar un jugador para salir de la crcel  

 public:

  // Constructores
  carcel ();  
  carcel ( int numero, string nombre, string nombreCorto, int maxTurnos, long multa );
  
  // Copy constructor
  carcel ( const carcel& right );

  // Operador de asignacin sobrecargado
  carcel& operator= (const carcel& right);

  // Destructor
  ~carcel ();

  // Mtodos GET
  int get_maxTurnos () const;   
  long get_multa () const;

  // Mtodos SET
  void set_maxTurnos ( int maxTurnos );
  void set_multa ( long multa );

  // MTODOS

  
  // Otros mtodos
  /**
   * Polimorfismo de la clase casilla. En este caso no se realiza ninguna accin sobre
   * un jugador al caer sobre la crcel. Simplemente se informar sobre la casilla actual.
   */
  int accion ( jugador* player );

  /**
   * Mtodo que libera a un jugador de la crcel. Para ello debe indicar al jugador que no 
   * debe permanecer encarcelado ningn turno ms.
   */
  void desencarcelar ( jugador* j );

  /**
   * Mtodo que restar al jugador la cantidad de dinero que indica la multa, y le liberar
   * de la crcel. Si no tiene dinero le informar de que deber reunir el dinero antes del
   * final de su turno.
   */
  int desencarcelarPagando ( jugador* j );
  
};

// Mtodos GET
inline int carcel::get_maxTurnos () const
{
  return _maxTurnos;
}

inline long carcel::get_multa () const
{
  return _multa;
}

// Mtodos SET
inline void carcel::set_maxTurnos ( int maxTurnos )
{
  _maxTurnos = maxTurnos;
}

inline void carcel::set_multa ( long multa )
{
  _multa = multa;
}

#endif
