/***********************************************************************************
 *  CLASE CASILLA_TARJETAS
 *   Clase derivada de la clase casilla que representa todas las casillas que obligan
 *   a un jugador a coger tarjetas de algunos de los mazos. Estas casillas llevarn
 *   asociado un mazo de tarjetas, de la cual se coger una tarjeta y se ejecutar
 *   la accin pertinente.
 *
 *  Requiere:
 *     - clase casilla ( jerarquia )
 *     - clase mazo 
 *
 *  Autor: Javier Aragn Zabalegui (PFC para la uc3m: street Master's)
 *  Contacto: 100039831@alumnos.uc3m.es
 *  version 1.0
 **********************************************************************************/



#ifndef __CASILLA_TARJETAS__
#define __CASILLA_TARJETAS__

#include "casilla.h"
#include "mazo.h"

class casilla_tarjetas : public casilla {

 private:

  //Atributos  
  mazo* _mazoAsociado;    //Mazo del que se recogeran las tarjetas
  
 public:

  // Constructores
  casilla_tarjetas ();  
  casilla_tarjetas ( int numero, string nombre, string nombreCorto );
  
  // Copy constructor
   casilla_tarjetas (const casilla_tarjetas& right);

  // Sobrecarga de operadores  
  casilla_tarjetas& operator= (const casilla_tarjetas& right);

  // Destructor
  ~casilla_tarjetas ();

  // Mtodos GET
  mazo *get_mazo () const;

  // Mtodos SET
  void set_mazo ( mazo *nuevoMazo );

  // Mtodos
  
  // Polimorfismos:

  /**
   * Mtodo que dibuja la fila indicada de la casilla correspondiente (en las filas de arriba y abajo del tablero).
   * Se indicar la fila que se tiene que dibujar y el ancho con el que se tiene que pintar. Como resultado se
   * devolver una cadena de caracteres.
   */
  string dibujarCasillaHorizontal ( int fila, int anchoCasilla );
 
  /**
   * Mtodo que dibuja la fila indicada de la casilla correspondiente (en las columnas derecha e izquierda del tablero).
   * Se indicar la fila que se tiene que dibujar y el ancho con el que se tiene que pintar. Como resultado se
   * devolver una cadena de caracteres.
   */
  string dibujarCasillaVertical ( int fila, int anchoCasilla );

  /**
   * Devuelve la informacin de la casilla de tarjetas en modo texto para mostrar por pantalla la informacin
   * en forma de tabla. 
   */
  string leyendaCasilla ( int num, int nombreCorto, int tipo, int nombre, int estado );
  
  /**
   * Polimorfismo del mtodo accion de la clase base casilla. ste tipo de casilla tienen la
   * particularidad de ejecutar sus acciones a traves de las tarjetas de su mazo asociado, 
   * por lo que se remitir a ellas para ejecutar las acciones.
   */
  int accion ( jugador* player );  
};

// Mtodos GET
inline mazo *casilla_tarjetas::get_mazo () const
{
  return _mazoAsociado;
}

// Mtodos SET
inline void casilla_tarjetas::set_mazo ( mazo *nuevoMazo )
{
  _mazoAsociado = nuevoMazo;
}
#endif
