/***********************************************************************************
 *  CLASE DADO
 *  Clase que emula el funcionamiento de un dado. Para ello se utiliza el algoritmo
 *  de Merssene Twister, que ofrece una aleatoriedad mucho mejor que la que se puede
 *  obtener con otros algoritmos de C/C++ del estilo random()
 *
 *  Requiere:
 *     - clase mtrand (incluye el algoritmo de Merssene Twister)
 *
 *  Autor: Javier Aragn Zabalegui (PFC para la uc3m: street Master's)
 *  Contacto: 100039831@alumnos.uc3m.es
 *  version 1.1
 **********************************************************************************/

#ifndef __DADO__
#define __DADO__

#include "mtrand.h"
using namespace std;

class dado{

 private:
  //Atributos
  int _dado1;
  int _dado2;
  int _dadoTotal;		//dado_total = dado1 + dado2
  bool _tiradaDoble;  
  MTRand_int32 _irand;         // Generador de enteros de 32 bits con el algortimo de Merssene Twister  

  //Asociaciones

 public:

  // Constructor por defecto
  dado ();
  
  // Constructor explcito
  dado ( int dado1, int dado2 );
  
  // Copy constructor
  dado (const dado& right);

  // Operador de asignacin
  // Asigno a un jugador los datos de otro
  dado& operator= (const dado& right);

  // Destructor
  ~dado ();

  // Get operators
  int get_dado1 () const;
  int get_dado2 () const;
  int get_dadoTotal () const;
  bool get_tiradaDoble () const;

  // Set operators
  void set_dado1 ( int dado1 );
  void set_dado2 ( int dado2 );
  void set_dadoTotal ( int dadoTotal );
  bool set_tiradaDoble ( bool tiradaDoble );

  /** 
   *  Mtodo que devuelve un nmero pseudoaleatorio entre 1 y 6. Con ello se pretende
   *  representar la tirada de un dado. 
   *  Para crear nmeros pseudoaleatorios se usa el algoritmo de Merssenne Twister que 
   *  es mucho ms efectivo que la funcin random() de C
   */
  void tirarDado ( void );
    
};

inline int dado::get_dado1 () const
{
  return _dado1;
}

inline int dado::get_dado2 () const
{
  return _dado2;
}

inline int dado::get_dadoTotal () const
{
  return _dadoTotal;
}

inline bool dado::get_tiradaDoble () const
{
  return _tiradaDoble;
}


inline void dado::set_dado1 ( int dado1 )
{
  _dado1 = dado1;
}

inline void dado::set_dado2 ( int dado2 )
{
  _dado2 = dado2;
}

inline void dado::set_dadoTotal ( int dadoTotal )
{
  _dadoTotal = dadoTotal;
}

inline bool dado::set_tiradaDoble( bool tiradaDoble )
{
  _tiradaDoble = tiradaDoble;
}


#endif
