/***********************************************************************************
 *  CLASE ESTACION
 *  Clase que representa las casillas de tipo estacin del tablero de Monopoly. Hereda
 *  de la clase propiedades los atributos _hipoteca, _hipotecado, _hipotecable y _precio
 *  y aporta especficamente el alquiler en funcin de las estaciones que posea el
 *  propietario. Esto se puede calcular gracias a la clase monopolio (cuya referencia 
 *  hereda tambin de propiedad).
 *
 *  Requiere:
 *     - clase propiedad (herencia)
 *
 *  Autor: Javier Aragn Zabalegui (PFC para la uc3m: street Master's)
 *  Contacto: 100039831@alumnos.uc3m.es
 *  version 1.0
 **********************************************************************************/

#ifndef __ESTACION__
#define __ESTACION__

#include "propiedad.h"

#include <string>
using namespace std;

class estacion : public propiedad {

 private:

  /**
   * Alquileres en funcin del nmero de estaciones que se posean
   */
  long _alquilerUnaEstacion;;
  long _alquilerDosEstaciones;
  long _alquilerTresEstaciones;
  long _alquilerCuatroEstaciones;  

 public:

  // Constructor por defecto
  estacion ();
  
  // Constructor explcito
  estacion ( int numero, string nombre, string nombreCorto, long precio, long hipoteca, 
	     bool hipotecado, bool hipotecable, long alquilerUnaEstacion, 
	     long alquilerDosEstaciones, long alquilerTresEstaciones, 
	     long alquilerCuatroEstaciones, propietario *p );
  
  // Copy constructor
  estacion (const estacion& right);

  // Operador de asignacin
  estacion& operator= (const estacion& right);

  //Sobrecarga del opreador de salida
  friend ostream& operator<< ( ostream& os, estacion *e );

  // Destructor
  ~estacion ();

  // Mtodos GET
  long get_alquilerUnaEstacion () const;
  long get_alquilerDosEstaciones () const;
  long get_alquilerTresEstaciones () const;
  long get_alquilerCuatroEstaciones () const;
  
  // Mtodos SET
  void set_alquilerUnaEstacion ( long alquilerUnaEstacion );
  void set_alquilerDosEstaciones ( long alquilerDosEstaciones );
  void set_alquilerTresEstaciones ( long alquilerTresEstaciones );
  void set_alquilerCuatroEstaciones ( long alquilerCuatroEstaciones );
  
  /**
   * Mtodo que devuelve en una cadena de caracteres toda la informacin necesaria, con un
   * formato preparado para mostrarse por pantalla, de una casilla de tipo estacin, que 
   * incluye:
   *    - Nombre
   *    - Nmero de casilla
   *    - Precio
   *    - Alquileres (todas las posibilidades)
   *    - Hipoteca 
   */
  string mostrar ();

  /**
   * Mtodo que dibuja la fila indicada de la casilla correspondiente (en las filas de arriba y abajo del tablero).
   * Se indicar la fila que se tiene que dibujar y el ancho con el que se tiene que pintar. Como resultado se
   * devolver una cadena de caracteres.
   */
  string dibujarCasillaHorizontal ( int fila, int anchoCasilla );

  /**
   * Mtodo que dibuja la fila indicada de la casilla correspondiente (en las columnas derecha e izquierda del tablero).
   * Se indicar la fila que se tiene que dibujar y el ancho con el que se tiene que pintar. Como resultado se
   * devolver una cadena de caracteres.
   */
  string dibujarCasillaVertical ( int fila, int anchoCasilla );

  /**
   * Devuelve la informacin de la casilla de impuesto especial en modo texto para mostrar por pantalla la informacin
   * en forma de tabla. 
   */
  string leyendaCasilla ( int num, int nombreCorto, int tipo, int nombre, int estado );

  /**
   * Polimorfismo de la clase propiedad. 
   * Mtodo que obtiene el alquiler en funcin del nmero de estaciones que posee el propietario
   * de esta instancia de la clase estacin.
   */
  long calcularAlquiler ( jugador *j );

};

inline long estacion::get_alquilerUnaEstacion () const
{
  return _alquilerUnaEstacion;
}

inline long estacion::get_alquilerDosEstaciones () const
{
  return _alquilerDosEstaciones;
}

inline long estacion::get_alquilerTresEstaciones () const
{
  return _alquilerTresEstaciones;
}

inline long estacion::get_alquilerCuatroEstaciones () const
{
  return _alquilerCuatroEstaciones;
}
  
//Set operators

inline void estacion::set_alquilerUnaEstacion ( long alquilerUnaEstacion )
{
  _alquilerUnaEstacion = alquilerUnaEstacion;
}

inline void estacion::set_alquilerDosEstaciones ( long alquilerDosEstaciones )
{
  _alquilerDosEstaciones = alquilerDosEstaciones;
}

inline void estacion::set_alquilerTresEstaciones ( long alquilerTresEstaciones )
{
  _alquilerTresEstaciones = alquilerTresEstaciones;
}

inline void estacion::set_alquilerCuatroEstaciones ( long alquilerCuatroEstaciones )
{
  _alquilerCuatroEstaciones = alquilerCuatroEstaciones;
}

#endif
