#include "fichero.h"

/**Funci.An que muestra un fichero por pantalla, que viene dado por el nombre 
 * que se le pasar como parmetro.
 * IN: Fichero-> Nombre del fichero que se desea mostrar
 * OUT: 0 si ok, -1 si error 
 */
int mostrar_fichero ( string fichero )
{
  fstream ifs ( fichero.c_str (), ios::in );
  if ( !ifs )
    {
      cerr << "Fichero no disponible. Compruebe que exista " << fichero << endl;
      return -1;
    }
		
  //leer datos del fichero
  const int N=80;
  string linea;
  int ncars=0;
  do
    {      
      getline ( ifs, linea, '\n' );     //lee una linea
      if ( ifs.eof() ) break;		//fin de fichero
      cout << linea;			//imprimir linea
		
      if ( ( ncars = ifs.gcount() ) == N-1 ) //se activo el failbit
	ifs.clear();
      else //se encontro el delimitador
	cout << endl;
    } while ( true );
  
  if ( !ifs.eof() && ifs.fail() )
    {
      cerr << "Error al leer el fichero " << fichero << endl;
      return -1;
    }
  ifs.clear();
  return 0;
}


/**
 * Funcin que devuelve en un string el contenido del fichero dado por el nombre 
 * que se le pasar como parmetro.
 * IN: Fichero-> Nombre del fichero que se desea mostrar
 * OUT: 0 si ok, -1 si error 
 */
int obtener_fichero ( string fichero, string &contenido )
{
  fstream ifs ( fichero.c_str (), ios::in );
  if ( !ifs )
    {
      cerr << "Fichero no disponible. Compruebe que exista " << fichero << endl;
      return -1;
    }
		
  //leer datos del fichero
  const int N=80;
  string linea;
  int ncars=0;
  do
    {      
      getline ( ifs, linea, '\n' );     //lee una linea
      if ( ifs.eof() ) break;		//fin de fichero
      contenido += linea;			//imprimir linea
		
      if ( ( ncars = ifs.gcount() ) == N-1 ) //se activo el failbit
	ifs.clear();
      else //se encontro el delimitador
	contenido += "\n";
    } while ( true );
  
  if ( !ifs.eof() && ifs.fail() )
    {
      cerr << "Error al leer el fichero " << fichero << endl;
      return -1;
    }
  ifs.clear();
  return 0;
}

/**
 * Funcin que muestra un fichero por pantalla, que viene dado por el nombre 
 * que se le pasar como parmetro. Se mostrarn las lneas que se encuentran
 * entre inicio y fin.
 * IN: Fichero-> Nombre del fichero que se desea mostrar
 * OUT: 0 si ok, -1 si error 
 */
int mostrar_fichero_acotado ( string fichero, int inicio, int fin, bool &endf )
{
  fstream ifs ( fichero.c_str (), ios::in );
  if ( !ifs )
    {
      cerr << "Fichero no disponible. Compruebe que exista " << fichero << endl;
      return -1;
    }
		
  //leer datos del fichero
  const int N=80;
  string linea;
  int ncars=0;
  int i = 0;
  do
    {      
      endf = false;
      getline ( ifs, linea, '\n' );     //lee una linea
      if ( ifs.eof() ) 
	{
	  endf = true;
	  break;		//fin de fichero
	}
      if ( i >= inicio && i <= fin )
	cout << linea;			//imprimir linea
      i++;
      
      if ( ( ncars = ifs.gcount() ) == N-1 ) //se activo el failbit
	ifs.clear();
      else if ( i >= inicio && i <= fin ) //se encontro el delimitador	
	cout << endl;
    } while ( i <= fin );
  
  if ( !ifs.eof() && ifs.fail() )
    {
      cerr << "Error al leer el fichero " << fichero << endl;
      return -1;
    }
  ifs.clear();
  return 0;
}

/**
 * Mtodo que muestra un fichero paginado en el tamao de pgina que se indica en la
 * variable pgina.
 */
void paginar_fichero ( string fichero, int pagina )
{
  int inicio = 0;
  int fin = inicio + pagina;
  string opcion;
  bool endf = false;
  int ok = 0;
  // Leera el fichero hasta que se indique quiere salir
  do
    {
      system ("clear");
      //Se muestran las lineas comprendidas entre inicio y fin
      ok = mostrar_fichero_acotado ( fichero, inicio, fin, endf );
      cout << endl;
      do
	{
	  cout << endl;
	  // Caso generico
	  if ( endf == false && inicio != 0 )
	    cout << "Introduzca la opcin para navegar que desee ( (S)alir, (A)nterior, si(G)uiente): ";

	  // Ultima pgina
	  else if ( endf == true && inicio != 0 )
	    cout << "Introduzca la opcin para navegar que desee ( (S)alir, (A)nterior): ";
	  
	  // Primera pagina
	  else if ( endf == false && inicio <= 0 )
	    cout << "Introduzca la opcin para navegar que desee ( (S)alir, si(G)uiente): ";
	  
	  // Otros casos
	  else
	    cout << "Introduzca la opcin para navegar que desee ( (S)alir, (A)nterior, si(G)uiente): ";

	  cin >> opcion;
	}
      while ( opcion != "salir" && opcion != "S" && opcion != "siguiente" && opcion != "G" && opcion != "anterior" && opcion !="A" 
	      && opcion != "s" && opcion != "g" && opcion != "a" );
      
      //Recoloca los punteros de lectura
      if ( ( opcion == "siguiente" || opcion == "G" || opcion == "g" ) && endf == false )
	{
	  inicio = fin;
	  fin = inicio + pagina;
	}
      else if ( ( opcion == "anterior" || opcion == "A" || opcion == "a" ) && inicio != 0 )
	{
	  inicio = inicio - pagina;
	  fin = inicio + pagina;
	}
    }
  while ( ok != -1 && ( opcion != "salir"  && opcion != "S" && opcion != "s" ) );
}

