/***********************************************************************************
 *  CLASE IMPUESTO
 *  Clase que representa los impuestos del tablero del Monopoly. Esta clase hereda de 
 *  la clase base casilla. En esta clase se almacena la cantidad de dinero que se debe
 *  pagar por caer sobre ella.
 *
 *  Requiere:
 *     - clase casilla (herencia)
 *     - librera ctype.h de C
 *
 *  Autor: Javier Aragn Zabalegui (PFC para la uc3m: street Master's)
 *  Contacto: 100039831@alumnos.uc3m.es
 *  version 1.0
 **********************************************************************************/

#ifndef __IMPUESTO__
#define __IMPUESTO__

#include "casilla.h"

#include <ctype.h>	//toupper
#include <iostream>
using namespace std;

class impuesto : public casilla {

 private:

  //Atributos 
  int _cantidad;

 public:

  // Constructor por defecto
  impuesto ();
  
  // Constructor explcito
  impuesto ( int numero, string nombre, string nombreCorto, int cantidad );
  
  // Copy constructor
  impuesto (const impuesto& right);

  // Operador de asignacin
  impuesto& operator= (const impuesto& right);

  // Destructor
  ~impuesto ();

  // Mtodos GET
  int  get_cantidad () const;  

  // Mtodos SET
  void set_cantidad ( int cantidad );
  
  /**
   *  MTODOS DERIVADOS
   */

  /**
   * Mtodo que devuelve en una cadena de caracteres toda la informacin necesaria, con un
   * formato preparado para mostrarse por pantalla, de una casilla de tipo impuesto, que 
   * incluye:
   *    - Nombre
   *    - Nmero de casilla
   *    - Cantidad a pagar
   */
  virtual string mostrar ();

   /**
    * Polimorfismo de la clase casilla. En este caso el jugador debe pagar la cantidad de
    * dinero que se indica en el atributo _cantidad de esta clase. En caso de no tener 
    * suficiente dinero, el jugador pasar a ser moroso.
    */
  virtual int accion ( jugador* player );

  /**
   * Mtodo que dibuja la fila indicada de la casilla correspondiente (en las filas de arriba y abajo del tablero).
   * Se indicar la fila que se tiene que dibujar y el ancho con el que se tiene que pintar. Como resultado se
   * devolver una cadena de caracteres.
   */
  virtual string dibujarCasillaHorizontal ( int fila, int anchoCasilla );
  
  /**
   * Mtodo que dibuja la fila indicada de la casilla correspondiente (en las columnas derecha e izquierda del tablero).
   * Se indicar la fila que se tiene que dibujar y el ancho con el que se tiene que pintar. Como resultado se
   * devolver una cadena de caracteres.
   */
  virtual string dibujarCasillaVertical ( int fila, int anchoCasilla );
 
  /**
   * Mtodo que devuelve una cadena de caracteres que representa como es la casilla en su 
   * imgen en el tablero, indicando aquellos datos de inters para un jugador que caiga 
   * sobre ella:
   *  -Numero
   *  -Nombre
   *  -Impuesto fijo
   */
  virtual string dibujarCasillaEnTablero ();

  /**
   * Devuelve la informacin de la casilla de impuesto en modo texto para mostrar por pantalla la informacin
   * en forma de tabla. 
   */
  virtual string leyendaCasilla ( int num, int nombreCorto, int tipo, int nombre, int estado );

};

inline int impuesto::get_cantidad () const
{
  return _cantidad;
}

inline void impuesto::set_cantidad ( int cantidad )
{
  _cantidad = cantidad;
}

#endif
