/***********************************************************************************
 *  CLASE IMPUESTO5
 *  Clase que representa los impuestos especiales del tablero del Monopoly. Esta clase
 *  hereda de la clase base impuesto (que a su vez hereda de casilla). En esta clase se
 *  almacena como atributo propio un tanto por ciento (_tpc) que representa el
 *  porcentaje sobre el capital que debe pagar el jugador. 
 *  El jugador puede elegir entre el porcentaje o pagar la cantidad fija que hereda de
 *  la clase impuesto.
 *
 *  Requiere:
 *     - clase impuesto (herencia)
 *
 *  Autor: Javier Aragn Zabalegui (PFC para la uc3m: street Master's)
 *  Contacto: 100039831@alumnos.uc3m.es
 *  version 1.0
 **********************************************************************************/


#ifndef __IMPUESTO5__
#define __IMPUESTO5__

#include "impuesto.h"

class impuesto5 : public impuesto {

 private:

  //Atributos
  int _tpc;  

 public:

  // Constructor por defecto
  impuesto5 ();
  
  // Constructor explcito
  impuesto5 ( int numero, string nombre, string nombreCorto, int cantidad, int tpc );
  
  // Copy constructor
  impuesto5 (const impuesto5& right);

  // Operador de asignacin
  impuesto5& operator= (const impuesto5& right);

  // Destructor
  ~impuesto5 ();

  // Mtodos GET 
  int  get_tpc () const;  
  
  // Mtodos SET
  void set_tpc ( int tpc );
  
  /**
   *  MTODOS DERIVADOS
   */
  
  /**
   * Mtodo que devuelve en una cadena de caracteres toda la informacin necesaria, con un
   * formato preparado para mostrarse por pantalle,  de una casilla de tipo impuesto5, que 
   * incluye:
   *    - Nombre
   *    - Nmero de casilla
   *    - Cantidad a pagar
   *    - Porcentaje 
   */
  string mostrar ();

  /**
   * Polimorfismo de la clase casilla. En este caso se preguntar al jugador si desea pagar
   * la cantidad fija que hereda de la clase impuesto o prefiere pagar un tanto por ciento del
   * capital calculado a partir del capital del jugador (player) y el porcentaje que se indica
   * en el atributo _tpc de esta misma clase.
   */
  int accion ( jugador *player ); 

  /**
   * Mtodo que dibuja la fila indicada de la casilla correspondiente (en las filas de arriba y abajo del tablero).
   * Se indicar la fila que se tiene que dibujar y el ancho con el que se tiene que pintar. Como resultado se
   * devolver una cadena de caracteres.
   */
  string dibujarCasillaHorizontal ( int fila, int anchoCasilla );
  
  /**
   * Mtodo que dibuja la fila indicada de la casilla correspondiente (en las columnas derecha e izquierda del tablero).
   * Se indicar la fila que se tiene que dibujar y el ancho con el que se tiene que pintar. Como resultado se
   * devolver una cadena de caracteres.
   */
  string dibujarCasillaVertical ( int fila, int anchoCasilla );

  /**
   * Mtodo que devuelve una cadena de caracteres que representa como es la casilla en su 
   * imgen en el tablero, indicando aquellos datos de inters para un jugador que caiga 
   * sobre ella:
   *  -Numero
   *  -Nombre
   *  -Impuesto fijo
   *  -Porcentaje
   */
  string dibujarCasillaEnTablero ();

  /**
   * Devuelve la informacin de la casilla de impuesto especial en modo texto para mostrar por pantalla la informacin
   * en forma de tabla. 
   */
  string leyendaCasilla ( int num, int nombreCorto, int tipo, int nombre, int estado );

};

inline int impuesto5::get_tpc () const
{
  return _tpc;
}

inline void impuesto5::set_tpc ( int tpc )
{
  _tpc = tpc;
}

#endif
