/**********************************************************************************
				jugador.cpp
	
Programa: 	3
Nombre:		Javier Aragn Zabalegui		100039831@alumnos.uc3m.es
Fecha_inicio:	30.Octubre.2005
Descripcion:	Clase jugador para Street Master's
			
Comentarios:	
**********************************************************************************/

/**********************************************************************************
Listado de contenidos:
	include "jugador.h"

**********************************************************************************/

#include "jugador.h"
#include "propiedad.h"
#include "estacion.h"
#include "servicio.h"
#include "calle.h"
#include "tarjeta.h"
#include "negociacion.h"

#include "tablero.h"


/**
 * Constructor por defecto
 */
jugador::jugador ()
  : _nombre ("Desconocido"),    
    _color ("blanco"),
    _dinero_total ( 150000 ),
    _capital ( 150000 ),
    _turnosEncarcelado ( 0 ),
    _tiradasDobles ( 0 ),
    _estado ( 0 ),
    _estaEn ( NULL ),
    _multiPago ( 1 ),
    lanza ( NULL )
{

  // There is nothing to do!
}

/**
 * Constructor explcito
 */
jugador::jugador ( string nombre, string color, long dinero_total, long capital, 
		   int turnosEncarcelados, int tiradasDobles, long estado, int multiPago )
  : _nombre ( nombre ),
    _color ( color ),
    _dinero_total ( dinero_total ),
    _capital ( capital ),
    _turnosEncarcelado ( turnosEncarcelados ),
    _tiradasDobles ( tiradasDobles ),
    _estado ( estado ),
    _estaEn ( NULL ),
    _multiPago ( multiPago ),
    lanza ( NULL )
{
  // No se hace nada
}

 
/**
 * Copy constructor
 */
jugador::jugador (const jugador& right)
{
  _nombre		= right._nombre;
  _color		= right._color;
  _dinero_total		= right._dinero_total;
  _capital		= right._capital;
  _turnosEncarcelado	= right._turnosEncarcelado;
  _tiradasDobles	= right._tiradasDobles;
  _estado		= right._estado;
  _multiPago            = right._multiPago;
  _estaEn               = right._estaEn;
  lanza                 = right.lanza;
}


/**
 * Operador de asignacin
 */
jugador& jugador::operator= (const jugador& right)
{
  _nombre		= right.get_nombre ();
  _color		= right.get_color ();
  _dinero_total		= right.get_dinero_total ();
  _capital		= right.get_capital ();
  _turnosEncarcelado	= right.get_turnosEncarcelado();
  _tiradasDobles	= right.get_tiradasDobles();
  _estado		= right.get_estado();
  _multiPago            = right._multiPago;
  _estaEn               = right._estaEn;
  lanza                 = right.lanza;
  return ( *this );
}

/**
 * Operador de desigualdad. Compara dos jugadores y determina si se trata del
 * mismo jugador. Como los jugadores cuentan con un color de ficha nivoco, se
 * comparar por este atributo.
 */
bool jugador::operator!= ( const jugador& player )
{
  if ( player.get_nombre() != get_nombre() )
    return true;
  else
    return false;
}

/**
 * Sobrecarga del operador de salida
 */
ostream& operator<< ( ostream &os, jugador* j )
{
  string aux;
  aux = "\n\t-NOMBRE:\t" + j->get_nombre () + "\n";
  aux += "\t-FICHA (color):\t" + j->get_color () + "\n";
  char capitalAlpha[30];
  sprintf( capitalAlpha, "%d", j->get_capital() );  
  aux += "\t-CAPITAL:\t" + string (capitalAlpha) + "\n";
  char dineroTotalAlpha[30];
  sprintf( dineroTotalAlpha, "%d", j->get_dinero_total () );
  aux += "\t\t+DINERO EFECTIVO:\t" + string (dineroTotalAlpha) + "\n";
  char capitalInvertidoAlpha[30];
  sprintf( capitalInvertidoAlpha, "%d", j->get_capital() - j->get_dinero_total () );
  aux += "\t\t+CAPITAL INVERTIDO:\t" + string(capitalInvertidoAlpha)  + "\n";  
  if ( j->estaEncarcelado() )
    {
      char turnosEncarceladoAlpha[5];
      sprintf( turnosEncarceladoAlpha, "%d", j->get_turnosEncarcelado () );  
      if ( j->get_turnosEncarcelado() > 1 )
	aux += "\tFaltan " + string (turnosEncarceladoAlpha) + " turnos para que salga de la crcel.\n";
      else
	aux += "\tFalta " + string (turnosEncarceladoAlpha) + " turno para que salga de la crcel.\n";
    }
  if ( j->tieneTarjetasCarcel () > 0 )
    {
      char tarjetasCarcelAlpha[20];
      sprintf( tarjetasCarcelAlpha, "%d", j->tieneTarjetasCarcel () );
      aux += "\tDispone de " + string (tarjetasCarcelAlpha) + " tarjetas para salir libre de la crcel.\n";
    }
  if ( j->get_tiradasDobles() > 0 )
    {
      char tiradasDoblesAlpha[10];
      sprintf( tiradasDoblesAlpha, "%d", j->get_tiradasDobles () );
      aux += "\t" + string (tiradasDoblesAlpha) + " tiradas dobles consecutivas.\n";
    }
  char numPropiedadesAlpha[10];
  sprintf( numPropiedadesAlpha, "%d", j->get_numPropiedades () );
  aux += "\tPROPIEDADES (" + string (numPropiedadesAlpha) + ") :\n";  
  
  aux += j->get_infoPropiedades();
  
  os << aux;
  return os;
}

/**
 * Destructor
 */
jugador::~jugador ()
{
  // no se hace nada
}

/**
 * Mtodo que lanza el dado y devuelve el valor de cada uno de
 * los dados lanzados y un booleano que indica si la tirada
 * que se a producido es doble.
 */
int jugador::tirarDado ( bool &tiradaDoble, int &dado1, int &dado2 )
{  
  // Si no existe ningun dado asociado, se crea uno y se asocia
  if ( lanza == NULL )
    {
      dado *d = new dado();
      lanza = d;
    }
  //Lanza el dado
  lanza->tirarDado();
  dado1 = lanza->get_dado1 ();
  dado2 = lanza->get_dado2 ();
  tiradaDoble = lanza->get_tiradaDoble ();
  return lanza->get_dadoTotal ();
}

/**
 * Lanza el dado y devuelve el valor total de la tirada.
 * En esta implementacin slo importa el resultado, por lo que
 * no se devuelve ni se muestra el valor intermedio de cada uno
 * de los dados, ni si se trata de una tirada doble.	
 */
int jugador::tirarDado ()
{
  // Si no existe ningn dado asociado al jugador, lo crea
  if ( lanza == NULL )
    {
      dado *d = new dado();
      lanza = d;
    }
  // Lanza el dado
  lanza->tirarDado();
  return lanza->get_dadoTotal ();
}

/**
 * Devuelve la ultima tirada realizada por el jugador
 */
int jugador::ultimaTirada ()
{
  if ( lanza != NULL )
    return lanza->get_dadoTotal();  
  return 6;  
}

/**
 * El jugador invierte una cantidad de donero, por lo que se resta la cantidad
 * del dinero total, pero se mantiene en el capital.
 */
long jugador::invertir ( long cantidad )
{
  if ( cantidad <= get_dinero_total() )    
    _dinero_total -= cantidad;          
  else    
    _estado -= cantidad;    
  return ( _dinero_total );
}

/**
 * Mtodo mediante el cual el jugador recupera una cantidad invertida. Del capital se
 * debe restar lo que costo cuando se invirti y sumar el dinero que entra en caja.
 * Se debe sumar la cantidad que recupera al dinero total.
 */
long jugador::recuperarInversion ( long recupera, long costoAntes )
{
  _dinero_total += recupera;
  _capital -= costoAntes;
  _capital += recupera;
}

/**
 * Mtodo por el cual un jugador paga una cantidad. Si la cantidad es 
 * positiva, el jugador pagar dinero, en caso de ser negativa, el jugador estar
 * cobrando dinero. Al pagar, se debe restar la cantidad del dinero total y del
 * capital. Si no tiene dinero suficiente, se indica que es moroso (devuelve -1).
 */
long jugador::pagar ( long cantidad )
{
  if ( cantidad <= get_dinero_total() )
    {
      _dinero_total -= cantidad;
      _capital -= cantidad;
    }
  else
    {
      _estado -= cantidad;
      return -1;
    }
  return ( _dinero_total );
}

int jugador::obtenerCasilla() const
{
  casilla *actual;
  actual = get_estaEn();
  return ( actual -> get_numero() );
  //return 0;
}

/**
 * Elimina la referencia de la casilla en la que est hacia el propio jugador
 */
void jugador::desvincularDeCasilla()
{
  _estaEn->eliminarJugador( this ); 
}

/**
 * Mtodo que comprueba si el jugador est encarcelado
 */
bool jugador::estaEncarcelado ()
{
  if ( get_turnosEncarcelado() > 0 )
    return true;
  return false;
}

/**
 * Mtodo que obtiene informacin de todas las propieadades del jugador
 */
string jugador::get_infoPropiedades()
{
  string aux = "";  
  list <propiedad*>::iterator p; 

  //Recorre toda la lista de propiedades obteniendo informacin bsica de cada una.
  for ( p = _posee.begin(); p != _posee.end(); p++ )
    {
      aux += "\t";
      aux += (*p)->infoString() + "\n";
    }
 
  return aux;
}

/**
 * Lista todas las propiedades hipotecables indicando nmero, nombre de propiedad y cantidad de
 * dinero que se obtiene al hipotecarla.
 */
string jugador::listarPropiedadesHipotecables ()
{
  int tamano = 0;
  string aux = "";
  bool existe = false;
  list <propiedad*>::iterator p; 
  //Recorre toda la lista de propiedades obteniendo la informacin de las hipotecables
  for ( p = _posee.begin(); p != _posee.end(); p++ )
    {
      // Si es hipotecable, recopila informacin
      if ( (*p)->get_hipotecable() )
	{
	  // Nmero
	  char numAlpha[10];
	  sprintf( numAlpha, "%d", (*p)->get_numero() );
	  
	  //Hipoteca
	  char hipotecaAlpha[30];
	  sprintf( hipotecaAlpha, "%d", (*p)->get_hipoteca() );
	  aux += "(" + string ( numAlpha ) + ") " + (*p)->get_nombre();
	  for ( int i = aux.size(); i <= 40 + tamano; i++ )
	    aux += " ";
	  aux += "Hipoteca = " + string ( hipotecaAlpha ) + "\n";
	  existe = true;
	  tamano = aux.size ();
	}
    }

  //Si no existe ninguna propiedad hipotecable, se informa de ello.
  if ( ! existe )
    aux = "No dispone de ninguna propiedad hipotecables\n";
  return ( aux );
}

/**
 * Lista todas las propiedades hipotecadas indicando nmero, nombre de propiedad y cantidad de
 * dinero que cuesta deshipotecarla.
 */
string jugador::listarPropiedadesHipotecadas ()
{
  int tamano = 0;
  string aux = "";
  bool existe = false;
  list <propiedad*>::iterator p; 
  // Recorre toda la lista de propiedades hipotecadas recopilando informacion
  for ( p = _posee.begin(); p != _posee.end(); p++ )
    {
      if ( ! (*p)->get_hipotecable() )
	{
	  // Numero
	  char numAlpha[10];
	  sprintf( numAlpha, "%d", (*p)->get_numero() );
	  
	  //Deshipoteca
	  char hipotecaAlpha[30];
	  //Deshipotecar cuesta hipoteca + 10% de intereses
	  long levantarHipoteca =  ( (*p)->get_deshipoteca() );
	  sprintf( hipotecaAlpha, "%d", levantarHipoteca );
	  
	  // Nombre
	  aux += "(" + string ( numAlpha ) + ") " + (*p)->get_nombre();
	  for ( int i = aux.size(); i <= 40 + tamano ; i++ )
	    aux += " ";
	  aux += "Levantar hipoteca = " + string ( hipotecaAlpha ) + "\n";
	  tamano = aux.size();
	  existe = true;
	}
    }

  //Si no hay propiedades hipotecadas, se indica.
  if ( ! existe )
    aux = "No dispone de ninguna propiedad hipotecada\n";
  return ( aux );
}

/**
 * Lista todas las propiedades edificables indicando nmero, nombre de propiedad, precio de un
 * nuevo edificio y los edificios disponibles actualmente.
 */
string jugador::listarCallesConstruibles ()
{
  int tamano = 0;
  string aux = "";
  bool existe = false;
  list <propiedad*>::iterator p; 
  for ( p = _posee.begin(); p != _posee.end(); p++ )
    {
      if ( calle *c = dynamic_cast <calle *> ( *p ) )
	{
	  if ( c->esConstruible () )
	    {
	      char numAlpha[15];
	      sprintf( numAlpha, "%d", c->get_numero() );
	      aux += "(" + string ( numAlpha ) + ") " + c->get_nombre();
	      for ( int i = aux.size(); i <= 40 + tamano; i++ )
		aux += " ";
	      aux += "ACTUALMENTE: ";
	      if ( c->get_numHoteles () == 0 )
		{
		  char numCasas[10];
		  sprintf( numCasas, "%d", c->get_numCasas() );
		  aux += string ( numCasas ) + " casas";
		}
	      else
		{
		  char numHoteles[10];
		  sprintf( numHoteles, "%d", c->get_numHoteles() );
		  aux += string ( numHoteles ) + " hotel";
		}
	      aux += "\n";
	      tamano = aux.size();
	    }
	}
    }
  if ( aux == "" )
    aux = "No hay ninguna casilla construible actualmente\n";
  return aux;
}

/**
 * Lista todas las propiedades edificadas indicando nmero, nombre de propiedad, precio de un
 * edificio y los edificios construidos actualmente.
 */
string jugador::listarCallesConstruidas ()
{
  int tamano = 0;
  string aux = "";
  bool existe = false;
  list <propiedad*>::iterator p; 
  // Recorre toda la lista de propieades buscando aquellas que se pueden vender edificios
  for ( p = _posee.begin(); p != _posee.end(); p++ )
    {
      //Slo se pueden vender edificios de las calles.
      if ( calle *c = dynamic_cast <calle *> ( *p ) )
	{
	  // Si se puede vender el edificio (indicar si hotel o casa)
	  if ( c->puedeVenderEdificio () )
	    {
	      //Existe hotel construido
	      if ( c->get_numHoteles () > 0 )
		{
		  // Nmero
		  char numAlpha[10];
		  sprintf( numAlpha, "%d", c->get_numero() );
		  aux += "(" + string ( numAlpha ) + ") " + c->get_nombre();
		  for ( int i = aux.size(); i <= 40 + tamano; i++ )
		    aux += " ";

		  // Indica que hay hotel
		  aux += "ACTUALMENTE: ";	      
		  char numHoteles[10];
		  sprintf( numHoteles, "%d", c->get_numHoteles() );
		  aux += string ( numHoteles ) + " hotel";
		  aux += "\n";
		}
	      else if ( c->get_numCasas() > 0 )
		{
		  // Nmero
		  char numAlpha[10];
		  sprintf( numAlpha, "%d", c->get_numero() );

		  // Nombre
		  aux += "(" + string ( numAlpha ) + ") " + c->get_nombre();
		  for ( int i = aux.size(); i <= 40 + tamano; i++ )
		    aux += " ";
		  
		  // Nmero de casas
		  aux += "ACTUALMENTE: ";	      
		  char numCasas[10];
		  sprintf( numCasas, "%d", c->get_numCasas() );
		  aux += string ( numCasas ) + " casas";
		  aux += "\n";
		}
	    }
	  tamano = aux.size();
	}      
    }
  
  //Si no hay propiedades construidas se indica.b
  if ( aux == "" )
    aux = "No hay ninguna casilla construible actualmente\n";
  return aux;
}

/**
 * Obtiene el total de casas que ha construido un jugador sobre sus calles.
 */
int jugador::get_casasConstruidas()
{
  list <propiedad*>::iterator p; 
  int numCasas = 0;

  //Recorre todas las propiedades, compruba si es calle y suma al total el nmero de casas
  for ( p = _posee.begin(); p != _posee.end(); p++ )
    {
      if ( calle *c = dynamic_cast<calle *> ( *p ) )
	numCasas += c->get_numCasas();
    }
  return numCasas;
}

/**
 * Obtiene el total de hoteles que ha construido un jugador sobre sus calles.
 */
int jugador::get_hotelesConstruidos()
{
  list <propiedad*>::iterator p; 
  int numHoteles = 0;

  //Recorre todas las propiedades, compruba si es calle y suma al total el nmero de hoteles (1)
  for ( p = _posee.begin(); p != _posee.end(); p++ )
    {
      if ( calle *c = dynamic_cast <calle *> ( *p ) )
	numHoteles += c->get_numHoteles();
    }
  return numHoteles;
}

/**
 * Mtodo que devuelve el nmero de tarjetas de suerte / caja que posee un jugador.
 * Si no tiene ninguna, devuelve 0.
 */
int jugador::tieneTarjetasCarcel ()
{
  // Si no tiene devolver 0
  return _tieneTarjeta.size();
}

/**
 * Mtodo que devuelve el mazo al que pertenece la primera tarjeta de la lista de tarjetas del jugador
 */
string jugador::mazoTarjetaSuerte ()
{  
  tarjeta *tjt = *_tieneTarjeta.begin();
  return ( tjt->get_tipoMazo() );
}

/**
 * El jugador devuelve una tarjeta, es decir, elimina la primera tarjeta de su lista.
 * Antes de eliminarla, la devuelve.
 */
tarjeta *jugador::devuelveTarjeta ()
{
  //Coge la primera tarjeta
  tarjeta *tjt = *_tieneTarjeta.begin ();

  //Elimina la primera tarjeta
  _tieneTarjeta.erase ( _tieneTarjeta.begin() );

  //Devuelve la primera tarjeta
  return ( tjt );
}

/**
 * El jugador actual inicia un proceso de negociacin con otro jugador indicado como argumento. Para ello
 * invocar a metodos de la clase negociacin
 */
void jugador::negociar ( jugador *receptor )
{
  cout << "Se inicia la negociacion entre " << get_nombre () << " y " << receptor->get_nombre() << endl;
  negociacion *n = new negociacion ();
  n->set_emisor ( this );
  n->set_receptor ( receptor );
  n->negociar ();
  delete ( n );
}

/**
 * Devuelve la primera tarjeta de la lista. No la elimina de ella.
 */
tarjeta *jugador::get_tarjeta ()
{
  return *_tieneTarjeta.begin();
}

/**
 * Devuelve la tarjeta de la lista que est en el orden 'orden'. No la elimina de ella.
 */
tarjeta *jugador::get_tarjeta ( int orden )
{
  list <tarjeta*>::iterator listaT; 
  listaT = _tieneTarjeta.begin();
  for ( int i = 0; i < orden; i++ )
    {
      listaT++;
    } 
  return *listaT;
}

/**
 * Mtodo que aade una lista de tarjetas a la lista de tarjetas ya existente. Es decir, une dos
 * listas de tarjetas en las posesiones del jugador.
 */
void jugador::anyadirListaTarjetas ( list <tarjeta*> listaTarjetas )
{
  list <tarjeta*>::iterator listaT; 
  listaT = _tieneTarjeta.begin();
  //Recorre la lista de tarjetas pasada como argumento, aadiendola al final de la lista del jugador.
  for ( int i = 1; i <= listaTarjetas.size(); i++ )
    {
      incluir_tieneTarjeta ( (*listaT) );
      listaT++;
    }
  return;
}

/**
 * Elimina una lista de tarjetas del jugador.
 */
void jugador::eliminarListaTarjetas ( list <tarjeta*> listaTarjetas )
{
  for ( int i = 1; i <= listaTarjetas.size(); i++ )
    {
      eliminarTarjeta ();      
    }
  return;
}

/**
 * Elimina la primera tarjeta del jugador.
 */
void jugador::eliminarTarjeta ()
{
  _tieneTarjeta.erase ( _tieneTarjeta.begin() );
  return;
}

/**
 * Mtodo que elimina todos los edificios de las calles y paga la cantidad estipulada por la
 * venta de todos ellos.
 */
long jugador::expropiarEdificios ()
{  
  long total = 0;
  list <propiedad*>::iterator listaP;
  
  // Recorre toda la lista de propiedades, eliminando de cada una todas las casas y hoteles
  // y obteniendo la cantidad de dinero de la venta de cada una
  for ( listaP = _posee.begin(); listaP != _posee.end(); listaP++ )    
    {
      if ( calle *c = dynamic_cast<calle *> ( *listaP ) )
	{
	  int numHoteles = c->get_numHoteles ();
	  int numCasas = c->get_numCasas ();
	  int ventaC = c->get_precioCasa () / 2;
	  // Si hay un hotel, se vende el hotel y las 4 casas
	  if ( numHoteles == 1 )
	    {
	      int ventaH = c->get_precioHotel() / 2;
	      total += ventaH + ( ventaC * 4 );
	    }
	  else
	    {
	      total += numCasas * ventaC;
	    }
	  c->set_numCasas ( 0 );
	  c->set_numHoteles ( 0 );
	}
    }
  return total;
}

/**
 * Mtodo que devuelve una cadena de caracteres sobre el jugador lista para ser guardada en 
 * un fichero de partida guardada. La informacin incluye nombre, color, dinero, capital, 
 * turnos encarcelado, tiradas dobles, estado y posicin actual.
 */
string jugador::guardarDatosPersonales ()
{
  string jugadorG;
  jugadorG = "NOMBRE=" + get_nombre () + "\n";
  jugadorG += "COLOR=" + get_color () + "\n";
  char numAlpha[30];
  sprintf ( numAlpha, "%d", get_dinero_total () );
  jugadorG += "DINERO= " + string ( numAlpha );
  jugadorG += "\n";
  sprintf ( numAlpha, "%d", get_capital () );
  jugadorG += "CAPITAL= " + string ( numAlpha );
  jugadorG += "\n";
  sprintf ( numAlpha, "%d", get_turnosEncarcelado () );
  jugadorG += "TURNOS ENCARCELADO= " + string ( numAlpha );
  jugadorG += "\n";
  sprintf ( numAlpha, "%d", get_tiradasDobles () );
  jugadorG += "TIRADAS DOBLES= " + string ( numAlpha );
  jugadorG += "\n";
  sprintf ( numAlpha, "%d", get_estado () );
  jugadorG += "ESTADO= " + string ( numAlpha );
  jugadorG += "\n";
  casilla *c = get_estaEn ();
  sprintf ( numAlpha, "%d", c->get_numero () );
  jugadorG += "POSICION= " + string ( numAlpha );
  return jugadorG;
}

/**
 * Mtodo que devuelve todos los datos de un jugador para guardarlo en un fichero.
 * Incluye los datos personales del jugador, los numeros de sus propiedades y las tarjetas
 * que posee.
 */
string jugador::guardar ()
{
  char numAlpha[30];
  string jugadorG;
  //Datos personales
  jugadorG = guardarDatosPersonales();
  
  // Propiedades (slo si posee alguna)
  if ( _posee.size() > 0 )
    { 
      //Etiqueta identificativa
      jugadorG += "\nPROPIEDADES = ";
      list <propiedad*>::iterator listaP;
      
      //Para cada propiedad guarda el nmero
      for ( listaP = _posee.begin(); listaP != _posee.end(); listaP++ )    
	{
	  if (! ( (listaP) == _posee.begin() ) )
	    jugadorG += ", ";
	  sprintf ( numAlpha, "%d", (*listaP)->get_numero () );
	  jugadorG += string ( numAlpha );
	}
    }

  // Tarjetas (slo si tiene)
  if ( _tieneTarjeta.size() > 0 )
    { 
      //Etiqueta
      jugadorG += "\nTARJETAS = ";
      list <tarjeta*>::iterator listaT;
      //Recorre la lista de propiedades guardando "nmero (mazo)"
      for ( listaT = _tieneTarjeta.begin(); listaT != _tieneTarjeta.end(); listaT++ )    
	{
	  if (! ( (listaT) == _tieneTarjeta.begin() ) )
	    jugadorG += ", ";
	  sprintf ( numAlpha, "%d", (*listaT)->get_numero () );
	  jugadorG += string ( numAlpha ) + "(" + (*listaT)->get_tipoMazo() + ")";
	}
    }
   jugadorG += ";\n";
  return jugadorG;
}

/**
 * Mtodo que aade una lista de varias propiedades a las que el jugador ya posee.
 * Se actualiza el capital del jugador
 */
void jugador::anyadirListaPropiedades ( list <propiedad*> listaPropiedades )
{
  long capital = get_capital ();
  long total = 0;
  list <propiedad*>::iterator listaP;

  //Recorre toda la lista que se pasa como argumento e inserta cada propiedad al final
  //de la lista de posesiones del jugador. Cuando se inserta hay que tener en cuenta
  //si hay que realizar cambios en los permisos de construccin de las calles.
  for ( listaP = listaPropiedades.begin(); listaP != listaPropiedades.end(); listaP++ )    
    {
      // Cantidad que debe aumentar al capital
      total += (*listaP)->get_precio ();

      //Indicar a la propiedad que su dueo es el jugador y al jugador que posee la propiedad
      (*listaP)->set_esPropiedad ( this );
      set_posee ( *listaP );

      //Si es una calle, actualiza los permisos de construccion
      if ( calle *c = dynamic_cast<calle *> ( *listaP ) )
	{
	  monopolio *mp = c->get_perteneceMonopolio ();
	  if ( mp->construible () )
	    mp->permitirConstruir();
	  else
	    mp->prohibirConstruir ();
	}
    }
  //Actualizar el capital.
  set_capital ( capital + total );
  return;
}

/**
 * Elimina de las posesiones del jugador todas las propiedades que se incluyen en la lista
 * de propiedades pasadas como argumento del mtodo.
 */
void jugador::eliminarListaPropiedades ( list <propiedad*> listaPropiedades )
{
  long capital = get_capital ();
  long total = 0;
  list <propiedad*>::iterator listaP; 
  for ( listaP = listaPropiedades.begin(); listaP != listaPropiedades.end(); listaP++ )        
    {
      total += (*listaP)->get_precio ();
      eliminarPropiedad ( *listaP );    
    }
  set_capital ( capital - total );
  return;
}


//MTODOS VIRUTALES: Se redefinene en la clase jugador_reglas

/**
 * Mtodo que hace que el jugador puje. Se devolver la cantidad que puja, 0 si pasa o
 * -1 si decide salir de la puja.
 */
long jugador::pujar ( propiedad *p, long maxActual, jugador *ganador )
{
  string puja;
  long pujaLong = 0;
  //Prompt de la puja
  cout << "Puja " << get_nombre() << " ( mximo " << get_dinero_total() << " ) : ";
  cin.clear();
  cin >> puja;
  cin.clear();
  cin.ignore( 2000, '\n' );
  //Si decide salir, se devuelve -1
  if ( puja == "salir" )
    {
      cout << get_nombre() << ", has abandonado la subasta." << endl;
      pujaLong = -1;
    }
  //Si no, se devuelve la puja ( 0 si pasar)
  else
    {
      pujaLong = atol ( puja.c_str() );
      if ( pujaLong > get_dinero_total () )
	{
	  cout << "No puedes pujar tanto dinero, has perdido el turno" << endl;
	  pujaLong = 0;
	}
    }
  return pujaLong;
}

/**
 * El jugador deber responder si quiere comprar una propiedad o no. La propiedad se pasa como
 * argumento. El jugador lo indicara mediante una resupuesta (s,n).
 */
bool jugador::decidirComprar ( propiedad *p )
{
  cout << "Esta propiedad no pertenece a ningn jugador, quieres comprarla? (S/n)";
  char opcion = cin.get();
  cin.clear();
  cin.ignore( 2000, '\n' );
  if ( opcion == 'N' || opcion == 'n' )
    return false;  
  return true;
}

/**
 * El jugador deber responder si quiere pagar para salir de la carcel o no. La propiedad se pasa como
 * argumento. El jugador lo indicara mediante una resupuesta (s,n).
 * Es necesario conocer la multa y se pasa el argumento PropiedadesLibres para que el jugador basado en
 * reglas pueda decidir.
 */
bool jugador::decidirSalirPagando ( int tpcPropiedadesLibres, long multa )
{
   cout << "Si lo deseas puedes pagar una multa de " << multa << " para salir de la crcel." << endl;
   cout << "Deseas pagar la multa? (s/N)";      
   char opcion = cin.get();
   if ( opcion == 'S' or opcion == 's' )
     return true;
   return false;
}

/**
 * El jugador deber responder si quiere salir de la carcel usando una tarjeta. El jugador lo indicara 
 * mediante una resupuesta (s,n).
 * El argumento PropiedadesLibres para que el jugador basado en reglas pueda decidir.
 */
bool jugador::decidirSalirTarjeta ( int tpcPropiedadesLibres )
{
  cout << "Puedes usar la tarjeta para salir de la crcel de la que dispones." << endl;
  cout << "Quieres hacerlo? (s,N)";
  char opcion = cin.get();
  if ( opcion == 'S' or opcion == 's' )          
    return true;    
  return false;
}

/**
 * Cuando un jugador recoja una tarjeta que tenga ms de una opcin, deber decidir cual de ellas
 * quiere ejecutar. Para ello se leera del teclado la opcin que desea ejecutar.
 */
int jugador::decidirAccionTarjeta ( int numAcciones, tarjeta *tjt )
{
  int opcion = 0;
  int intento = 0;
  bool valor = false;
  do
    {
      intento++;
      cout << endl << "Elija la opcin que desee ejecutar (Nmero entre 1 y " << numAcciones << ") ";
      cin >> opcion;	   
      if ( !( ( opcion >= 1 ) && ( opcion <= numAcciones ) ) )
	{
	  cerr << "Opcin no vlida.\n";	      
	  cin.clear(); //limpio las banderas de cin. Colocarlas en estado de bueno
	  cin.ignore( 2000, '\n' ); // vacia el buffer de la entrada ya que contiene "basura"
	}
      else
	valor = true;
    } while ( cin.fail() || ( intento < 3 && valor == false ) );
  if ( intento == 3 )
    {
      cout << "Opcin 0 asignada, por la indecisin del jugador" << endl;
      opcion = 0;
    }
  else
    opcion -= 1;
  return opcion;
}

/**
 * El jugador deber decidir si quiere pagar el tanto por ciento de su capital o la cantidad fija
 * indicada. Esta decisin se toma al caer en un impuesto especial.
 */
char jugador::decidirImpuestoEspecial ( long cantidad, int tpc )
{
  char opcionPago;
  int intento = 0;
  do
    {
      if ( intento != 0 )
	cerr << "Opcin incorrecta. Selecciona un valor correcto (C,P)" << endl;
      cin >> opcionPago;
      cin.clear();
      cin.ignore( 2000, '\n' ); // vacia el buffer de la entrada ya que contiene "basura"
      intento++;      
    }
  while ( toupper( opcionPago ) != 'C' && toupper( opcionPago ) != 'P' && intento < 3 );
  if ( intento > 3 )
    {
      cout << "Ante la indecisin reinante en el ambiente, se ha decidido por t." << endl;
      opcionPago = 'C';
    }
  return opcionPago;
}

/**
 * Mtodo que devuelve el nmero de estaciones que posee el jugador.
 */
int jugador::poseeEstacion ()
{
  list <propiedad*>::iterator listaP; 
  int numEstaciones = 0;
  for ( listaP = _posee.begin(); listaP != _posee.end(); listaP++ )        
    {
      if ( dynamic_cast <estacion*> ( *listaP ) )
	numEstaciones ++;
    }
  return numEstaciones;
}

/**
 * Mtodo que devuelve el nmero de servicios que posee el jugador.
 */
int jugador::poseeServicio ()
{
  list <propiedad*>::iterator listaP; 
  int numServicios = 0;
  for ( listaP = _posee.begin(); listaP != _posee.end(); listaP++ )
    {
      if ( dynamic_cast <servicio*> ( *listaP ) )
	numServicios ++;
    }
  return numServicios;
}

/**
 * Mtodo que devuelve el nmero de calles edificables que posee el jugador.
 */
int jugador::poseeConstruibles ()
{
  list <propiedad*>::iterator listaP; 
  int numConstruibles = 0;
  
  // Recorre la lista de propiedades en busca de calles construibles (permiso_construccion).
  for ( listaP = _posee.begin(); listaP != _posee.end(); listaP++ )
    {
      if ( calle *c = dynamic_cast <calle*> ( *listaP ) )
	if ( c->get_permisoConstruccion() )
	  numConstruibles ++;
    }
  return numConstruibles;
}

/**
 * Aade el valor actual de cada atributo al vector de logs
 */
void jugador::actualizarLogs ()
{
  _logDinero.push_back ( get_dinero_total() );
  _logPropiedades.push_back ( _posee.size() );
  _logEdificios.push_back ( get_casasConstruidas() + get_hotelesConstruidos() );
  _logCapital.push_back ( get_capital() - get_dinero_total() );
  _logConstruibles.push_back ( poseeConstruibles () );
  return;
}

/**
 * Visualizar el log completo de dinero
 */
void jugador::verLogDinero ()
{
  for ( int i = 0; i != _logDinero.size(); i++ )
    cout << i << "\t" << _logDinero[i] << endl;
  return;
}

/**
 * Visualizar el log completo de capital
 */
void jugador::verLogCapital ()
{
  for ( int i = 0; i != _logCapital.size(); i++ )
    cout << i << "\t" << _logCapital[i] << endl;
  return;
}

/**
 * Visualizar el log completo de calles edificables
 */
void jugador::verLogConstruibles ()
{
  for ( int i = 0; i != _logConstruibles.size(); i++ )
    cout << i << "\t" << _logConstruibles[i] << endl;
  return;
}

/**
 * Visualizar el log completo de numero de propiedades
 */
void jugador::verLogPropiedades ()
{
  for ( int i = 0; i != _logPropiedades.size(); i++ )
    cout << i << "\t" << _logPropiedades[i] << endl;
  return;
}

/**
 * Visualizar el log completo de edificios construidos
 */
void jugador::verLogEdificios ()
{
  for ( int i = 0; i != _logEdificios.size(); i++ )
    cout << i << "\t" << _logEdificios[i] << endl;
  return;
}

/**
 * Mtodo que guarda todos los logs en los ficheros de registro.
 */
void jugador::guardarLogs ()
{
  //Se guardarn los cuatro vectores de log.
  string fichero;
  fichero = "logs/dinero-" + get_nombre () + "-" + get_color();
  ofstream fs ( fichero.c_str() );
  for ( int i = 0; i != _logDinero.size(); i++ )
    fs << i << "\t" << _logDinero[i] << endl;
  fs.close();

  fichero = "logs/capital-" + get_nombre () + "-" + get_color();
  ofstream fs2 ( fichero.c_str() );
  for ( int i = 0; i != _logCapital.size(); i++ )
    fs2 << i << "\t" << _logCapital[i] << endl;
  fs2.close();

  fichero = "logs/propiedades-" + get_nombre () + "-" + get_color();
  ofstream fs3 ( fichero.c_str() );
  for ( int i = 0; i != _logPropiedades.size(); i++ )
    fs3 << i << "\t" << _logPropiedades[i] << endl;
  fs3.close();

  fichero = "logs/edificios-" + get_nombre () + "-" + get_color();
  ofstream fs4 ( fichero.c_str() );
  for ( int i = 0; i != _logEdificios.size(); i++ )
    fs4 << i << "\t" << _logEdificios[i] << endl;
  fs4.close();

  fichero = "logs/construibles-" + get_nombre () + "-" + get_color();
  ofstream fs5 ( fichero.c_str() );
  for ( int i = 0; i != _logConstruibles.size(); i++ )
    fs5 << i << "\t" << _logConstruibles[i] << endl;
  fs5.close();
  return;
}

/**
 * Finaliza el progreso de todos los logs con valor 0.
 */
void jugador::cerrarLogs ()
{
  _logDinero.push_back ( 0 );
  _logPropiedades.push_back ( 0 );
  _logEdificios.push_back ( 0 );
  _logCapital.push_back ( 0 );
  _logConstruibles.push_back ( 0 );
  return;
}



// MTODOS NECESARIOS PARA CREAR LAS REGLAS DEL JUGADOR INTELIGENTE BASADO EN REGLAS

/**
 * Calcula la suma de la utilidad esperada de los alquileres de una lista de propiedades.
 */
long jugador::utilidadEsperadaAlquiler ( vector <propiedad*> propiedades, int n )
{  
  long total = 0;  
  for ( int i = 0; i != propiedades.size (); i ++ )    
    total += propiedades[i]->calcularAlquiler( this );  
  return ( total / n );
}

/**
 * Suma las hipotecas de todas las propiedades del vector de propiedades
 */
long jugador::sumaHipoteca ( vector <propiedad*> propiedades )
{
  long total = 0;
  for ( int i = 0; i != propiedades.size (); i ++ )    
    {      
      total += propiedades[i]->get_hipoteca();      
    }
  return total;
}

/** 
 * Imprime una lista de propiedades indicando adems su perdida estimada y el dinero que
 * se consigue con su hipoteca (venta).
 */
void jugador::imprimir ( vector <propiedad*> candidato  )
{  
  for ( int m = 0; m != candidato.size(); m++ )
    {     
      cout << candidato[m]->get_nombre() << ", ";
    }  
  cout << "=> perdida estimada: " << utilidadEsperadaAlquiler ( candidato, 1 );  
  cout << "          ( consigue " << sumaHipoteca ( candidato ) << ")" << endl; 
  return;
}

/**
 * Mtodo que une dos vectores de propiedades
 */
void jugador::unir ( vector <propiedad*> &v1, vector <propiedad*> v2 )
{ 
  for ( int i = 0; i < v2.size(); i++ )
    v1.push_back ( v2[ i ] );
  return;
}

/**
 * Algoritmo recursivo que busca la mejor alternativa de propiedades del jugador a hipotecar. Se debe conseguir
 * una cantidad de dinero superior a la variable cantidad. Las propiedades se introducen en el vector L. 
 * Index es la posicin a partir de la cual se comienza a buscar.
 */
vector <propiedad*> jugador::alternativasHipotecaImprimir ( long cantidad, vector <propiedad*> L, int index, bool imprime )
{
  vector <propiedad*> mejor;
  vector <propiedad*> aux;
  aux.clear();
  //Si ha llegado al final, no devuelve nada.
  if ( index == L.size ()  )
    {
      aux.clear();
      return aux;
    }

  // Si con la propiedad actual, es suficiente para cubrir la cantidad, se devuelve este nodo
  if ( cantidad <= L [index]->get_hipoteca() )
    {      
      aux.push_back ( L[ index ] );
      return aux;
    }  
  
  // Recorre la lista de propiedades buscando la mejor opcion
  for ( int i = index; i != L.size(); i++ )
    {
      aux.clear();
      aux.push_back ( L[ index ] );
      vector <propiedad*> recursivo;
      // Llama recursivamente al mtodo, actualizando la cantidad y comenzando a buscar en el sgt elemento.
      recursivo = alternativasHipotecaImprimir ( cantidad - L[ index ]->get_hipoteca(), L, i + 1, false );    
      unir ( aux, recursivo );
      //Control de factibilidad
      if ( sumaHipoteca ( aux ) >= cantidad )
	{
	  // Solo imprime en los casos correspondientes a la primera llamada, q son las alternativas completas
	  if ( imprime )
	    imprimir ( aux );	    
	  // Comprueba si es la mejor alternativa
	  if ( utilidadEsperadaAlquiler( aux, 1 ) < utilidadEsperadaAlquiler ( mejor, 1 ) || mejor.empty() )
	    {
	      mejor.clear();
	      mejor = aux;
	    }
	}
    }
  return mejor;
}

/**
 * Mtodo que busca las propiedades hipotecables y buscar la mejor alternativa para superar la cantidad 
 * minimizando las prdidas, usando el mtodo alternativasHipotecarImprimir.
 */
void jugador::calculoUtilidadEsperadaHipotecarImprimir ( long cantidad )
{
  cout << endl << endl << endl << endl << endl;
  cout << "CANTIDAD QUE SE NECESITA CONSEGUIR: " << cantidad << endl;

  //1- Copiar ordenadamente las propiedades hipotecables

  vector <propiedad*> propiedadesHipotecables;
  list <propiedad*>::iterator p; 
  for ( p = _posee.begin(); p != _posee.end(); p++ )
    {
      if ( (*p)->get_hipotecable() )
	//Insertar ordenado de mayor a menor hipoteca
	{
	  int h = 0; 
	  while ( h != propiedadesHipotecables.size () && propiedadesHipotecables[h]->get_hipoteca() > (*p)->get_hipoteca ()  )
	    {
	      h++;
	    }
	  propiedadesHipotecables.insert( propiedadesHipotecables.begin() + h, (*p) );
	}
    }

  int k = 0; 
  if ( propiedadesHipotecables.empty() )
    {
      cout << "No hay propiedades hipotecables, por lo que no existe una combinacin ptima" << endl;
      return;
    }
  cout << "PROPIEDADES HIPOTECABLES ( propiedad, dinero obtenido al hipotecar, perdida estimada ):" << endl;
  for ( k = 0; k != propiedadesHipotecables.size(); k++ )    
      cout << propiedadesHipotecables[k]->get_nombre() 
	   << " hipoteca: " << propiedadesHipotecables[k]->get_hipoteca () 
	   << " prdida: " << propiedadesHipotecables[k]->calcularAlquiler( this ) << endl;
  cout << endl << endl << endl;
  
  //2- Buscar la mejor alternativa
  vector <propiedad*> mejor;
  mejor.clear();
  
  for (int i = 0; i!=propiedadesHipotecables.size(); i++)
    {    
      vector <propiedad*> aux;      
      aux = alternativasHipotecaImprimir ( cantidad, propiedadesHipotecables, i, true );
      imprimir( aux );      
      if ( sumaHipoteca ( aux ) >= cantidad )
	if ( utilidadEsperadaAlquiler( aux, 1 ) < utilidadEsperadaAlquiler ( mejor, 1 ) || mejor.empty() )
	  {
	    mejor.clear();
	    mejor = aux;	    
	  }
    }  
  cout << "Alternativa recomendada: ";
  imprimir ( mejor );  
  return;
}

/**
 * Mtodo que une dos vectores de calles sobre el primero de ellos.
 */
void jugador::unirCalles ( vector <calle*> &v1, vector <calle*> v2 )
{ 
  for ( int i = 0; i < v2.size(); i++ )
    v1.push_back ( v2[ i ] );
  return;
}

/**
 * Mtodo que calcula la prdida que se estima que se ocasionar si se vende una serie de edificios
 * representada mediante un vector de calles.
 */
long jugador::perdidaEsperadaVenderEdificio ( calle *c, vector <calle*> L, int pos )
{
  //Cuento cuantos edificios se han incluido en la lista L
  int aparece = 0;
  for ( int i = 0; i != pos; i++ )
    if ( c == L[ i ] )
      aparece++;
  int hotel = c->get_numHoteles ();
  int casas = c->get_numCasas ();
  int edificios = 0;
  //Calcula los edificios que vende de cada calle
  if ( hotel == 1 )
    edificios = 5;
  else 
    edificios = casas; 
  edificios = edificios - aparece;

  //Devuelve el alquiler estimado que perder.
  switch ( edificios )
    {
    case 5:
      return c->get_alquilerHotel ();
      break;
    case 4:
      return c->get_alquilerCuatroCasas ();
      break;
    case 3:
      return c->get_alquilerTresCasas ();
      break;
    case 2:
      return c->get_alquilerDosCasas ();
      break;
    case 1: 
      return c->get_alquilerUnaCasa ();
      break;
    default:
      return -1;
    }
  return -1;
}

/**
 * Suma el beneficio obtenido por la venta de una serie de edificios identificados por un vector de calles
 */
long jugador::sumaVenta (  vector <calle*> candidato )
{
  long total = 0;
  for ( int i = 0; i != candidato.size (); i++ )
    total += (candidato [ i ]->get_precioCasa() / 2);
  return total;
}

/**
 * Calcula la utilidad esperada que se da al vender una serie de edificios representada por un vector de calles
 */
long jugador::utilidadEsperadaVender ( vector <calle*> L, vector <calle*> candidato )
{
  long total = 0;
  for ( int i = 0; i != candidato.size (); i++ )
    total += perdidaEsperadaVenderEdificio ( candidato[i], candidato, i );
  return total;
}

/**
 * Algoritmo recursivo que busca la mejor alternativa de para vender edificios de un jugador. Se debe conseguir
 * una cantidad de dinero superior a la variable cantidad. Las calles de donde se venden se introducen en el vector L. 
 * Index es la posicin a partir de la cual se comienza a buscar.
 */
vector <calle*> jugador::alternativasVenderImprimir ( long cantidad, vector <calle*> L, int index, bool imprime  )
{
  vector <calle*> mejor;
  vector <calle*> aux;
  aux.clear();
  //Si ha llegado al final de la lista, se devuelve vacio
  if ( index == L.size ()  )
    {
      aux.clear();
      return aux;
    }
  //Si se supera la cantidad con el nodo actual, se devuelve
  if ( cantidad <= ( L [index]->get_precioCasa() / 2 ) )
    {      
      aux.push_back ( L[ index ] );
      return aux;
    } 
  // Buscar las diferentes alternativas y se escoge la mejor
  for ( int i = index; i != L.size(); i++ )
    {
      aux.clear();
      aux.push_back ( L[ index ] );
      vector <calle*> recursivo;
      //Busca recursivamente
      recursivo = alternativasVenderImprimir ( cantidad - ( L[ index ]->get_precioCasa() / 2 ), L, i + 1, false );    
      unirCalles ( aux, recursivo );
      // Controla que se supere la cantidad
      if ( sumaVenta ( aux ) >= cantidad )
	{
	  if ( imprime )
	    {
	      //Controla que se pueda realizar la operacin segn las normas.
	      if ( controlFactibilidadVenta ( aux ) )
		cout << "FACTIBLE: ";
	      else
		cout << "INFACTIBLE; "; 
	      imprimirCalles ( L, aux );
	    }
	  //Controla que se pueda realizar la operacin segn las normas.
	  if ( controlFactibilidadVenta ( aux ) )
	    // Elige el mejor
	    if ( utilidadEsperadaVender ( L, aux ) < utilidadEsperadaVender ( L, mejor ) || mejor.empty() )
	      {
		mejor.clear();
		mejor = aux;
	      }
	}
    }
  return mejor;
}

/**
 * Imprime una lista de calles incluyendo su perdida estimada y el dinero que se consigue
 */
void jugador::imprimirCalles ( vector <calle*> original, vector <calle*> candidato  )
{  
  for ( int m = 0; m != candidato.size(); m++ )
    {     
      cout << candidato[m]->get_nombre() << ", ";
    }  
  cout << "=> perdida estimada: " << utilidadEsperadaVender ( original, candidato );  
  cout << "          ( consigue " << sumaVenta ( candidato ) << ")" << endl; 
  return;
}

/**
 * Mtodo que cuenta los edificios que pertenecen a una calle dentro de una lista de calles
 */
int jugador::contarEdificios (  vector <calle*> candidato, calle* c )
{
  int i = 0;
  while ( i != candidato.size() && candidato[i] != c )
    i++;
  
  int j = i;
  while ( j != candidato.size() && candidato[j] == c )
    j++;
  
  return j - i;
}

/**
 * Se debe comprobar que se cumplan las normas del monopoly:
 *  1- Construcciones escalonadas
 *  2- Casas / hoteles disponibles en la banca
 */
bool jugador::controlFactibilidadVenta ( vector <calle*> candidato )
{
  //set <propiedad*> get_monopolioCompleto (*propiedad);
  bool factible = true;
  //Comprobar para cada calle si cumple
  int i = 0;
  int casasNecesarias = 0;
  int casasLiberadas = 0;
  int numEdificiosFinales = 0;
  int numEdificios;

  while ( i != candidato.size () )
    {
      //Contar cuantas edificaciones se venden de esta calle. 
      //Si es un hotel debo indicarlo, ya que se deber almacenar las casas que se 
      //necesitarn para sustituirlo, ya que se deber comprobar que haya suficientes     
      numEdificios = contarEdificios ( candidato, candidato[i] );     

      if ( candidato[i]->get_numHoteles() == 1 )
	{
	  numEdificiosFinales = 5 - numEdificios;
	  casasNecesarias += numEdificiosFinales;
	  casasLiberadas += 4 - numEdificios;
	}
      else
	{
	  numEdificiosFinales = candidato[i]->get_numCasas() - numEdificios;
	  //casasNecesarias -= numEdificiosFinales;
	  casasLiberadas += numEdificios;
	}
      //Comrpuebo las edificaciones del resto del monopolio
      set <propiedad*> mp = candidato[i]->get_monopolioCompleto ( candidato[i] );
      set <propiedad*>::iterator itPropiedad;
      for ( itPropiedad = mp.begin(); itPropiedad != mp.end(); itPropiedad++ )
	{
	  if ( calle *c = dynamic_cast <calle*> (*itPropiedad) )	    
	    if ( c != candidato [i] )
	      {
		int numEdificiosOtro = contarEdificios ( candidato, c );
		int edificiosFinalesOtro;
		if ( c->get_numHoteles() == 1 )	    
		  edificiosFinalesOtro = 5 - numEdificiosOtro;	    
		else
		  edificiosFinalesOtro = c->get_numCasas() - numEdificiosOtro;	
		if ( ( edificiosFinalesOtro - numEdificiosFinales > 1 ) ||  ( edificiosFinalesOtro - numEdificiosFinales < -1 ) )
		  factible = false;
	      }
	}
      if ( numEdificios == 0 )
	i++;
      else
	i += numEdificios;
    }

  if ( casasNecesarias > ( tablero::obtenerCasasDisponibles() + casasLiberadas ) )
    factible = false;

  return factible;
}

/**
 * Mtodo que busca los edificios vendibles y buscar la mejor alternativa para superar la cantidad 
 * minimizando las prdidas, usando el mtodo alternativasVenderImprimir.
 * Una lista de edificios se representa como una lista de calles, cada aparicin de una calle, representa
 * un edificio.
 */
void jugador::calculoUtilidadEsperadaVenderImprimir ( long cantidad )
{
  cout << endl << endl << endl << endl << endl;
  cout << "CANTIDAD QUE SE NECESITA CONSEGUIR: " << cantidad << endl;

  //1- Copiar ordenadamente las calles vendibles en forma de calles

  vector <calle*> propiedadesVendibles;
  list <propiedad*>::iterator p; 
  for ( p = _posee.begin(); p != _posee.end(); p++ )
    {
      if ( calle *c = dynamic_cast <calle*> (*p) )
	{	  
	  int numHoteles = c->get_numHoteles ();
	  int numCasas = c->get_numCasas ();
	  int edificios;
	  if ( numHoteles == 1 )
	    edificios = 5;
	  else 
	    edificios = numCasas;
	  if ( edificios > 0 )
	    //Insertar ordenado de mayor a menor hipoteca
	    {
	      int h = 0;
	      while ( h != propiedadesVendibles.size () && propiedadesVendibles[h]->get_precioCasa() > c->get_precioCasa ()  )
		{
		  h++;
		}
	      propiedadesVendibles.insert( propiedadesVendibles.begin() + h, edificios, c );
	    }
	}
    }   
   
  if ( propiedadesVendibles.empty() )
    {
    cout << "No hay calles edificadas, por lo que no existe una combinacin ptima" << endl;
    return;
    }
  
  cout << "PROPIEDADES CONSTRUIDAS ( propiedad, dinero obtenido al hipotecar, perdida estimada ):" << endl;
  for ( int k = 0; k != propiedadesVendibles.size(); k++ )    
    cout << propiedadesVendibles[k]->get_nombre() 
	 << " venta de edificio: " << propiedadesVendibles[k]->get_precioCasa () / 2 
	 << " prdida: " << perdidaEsperadaVenderEdificio ( propiedadesVendibles[k], propiedadesVendibles, k )  << endl;
  cout << endl << endl << endl;
    
  //2- Buscar la mejor alternativa
  vector <calle*> mejor;
  mejor.clear();
  int i = 0;
  //Comenzar la bsqueda cada vez por un nodo distitno y se seleccionara la mejor alternativa
  //que alcance la cantidad indicada
  while (i != propiedadesVendibles.size() )
    {    
      vector <calle*> aux;      
      aux = alternativasVenderImprimir ( cantidad, propiedadesVendibles, i, true );
      imprimirCalles ( propiedadesVendibles, aux );      
      if ( sumaVenta ( aux ) >= cantidad )
	if ( controlFactibilidadVenta ( aux ) )
	  if ( utilidadEsperadaVender( propiedadesVendibles, aux ) < utilidadEsperadaVender ( propiedadesVendibles, mejor ) 
	       || mejor.empty() )
	    {
	      mejor.clear();
	      mejor = aux;	    
	    }
      int j = i;
      while ( propiedadesVendibles[j] == propiedadesVendibles[i] && j != propiedadesVendibles.size() )
	j++;
      if ( j == i )
	i++;
      else
	i = j;
    }  
  
  if ( mejor.empty() )
    mejor = propiedadesVendibles;
  cout << endl << "Eleccin recomendada: ";
  imprimirCalles ( propiedadesVendibles, mejor );  

  return;
}

/**
 * Suma todos los valores de deshipoteca de un vector de propiedades
 */
long jugador::sumaDeshipoteca ( vector <propiedad*> propiedades )
{  
  long total = 0; 
  for ( int i = 0; i != propiedades.size (); i ++ )    
    total += propiedades[i]->get_deshipoteca();    
  return total;
}

/**
 * Algoritmo recursivo que busca la mejor alternativa de deshipotecas de propiedades. La cantidad mxima que se 
 * puede invertir es la cantidad que se marca en la variable 'cantidad'. La lista contiene todas las propiedades
 * hipotecadas ordenadas por precio de deshipoteca.
 */
vector <propiedad*> jugador::alternativasDeshipotecarImprimir ( long cantidad, vector <propiedad*> L, int index, bool imprime )
{
  vector <propiedad*> mejor;
  mejor.clear ();
  vector <propiedad*> aux;
  aux.clear(); 
  //Si alcanza el final de la lista, devuelve falso
  if ( index == L.size ()  )
    {
      aux.clear();
      return aux;
    }
  
  //Si no existe una posible inversin con el dinero que queda
  if ( cantidad < L [ L.size() - 1 ]->get_deshipoteca() )
    {      
      aux.push_back ( L[ index ] );
      return aux;
    } 
 
  for ( int i = index; i != L.size(); i++ )
    {       
      aux.clear();
      aux.push_back ( L[ index ] );
      vector <propiedad*> recursivo;            
      recursivo = alternativasDeshipotecarImprimir ( cantidad - L[ index ]->get_deshipoteca(), L, i + 1, false );
      unir ( aux, recursivo );
      if ( sumaDeshipoteca ( aux ) < cantidad )
	{
	  if ( imprime )
	    imprimir ( aux );	 
	  if ( utilidadEsperadaAlquiler( aux, 1 ) > utilidadEsperadaAlquiler ( mejor, 1 ) )
	    {
	      mejor.clear();
	      mejor = aux;
	    }
	}
    }
  return mejor;
}

/**
 * Mtodo que busca las propiedades hipotecadas y buscar la mejor combinacin para deshipotecarlas sin
 * superar la cantidad indicada en el atributo cantidad. Utilizar el mtodo alternativasDeshipotecarImprimir.
 */
void jugador::calculoUtilidadEsperadaDeshipotecarImprimir ( long cantidad )
{
  cout << endl << endl << endl << endl << endl;
  cout << "CANTIDAD QUE SE QUIERE INVERTIR: " << cantidad << endl;


  //1- Copiar ordenadamente las propiedades hipotecables
  vector <propiedad*> propiedadesHipotecadas;
  list <propiedad*>::iterator p; 
  for ( p = _posee.begin(); p != _posee.end(); p++ )
    {
      if ( (*p)->get_hipotecado() )
	//Insertar ordenado de mayor a menor hipoteca
	{
	  int h = 0;
	  while ( h != propiedadesHipotecadas.size () && propiedadesHipotecadas[h]->get_deshipoteca() > (*p)->get_deshipoteca ()  )
	    {
	      h++;
	    }
	  propiedadesHipotecadas.insert ( propiedadesHipotecadas.begin() + h, (*p) );
	}
    }    
  
  if ( propiedadesHipotecadas.empty() )
    {
      cout << "No hay calles hipotecadas, por lo que no existe una combinacin ptima" << endl;
      return;
    }
  
  cout << "PROPIEDADES HIPOTECADAS ( propiedad, dinero necesario para deshipotecar, beneficio estimado ):" << endl;
  for ( int k = 0; k != propiedadesHipotecadas.size(); k++ )    
    cout << propiedadesHipotecadas[k]->get_nombre() 
	 << " dinero necesario: " << propiedadesHipotecadas[k]->get_deshipoteca () 
	 << " beneficio: " << propiedadesHipotecadas [k]->calcularAlquiler ( this )   << endl;
  cout << endl << endl << endl;
    
  //2- Buscar la mejor alternativa
  vector <propiedad*> mejor;
  mejor.clear();
  
  for (int i = 0; i != propiedadesHipotecadas.size(); i++)
    {    
      vector <propiedad*> aux;      
      cout << "entra!" << endl;
      aux = alternativasDeshipotecarImprimir ( cantidad, propiedadesHipotecadas, i, true );
      cout << "2" << endl;
      imprimir( aux );
      if ( sumaHipoteca ( aux ) < cantidad )
	if ( utilidadEsperadaAlquiler( aux, 1 ) > utilidadEsperadaAlquiler ( mejor, 1 ) )
	  {	    
	    mejor.clear();
	    mejor = aux;	    
	  }
    }

  if ( mejor.empty() )
    cout << "No hay alternativas para deshipotecar" << endl;
  else
    cout << "Mejor alternativa: ";
  imprimir ( mejor );
  return;
}

/**
 * Imprime una lista de calles incluyendo su beneficio estimado al construir y la inversin que se debe realizar
 */
void jugador::imprimirCallesConstruir ( vector <calle*> original, vector <calle*> candidato  )
{  
  for ( int m = 0; m != candidato.size(); m++ )
    {     
      cout << candidato[m]->get_nombre() << ", ";
    }  
  cout << "=> beneficio estimado: " << utilidadEsperadaConstruir ( original, candidato );
  cout << "          ( invierte " << sumaConstruccion ( candidato ) << ")" << endl; 
  return;
}

/**
 * Suma el precio de construir los edificios indicados en la variable candidato.
 */
long jugador::sumaConstruccion (  vector <calle*> candidato )
{
  long total = 0;
  for ( int i = 0; i != candidato.size (); i++ )
    {
      //cout << "total" << total << endl;; 
      total += (candidato [ i ]->get_precioCasa() );
    }
  return total;
}

/**
 * Mtodo que calcula el beneficio esperado al construir un edificio en la calle c, teniendo
 * una lista de edificios construibles
 */
long jugador::beneficioEsperadoConstruirEdificio ( calle* c, vector <calle*> L, int pos )
{
  //Cuento cuantos edificios se han incluido en la lista L
  int aparece = 0;
  
  for ( int i = 0; i != pos; i++ )
    {   
      if ( c == L[ i ] )
	aparece++;
    }    
  int casas = c->get_numCasas ();
  int edificios = casas + aparece + 1;
  switch ( edificios )
    {
    case 5:
      return c->get_alquilerHotel ();
      break;
    case 4:
      return c->get_alquilerCuatroCasas ();
      break;
    case 3:
      return c->get_alquilerTresCasas ();
      break;
    case 2:
      return c->get_alquilerDosCasas ();
      break;
    case 1: 
      return c->get_alquilerUnaCasa ();
      break;
    case 0:
      return c->get_alquilerBase() * 2;
      break;
    default:
      return -1;
    }
  return -1;
}

/**
 * Mtodo que calcula la utilidad esperada de construir una serie de edificios indicados en la lista candidato.
 */
long jugador::utilidadEsperadaConstruir ( vector <calle*> L, vector <calle*> candidato )
{
  long total = 0;
  for ( int i = 0; i != candidato.size (); i++ )
    {
      total += beneficioEsperadoConstruirEdificio ( candidato[i], candidato, i );
    }
  return total;
}

/**
 * Comprueba que una alternativa a construir cumple con las normas de Monopoly en el mbito de la edificacin
 */
bool jugador::controlFactibilidadConstruir ( vector <calle*> candidato )
{
  bool factible = true;
  //Comprobar para cada calle si cumple
  int i = 0;
  int casasNecesarias = 0;
  int hotelesNecesarios = 0;
  int numEdificiosFinales = 0;
  int numEdificios;

  while ( i != candidato.size () )
    {
      //Contar cuantas edificaciones se construyen en esta calle. 
      //Si se alcanza un hotel, se deber indicar con el objetivo de comprobar la
      //disponibilidad en la banca         
      numEdificios = contarEdificios ( candidato, candidato[i] );      
      if ( candidato[i]->get_numCasas() + numEdificios == 5 )
	{
	  numEdificiosFinales = 5;
	  hotelesNecesarios ++;
	}
      else
	{
	  numEdificiosFinales = candidato[i]->get_numCasas() + numEdificios;
	  casasNecesarias += numEdificios;
	}
      //Comrpuebo las edificaciones del resto del monopolio
      set <propiedad*> mp = candidato[i]->get_monopolioCompleto ( candidato[i] );
      set <propiedad*>::iterator itPropiedad;
      for ( itPropiedad = mp.begin(); itPropiedad != mp.end(); itPropiedad++ )
	{
	  if ( calle *c = dynamic_cast <calle*> (*itPropiedad) )	    
	    if ( c != candidato [i] )
	      {
		int numEdificiosOtro = contarEdificios ( candidato, c );
		int edificiosFinalesOtro;
		edificiosFinalesOtro = c->get_numCasas() + numEdificiosOtro;	
		if ( ( edificiosFinalesOtro - numEdificiosFinales > 1 ) ||  ( edificiosFinalesOtro - numEdificiosFinales < -1 ) )
		  factible = false;
	      }
	}          
      i += numEdificios;
    }

  if ( casasNecesarias > tablero::obtenerCasasDisponibles() || hotelesNecesarios > tablero::obtenerHotelesDisponibles() )
    factible = false;

  return factible;
}

/**
 * Mtodo que busca todas las alternativas para construir de una lista de calles y devuelve la mejor alternativa.
 * Un edificio se representa en la lista por la calle en la que se construir.
 */
vector <calle*> jugador::alternativasConstruirImprimir ( long cantidad, vector <calle*> L, int index, bool imprime )
{
  vector <calle*> mejor;
  mejor.clear ();
  vector <calle*> aux;
  aux.clear(); 
  
  // Si ha llegado al final de la lista, devuelve falso
  if ( index == L.size ()  )
    {
      aux.clear();
      return aux;
    }
  
  //Si con cualquier nodo se sobrepasa el precio devuelve el nodo actual
  if ( cantidad < L [ L.size() - 1 ]->get_precioCasa () )
    {      
      aux.push_back ( L[ index ] );
      return aux;
    } 
 
  for ( int i = index; i != L.size(); i++ )
    {  
      aux.clear();
      aux.push_back ( L[ index ] );
      vector <calle*> recursivo;
      recursivo = alternativasConstruirImprimir ( cantidad - L[ index ]->get_precioCasa(), L, i + 1, false );
      unirCalles ( aux, recursivo );
      //Control de factibilidad
      if ( controlFactibilidadConstruir ( aux ) )     
	if ( sumaConstruccion ( aux ) < cantidad )
	  {
	    if ( imprime )
	      if ( aux.empty() )		  
		imprimirCallesConstruir ( L, aux );

	    // Mira si es la mejor opcin
	    if ( utilidadEsperadaConstruir ( L, aux ) > utilidadEsperadaConstruir ( L, mejor ) )
	      {	
		mejor.clear();
		mejor = aux;
	      }
	  }
    }
  return mejor;
}

/**
 * Mtodo que usando el metodo alternativasConstruirImprimir busca la mejor alternativa para construir sin 
 * sobrepasar la cantidad indicada. Iterativamente buscar la mejor opcin indicando como inicio cada vez
 * un nodo de la lista.
 * La lista de calles debe estar ordenada de mayor a menor precio de edificio.
 */
void jugador::calculoUtilidadEsperadaConstruirImprimir ( long cantidad )
{
  cout << endl << endl << endl << endl << endl;
  cout << "CANTIDAD QUE SE QUIERE INVERTIR: " << cantidad << endl;

  //1- Copiar ordenadamente las propiedades edificables

  vector <calle*> propiedadesConstruibles;
  list <propiedad*>::iterator p; 
  for ( p = _posee.begin(); p != _posee.end(); p++ )
    {
      if ( calle *c = dynamic_cast <calle*> (*p) )
	{	  
	  int numHoteles = c->get_numHoteles ();
	  int numCasas = c->get_numCasas ();
	  int edificios;
	  if ( numHoteles == 1 )
	    edificios = 0;
	  else 
	    edificios = 5 - numCasas;
	  if ( edificios > 0 && c->get_permisoConstruccion() )
	    //Insertar ordenado de mayor a menor precio de edificio
	    {
	      int h = 0;
	      while ( h != propiedadesConstruibles.size () && propiedadesConstruibles[h]->get_precioCasa() > c->get_precioCasa ()  )
		{
		  h++;
		}
	      propiedadesConstruibles.insert( propiedadesConstruibles.begin() + h, edificios, c );
	    }
	}
    }   
   
  if ( propiedadesConstruibles.empty() )
    {
    cout << "No se puede edificar ninguna calle, por lo que no existe una combinacin ptima" << endl;
    return;
    }
  cout << "PROPIEDADES CONSTRUIBLES ( propiedad, dinero obtenido al hipotecar, perdida estimada ):" << endl;
  for ( int k = 0; k != propiedadesConstruibles.size(); k++ )    
    cout << propiedadesConstruibles[k]->get_nombre() 
	 << " construir edificio: " << propiedadesConstruibles[k]->get_precioCasa ()  
	 << " beneficio: " << beneficioEsperadoConstruirEdificio ( propiedadesConstruibles[k], propiedadesConstruibles, k )  << endl;
    
  cout << endl << endl << endl;
  
  //2- Buscar la mejor alternativa
  vector <calle*> mejor;
  mejor.clear();
  
  int i = 0;
  
  //Entra cada vez iniciando la alternativa con una calle distinta
  while (i != propiedadesConstruibles.size() )
    {    
      vector <calle*> aux;            
      //Busca la mejor alternativa
      aux = alternativasConstruirImprimir ( cantidad, propiedadesConstruibles, i, true );     
      if ( !aux.empty() )
	imprimirCallesConstruir ( propiedadesConstruibles, aux );
      //Controla la factibilidad de la solucin
      if ( sumaConstruccion ( aux ) < cantidad )
	if ( controlFactibilidadConstruir ( aux ) )     
	  {	   
	    //Compara si es la mejor solucin
	    if (utilidadEsperadaConstruir( propiedadesConstruibles, aux ) > utilidadEsperadaConstruir ( propiedadesConstruibles, mejor ) )
	      {	
		mejor.clear();
		mejor = aux;	    
	      }
	  }
      int j = i;
      //Avanza hasta la siguiente calle en la lista
      while ( propiedadesConstruibles[j] == propiedadesConstruibles[i] && j != propiedadesConstruibles.size() )
	j++;
      if ( j == i )
	i++;
      else
	i = j;
    }

  if ( mejor.empty() )
    cout << "No hay alternativas para construir" << endl;
  else
    cout << "Mejor alternativa: ";
  imprimirCallesConstruir ( propiedadesConstruibles, mejor );  
  
  return;    
}
