/***********************************************************************************
 *  CLASE MAZO
 *  Clase que representa un mazo de tarjetas. Un mazo estar compuesto por un nmero
 *  indeterminado de tarjetas. 
 *  Los jugadores recogeran una tarjeta del mazo correspondiente.
 *
 *  Requiere:
 *     - libreras de C++ (fstream, string, deque, iostream, ctime, cstdlib)
 *     - clase tarjeta
 *     - clase mtrand (algoritmo Merssene Twister para generar numeros aleatorios)     
 *
 *  Autor: Javier Aragn Zabalegui (PFC para la uc3m: street Master's)
 *  Contacto: 100039831@alumnos.uc3m.es
 *  version 1.0
 **********************************************************************************/

#ifndef __MAZO__
#define __MAZO__

#include "cadenas.h"
#include "configuracion.h"
#include "tarjeta.h"

#include "mtrand.h"

#include <fstream>
#include <string>
#include <deque>
#include <iostream>
#include <ctime>
#include <cstdlib>
using namespace std;

class mazo{

 private:
  //Atributos
  string _nombre;
  
  //Asociaciones
  deque <tarjeta*> _contiene;  //Tarjetas que forman el mazo
 public:

  // Constructor por defecto
  mazo ();
  
  // Constructor explcito
  mazo ( string nombre );
  
  // Copy constructor
  mazo (const mazo& right);

  // Operador de asignacin
  mazo& operator= (const mazo& right);

  // Destructor
  ~mazo ();
  
  // Mtodos GET
  string get_nombre () const;
  
  // Mtodos SET
  void set_nombre ( string nombre );
  
  // Otros mtodos
  
  /**
   * Mtodo que carga las tarjetas de un mazo desde el fichero indicado
   */
  int cargar_tarjetas ( string ficheroTarjetas, string tipo );
  
  /**
   * Se genera una posicin aleatoria en la que insertar la tarjeta actual y se colocar
   * en dicha posicin simulando un barajeo de cartas.
   */
  int anyadirTarjeta ( string nombre, int numero, string tipo );
  
  /**
   * Se aade una accin a una tarjeta ya creada que contenga los datos pasados como parmetro.
   * Se aadir al final de la lista de acciones.
   */
  int anyadirAccion ( int numero, int pay, int advance, int advanceto, int payeach, 
		      int payhouse, int payhotel, int multiPago, bool nextutil, bool tojail,
		      bool nextrr, bool jailcard, bool cobrarSalida, int multiPagoDado, 
		      bool nextPropiedad );

  /**
   * Comprueba que exista una tarjeta con un nmero dado (identificativo) en el mazo
   */
  bool existeTarjeta ( int numero );

  /**
   * El jugador actual recoge la tarjeta del mazo y se invocar a la accin de la tarjeta.
   * La instancia de la tarjeta se encargar de gestionar la accin
   */
  int cogerTarjeta ( jugador *j );

  /**
   * El jugador devuelve la tarjeta al mazo en el caso de que se trate de una tarjeta de crcel que
   * el jugador s puede poseer. Se coloca al final del mismo.
   */
  int devolverTarjeta ( jugador *j );

  /**
   * Devuelve la tarjeta del mazo que se indica con el nmero que se pasa como argumento del mtodo
   */
  tarjeta *obtenerTarjeta ( int numero );

  /**
   * Elimina del mazo la tarjeta identificada por el nmero pasado como argumento
   */
  int eliminarTarjeta ( int numero );

};

inline string mazo::get_nombre () const
{
  return _nombre;
}

inline void mazo::set_nombre ( string nombre )
{
  _nombre = nombre;
}

#endif
