/**********************************************************************************
				monopolio.cpp
	
Programa: 	31
Nombre:		Javier Aragn Zabalegui		100039831@alumnos.uc3m.es
Fecha_inicio:	28.Marzo.2006
Descripcion:	Clase monopolio para Street Master's
			
Comentarios:	
**********************************************************************************/

/**********************************************************************************
Listado de contenidos:
	include "monopolio.h"

**********************************************************************************/

#include "monopolio.h"
#include "propiedad.h"
#include "calle.h"

// Constructor por defecto
monopolio::monopolio ()
  : _color ( "" ),
    _numPropiedades ( 0 ),
    _colorAbreviado ("")
{
  // No se hace nada!
}

// constructor explcito
monopolio::monopolio ( string color, int numPropiedades )
  : _color ( color ),
    _numPropiedades ( numPropiedades )
{  
  _colorAbreviado = abreviar ( color, 2 );
}
 
// Copy constructor
monopolio::monopolio (const monopolio& right)
{
  _color		= right._color;
  _numPropiedades	= right._numPropiedades;
  _colorAbreviado	= right._colorAbreviado;
}

// Assignment operator
monopolio& monopolio::operator= (const monopolio& right)
{
  _color		= right.get_color ();
  _numPropiedades	= right.get_numPropiedades ();
  _colorAbreviado	= right._colorAbreviado;
  return ( *this );
}

// Destructor
monopolio::~monopolio ()
{
  // No se hace nada!
}

 /**
  * Aade una propiedad al conjunto de propiedades que pertenecen al monopolio
  */
void monopolio::anyadirPropiedad ( propiedad* nuevaPropiedad )
{
  _contienePropiedad.insert ( nuevaPropiedad );
}

/**
 * Mtodo que comprueba si el monopolio completo pertenece al mismo jugador
 */
bool monopolio::monopolioCompleto ()
{   
  bool ok = false;  
  propiedad* primerMiembro;
  primerMiembro = *_contienePropiedad.begin();
  //Si las propiedades que pertenecen al mismo dueo que tiene el primer elemento del
  //conjunto es igual que el total de propiedades, entonces es un monopolio completo.
  if ( contarPropiedadesConMismoPropietario ( primerMiembro ) == get_numPropiedades () )
    {
      ok = true;
      cout << "Excelente, dispones de un monopolio completo listo para cobrar!" << endl;
    }
  else
    ok = false;
  return ok;
}

/**
 * Mtodo que cuenta cuantas propiedades del monopolio pertencen al mismo propietario
 */
int monopolio::contarPropiedadesConMismoPropietario ( propiedad *prop )
{
  set <propiedad*>::iterator p;
  int numPropiedades = 0;
  // Recorre la lista comprobando coincidencias
  for ( p = _contienePropiedad.begin(); p != _contienePropiedad.end(); p++ )
    {
      if ( (*p)->get_hipotecado() == false )
	if ( (*p)->mismoPropietario ( prop ) )
	  numPropiedades++;
    }
  return numPropiedades;
}

/**
 * Comprueba si alguna propiedad de tipo calle del monopolio tiene un edificio construido
 */
bool monopolio::existeConstruido ()
{
  bool construido = false;
  set <propiedad*>::iterator p;  
  //Recorre el conjunto comprobando si es calle y, si es calle, si tiene edificios
  for ( p = _contienePropiedad.begin(); p != _contienePropiedad.end(); p++ )
    {
      if ( calle *c = dynamic_cast<calle *> ( *p ) )
	if ( c->get_numCasas() > 0 || c->get_numHoteles() )
	  construido = true;      
    }
  return construido;
}

/**
 * Pone el valor del atributo _permisoConstruccion de todas las calles del monopolio a verdadero
 */
void monopolio::permitirConstruir ()
{
  set <propiedad*>::iterator p;
  for ( p = _contienePropiedad.begin(); p != _contienePropiedad.end(); p++ )    
    if ( calle *c = dynamic_cast<calle *> ( *p ) )
      c->set_permisoConstruccion ( true );
  return ;
}

/**
 * Pone el valor del atributo _permisoConstruccion de todas las calles del monopolio a falso
 */
void monopolio::prohibirConstruir ()
{
  set <propiedad*>::iterator p;
  for ( p = _contienePropiedad.begin(); p != _contienePropiedad.end(); p++ )
    if ( calle *c = dynamic_cast<calle *> ( *p ) )       
      c->set_permisoConstruccion ( false );    
  return ;
}

/**
 * Pone todo el monopolio con el estado hipotecable como indique el argumento
 */
void monopolio::hipotecable ( bool permitido )
{
  set <propiedad*>::iterator p;
  for ( p = _contienePropiedad.begin(); p != _contienePropiedad.end(); p++ )    
    (*p)->set_hipotecable ( permitido );
  return ;
}

/**
 * Devuelve el nmero de casas que tiene la calle ms construida
 */
int monopolio::maxConstruido ()
{
  int actual = 0;
  int max = 0;
  set <propiedad*>::iterator p;

  // Recorre todo el conjunto de calles comparando en busca de la que ms edificios tiene
  for ( p = _contienePropiedad.begin(); p != _contienePropiedad.end(); p++ )
    if ( calle *c = dynamic_cast<calle *> ( *p ) )
      {
	actual = c->get_numCasas();
	if ( c->get_numHoteles() > 0 )
	  actual = 5;
	if ( actual > max )
	  max = actual;
      }
  return max;
}

/**
 * Devuelve el nmero de casas que tiene la calle menos construida
 */
int monopolio::minConstruido ()
{
  int actual = 0;
  int min = 100;
  set <propiedad*>::iterator p;

  // Recorre todo el conjunto de calles comparando en busca de la que ms edificios tiene
  for ( p = _contienePropiedad.begin(); p != _contienePropiedad.end(); p++ )
    if ( calle *c = dynamic_cast<calle *> ( *p ) )
      {
	actual = c->get_numCasas();
	if ( c->get_numHoteles() > 0 )
	  actual = 5;
	if ( actual < min )
	  min = actual;
      }
  return min;
}

/**
 * Comprueba si una calle se corresponde con la mas construida
 */
bool monopolio::esCalleMasConstruida ( calle *c )
{
  bool maximo;
  int max = maxConstruido ();
  int construido = 0;

  //Obtiene los edificios construidos
  if ( c->get_numCasas () > 0 )
    construido = c->get_numCasas();
  else if ( c->get_numHoteles () > 0 )
    construido = 5;

  //Comprueba si es mximo
  if ( construido == max )
    maximo = true;
  else
    maximo = false;
  return maximo;
}

/**
 * Comprueba si un monopolio completo es edificable comprobando el estado de todas las calles
 * del mismo.
 */
bool monopolio::construible ()
{  
  bool construir = true;
  if ( monopolioCompleto() )
    {
       set <propiedad*>::iterator p;
       //Recorre el conjunto mirando si esta hipotecado, si lo est no se puede construir
       for ( p = _contienePropiedad.begin(); p != _contienePropiedad.end(); p++ )
	 if ( (*p)->get_hipotecado () == true )
	   construir = false;
    }
  else
    construir = false;
  return construir;
}

/**
 * Comrpueba si todas las propiedades del monopolio estn sin dueo
 */
bool monopolio::estaLibre ()
{
  propiedad* primerMiembro;
  primerMiembro = *_contienePropiedad.begin();
  //Si todas las propiedades son del mismo dueo que la primera y el dueo es la banca,
  //entonces est libre
  if ( contarPropiedadesConMismoPropietario ( primerMiembro ) == get_numPropiedades () )
    if ( dynamic_cast <banca *> ( primerMiembro->get_esPropiedad() ) )
      return true;
  return false;
}

/**
 * Comprueba si es la ltima propiedad del monopolio para un jugador en concreto. Esto implica
 * que posee el resto de propiedades.
 */
bool monopolio::ultimaPropiedad ( jugador *j )
{
  int esPropiedadJugador = 0;
  set <propiedad*>::iterator p;
  for ( p = _contienePropiedad.begin(); p != _contienePropiedad.end(); p++ )
    {
      propietario *prop = (*p)->get_esPropiedad();
      if ( jugador *j2 = dynamic_cast <jugador *> (prop) )
	if ( j2 == j )
	  esPropiedadJugador ++;
    }
  if ( esPropiedadJugador == get_numPropiedades() - 1 )
    return true;
  else
    return false;
}

/**
 * Comrpueba si el jugador j es el nico que posee alguna propiedad del monopolio.
 */
bool monopolio::unicoPoseedor ( jugador *j )
{
  int esPropiedadOtroJugador = 0;
  int esPropiedadJugador = 0;
  set <propiedad*>::iterator p;

  //Recorre el conjunto comprobando cual es el poseedor. Sumar las coincidencias
  //y las propiedades de otros jugadores
  for ( p = _contienePropiedad.begin(); p != _contienePropiedad.end(); p++ )
    {
      propietario *prop = (*p)->get_esPropiedad();
      if ( jugador *j2 = dynamic_cast <jugador *> (prop) )
	if ( j2 == j )
	  esPropiedadJugador ++;
	else
	  esPropiedadOtroJugador ++;
    }
  if ( esPropiedadJugador > 0 && esPropiedadOtroJugador == 0 )
    return true;
  return false;
}

/** 
 * Comprueba si un jugador posee parte de un monopolio.
 */
bool monopolio::poseeParte ( jugador *j )
{
  int esPropiedadJugador = 0;
  set <propiedad*>::iterator p;

  //Comprueba todos los dueos de las propiedades. Si coincide alguna ser verdadero
  for ( p = _contienePropiedad.begin(); p != _contienePropiedad.end(); p++ )
    {
      propietario *prop = (*p)->get_esPropiedad();
      if ( jugador *j2 = dynamic_cast <jugador *> (prop) )
	if ( j2 == j )
	  esPropiedadJugador ++;
    }
  if ( esPropiedadJugador > 0 )
    return true;
  return false;
}

/**
 * Comprueba si es la ltima propiedad del monopolio para un jugador distinto al pasado como argumento.
 */
bool monopolio::ultimaPropiedadOtroJugador ( jugador *j )
{
  int esPropiedadOtroJugador = 0;
  set <propiedad*>::iterator p;
  for ( p = _contienePropiedad.begin(); p != _contienePropiedad.end(); p++ )
    {
      propietario *prop = (*p)->get_esPropiedad();
      if ( jugador *j2 = dynamic_cast <jugador *> (prop) )
	if ( j2 != j )
	  esPropiedadOtroJugador ++;
    }
  if ( esPropiedadOtroJugador == get_numPropiedades() - 1 )
    return true;
  else
    return false;
}

