/***********************************************************************************
 *  CLASE MONOPOLIO
 *  La clase monopolio representa un grupo de propiedades que tienen el mismo color
 *  (o tipo en el caso de las estaciones y servicios). Cuando un monopolio de propiedades
 *  pertenecen a un mismo jugador, estas propiedades adquieren propiedades especiales.
 *
 *  Requiere:
 *     - clase jugador
 *     - clase iostream
 *     - clase string
 *     - clase set
 *
 *  Autor: Javier Aragn Zabalegui (PFC para la uc3m: street Master's)
 *  Contacto: 100039831@alumnos.uc3m.es
 *  version 1.0
 **********************************************************************************/

#ifndef __MONOPOLIO__
#define __MONOPOLIO__

#include "cadenas.h"
#include "jugador.h"

#include <iostream>
#include <string>
#include <set>
using namespace std;

class calle;      // Forward declaration
class propiedad;  // Forward declaration

class monopolio {

 private:
  //Atributos
  string _color;
  int _numPropiedades;
  string _colorAbreviado;

  //Asociaciones
  set <propiedad*> _contienePropiedad;
  

 public:

  // Constructor por defecto
  monopolio ();
  
  // Constructor explcito
  monopolio ( string color, int numPropiedades );
  
  // Copy constructor
  monopolio (const monopolio& right);

  // Operador de asignacin
  monopolio& operator= (const monopolio& right);

  // Destructor
  ~monopolio ();

  // Mtodos GET
  string get_color () const;
  int get_numPropiedades () const;
  string get_colorAbreviado () const;
  set <propiedad*> get_contienePropiedad ();

  // Mtodos SET
  void set_color ( string color );
  void set_numPropiedades ( int numPropiedades );
  void set_colorAbreviado ( string colorAbr );

  //Otros mtodos

  /**
   * Aade una propiedad al conjunto de propiedades que pertenecen al monopolio
   */
  void anyadirPropiedad ( propiedad* nuevaPropiedad );

  /**
   * Mtodo que comprueba si el monopolio completo pertenece al mismo jugador
   */
  bool monopolioCompleto ();

  /**
   * Mtodo que cuenta cuantas propiedades del monopolio pertencen al mismo propietario
   */
  int contarPropiedadesConMismoPropietario ( propiedad *prop );

  /**
   * Comprueba si alguna propiedad de tipo calle del monopolio tiene un edificio construido
   */
  bool existeConstruido ();

  /**
   * Pone el valor del atributo _permisoConstruccion de todas las calles del monopolio a verdadero
   */
  void permitirConstruir ();

  /**
   * Pone el valor del atributo _permisoConstruccion de todas las calles del monopolio a falso
   */
  void prohibirConstruir ();

  /**
   * Pone todo el monopolio con el estado hipotecable como indique el argumento
   */
  void hipotecable ( bool permitido );

  /**
   * Devuelve el nmero de casas que tiene la calle ms construida
   */
  int maxConstruido ();
  
  /**
   * Devuelve el nmero de casas que tiene la calle menos construida
   */
  int minConstruido ();
  
  /**
   * Comprueba si una calle se corresponde con la mas construida
   */
  bool esCalleMasConstruida ( calle *c );

  /**
   * Comprueba si un monopolio completo es edificable comprobando el estado de todas las calles
   * del mismo.
   */
  bool construible ();

  /**
   * Comrpueba si todas las propiedades del monopolio estn sin dueo
   */
  bool estaLibre ();

  /**
   * Comprueba si es la ltima propiedad del monopolio para un jugador en concreto. Esto implica
   * que posee el resto de propiedades.
   */
  bool ultimaPropiedad ( jugador *j );

  /**
   * Comrpueba si el jugador j es el nico que posee alguna propiedad del monopolio.
   */
  bool unicoPoseedor ( jugador *j );

  /** 
   * Comprueba si un jugador posee parte de un monopolio.
   */
  bool poseeParte ( jugador *j );

  /**
   * Comprueba si es la ltima propiedad del monopolio para un jugador distinto al pasado como argumento.
   */
  bool ultimaPropiedadOtroJugador ( jugador *j );
};

inline string monopolio::get_color () const
{
  return _color;
}

inline int monopolio::get_numPropiedades () const
{
  return _numPropiedades;
}

inline string monopolio::get_colorAbreviado () const
{
  return _colorAbreviado;
}

inline set <propiedad*> monopolio::get_contienePropiedad ()
{
  return _contienePropiedad;
}


/*inline calle monopolio::get_contieneCalle () const
{
  return _contieneCalle;
  }*/

inline void monopolio::set_color ( string color )
{
  _color = color;
}

inline void monopolio::set_numPropiedades ( int  numPropiedades )
{
  _numPropiedades = numPropiedades;
}

inline void monopolio::set_colorAbreviado ( string colorAbr )
{
  _colorAbreviado = colorAbr;
}

/*inline void monopolio::set_contieneCalle ( calle contieneCalle )
{
  _contieneCalle = contieneCalle;
  }*/

#endif
