#include "negociacion.h"

// Constructor por defecto
negociacion::negociacion ()
  : _dineroEfectivo ( 0 ),
    _dineroEfectivoPedido ( 0 ),
    _emisor ( NULL ),
    _receptor ( NULL )
{
  //No se hace nada
}

// Constructor explicito
negociacion::negociacion ( long dineroEfectivo,  long dineroEfectivoPedido, jugador *emisor, jugador *receptor, 
			   list<propiedad*> propiedadesOfertadas, list<tarjeta*> tarjetasOfertadas, list<propiedad*> propiedadesPedidas,
			   list<tarjeta*> tarjetasPedidas )
  : _dineroEfectivo ( dineroEfectivo ),
    _dineroEfectivoPedido ( dineroEfectivo ),
    _emisor ( emisor ),
    _receptor ( receptor ),
    _propiedadesOfertadas ( propiedadesOfertadas ),
    _propiedadesPedidas ( propiedadesPedidas ),
    _tarjetasOfertadas ( tarjetasOfertadas ),
    _tarjetasPedidas ( tarjetasPedidas )
{
  //No se hace nada
}

// Copy constructor
negociacion::negociacion (const negociacion& right)
{
  _dineroEfectivo	= right._dineroEfectivo;
  _dineroEfectivoPedido = right._dineroEfectivoPedido;
  _emisor        	= right._emisor;
  _receptor             = right._receptor;
  _propiedadesOfertadas = right._propiedadesOfertadas;
  _propiedadesPedidas   = right._propiedadesPedidas;
  _tarjetasOfertadas    = right._tarjetasOfertadas;
  _tarjetasPedidas      = right._tarjetasPedidas;
}

// Operador de asignacion
negociacion& negociacion::operator= (const negociacion& right)
{
  _dineroEfectivo	= right._dineroEfectivo;
  _dineroEfectivoPedido = right._dineroEfectivoPedido;
  _emisor        	= right._emisor;
  _receptor             = right._receptor;
  _propiedadesOfertadas = right._propiedadesOfertadas;
  _propiedadesPedidas   = right._propiedadesPedidas;
  _tarjetasOfertadas    = right._tarjetasOfertadas;
  _tarjetasPedidas      = right._tarjetasPedidas;
  return ( *this );
}

// Sobrecarga del opreador de salida
ostream& operator<< ( ostream &os, negociacion* n )
{
  jugador *emisor = n->get_emisor ();
  jugador *receptor = n->get_receptor();
  list <propiedad*> propiedadesOfertadas = n->get_propiedadesOfertadas ();
  list <propiedad*> propiedadesPedidas = n->get_propiedadesPedidas ();
  list <tarjeta*> tarjetasOfertadas = n->get_tarjetasOfertadas ();
  list <tarjeta*> tarjetasPedidas = n->get_tarjetasPedidas ();
  
  // Indica quien interviene en la negociacion
  string aux = "NEGOCIACION ENTRE " + emisor->get_nombre() + " Y " + receptor->get_nombre() + "\n";
  aux += "\n";
  aux += emisor->get_nombre() + " ofrece a " + receptor->get_nombre() + ":\n";

  // Dinero que se ofrece
  if ( n->get_dineroEfectivo() > 0 )
    {
      char numAlpha[50];
      sprintf ( numAlpha, "%d", n->get_dineroEfectivo() );
      aux += "Dinero = " + string ( numAlpha ) + "\n";      
    }

  // Propiedades que se ofrecen
  if ( propiedadesOfertadas.size () > 0 )
    {
      aux += "Propiedades: \n";
      aux += n->verPropiedadesOfertadas ();
    }
  
  // Tarjetas que se ofrecen
  if ( tarjetasOfertadas.size () == 1 )    
      aux += "Incluye 1 tarjeta para salir libre de la crcel\n";
  else if ( tarjetasOfertadas.size () > 1 )
    {
      char numAlpha[20];
      sprintf ( numAlpha, "%d", tarjetasOfertadas.size () );
      aux += "Incluye " + string ( numAlpha ) + " tarjetas para salir libre de la crcel\n";
    }
  aux += "\n";

  // Todo lo que se pide a cambio
  aux += "A cambio, " + receptor->get_nombre() + " debe darle: \n";
  // Dinero en efectivo
  if ( n->get_dineroEfectivoPedido() > 0 )
    {
      char numAlpha[50];
      sprintf ( numAlpha, "%d", n->get_dineroEfectivoPedido() );
      aux += "Dinero = " + string ( numAlpha ) + "\n";      
    }

  // Propiedades que se piden
  if ( propiedadesPedidas.size () > 0 )
    {
      aux += "Propiedades: \n";
      aux += n->verPropiedadesPedidas ();
    }

  // Tarjetas pedidas
  if ( tarjetasPedidas.size () == 1 )    
      aux += "Adems, deber aportar 1 tarjeta para salir libre de la crcel\n";
  else if ( tarjetasPedidas.size () > 1 )
    {
      char numAlpha[20];
      sprintf ( numAlpha, "%d", tarjetasPedidas.size () );
      aux += "Adems, deber aportar " + string ( numAlpha ) + " tarjetas para salir libre de la crcel\n";
    }
  aux += "\n";

  os << aux;
  return os;
}


negociacion::~negociacion ()
{
  // no se hace nada
}

/**
 * Mtodo que dirige una negociacion por el proceso normal:
 *  - Identificar el emisor
 *  - Identificar el receptor
 *  - Indicar lo que se ofrece
 *  - Indicar lo que se pide a cambio
 */
int negociacion::negociar ()
{
  int ok;
  cout << _emisor->get_nombre() << " indica lo que ofreces " << endl;

  // Seleccionar lo que se ofrece
  if ( ofrecer () >= 0 )
    {
      ok = 0;
      cout << _emisor->get_nombre() << " ahora debes indicar lo que demandas a cambio " << endl;
      // Indicar lo que se pide
      if ( pedir () >= 0 )
	{
	  ok = 0;
	}
      else
	{
	  cout << "No se ha completado la negociacin, ya que se han producido errores" << endl ;
	  ok = -1;
	}
    }
  else
    {
      cout << "No se ha completado la negociacin, ya que se han producido errores" << endl ;
      ok = -1;
    }

  // Si la oferta ha sido correcta, se preguntar al receptor si la acepta
  if ( ok == 0 )
    {
      cout << this;
      cout << _receptor->get_nombre () << ", aceptas la oferta? (s/N)";
      char opcion = cin.get();
      if ( ( opcion == 's' ) || ( opcion =='S' ) )
	{
	  // Distribuir los bienes de los jugadores de la oferta aceptada
	  distribuirBienes ();
	  cout << "Operacin completada satisfactoriamente" << endl;
	}
      else
	{
	  cout << "Negociacin cancelada" << endl << endl;
	  ok = -1;
	}
    }
  return ok;
}

/**
 * Seleccionar todo lo que se quiere ofrecer. Debe incluir tarjetas, propieades y dinero.
 */
int negociacion::ofrecer ()
{
  long cantidad = 0;
  cout << _emisor->get_nombre () << " indica la cantidad de dinero que quieres ofrecer. Si no quieres ofrecer dinero efectivo, introduce 0." << endl;
  cout << "\tDinero en efectivo: ";
  string dinero;
  int intento = 0;
  
  // Seleccionar el dinero que se ofrece
  do 
    {
      if ( intento > 0 )
	{
	  cout << "No tienes tanto dinero, introduce un valor menor o igual a " << _emisor->get_dinero_total () << endl;
	  cout << "\tDinero en efectivo: ";
	}
      cin >> dinero;
      cin.clear();
      cin.ignore( 2000, '\n' );
      cantidad = atol ( dinero.c_str() );
      intento ++;
    }
  while ( ( cantidad > _emisor->get_dinero_total() ) && ( intento <= 3 ) );

  // Si no es correcta la cantidad de dinero introducida, se devuelve error
  if ( ( cantidad > _emisor->get_dinero_total () ) || ( intento > 3 ) )
    return -1;
  else 
    {
      //Si es correcto, se establece el valor de cantidad
      int incluido = 0;
      set_dineroEfectivo ( cantidad );
      //Incluir propiedades ofertadas      
      cout << "Deseas incluir alguna propiedad en la operacion? (S/n)";
      char opcion;
      do 
	{	  
	  opcion = getchar();
	  cin.clear();
	  cin.ignore( 2000, '\n' );
	  if ( opcion != 'n' && opcion !='N' )
	    // incluir propiedades
	    {
	      incluido ++;
	      incluirPropiedadesOfertadas( );  
	      if ( incluido < _emisor->get_numPropiedades () )		
		cout << "Deseas incluir alguna propiedad ms en la operacion? (S/n)";
	    }	  
	}
      while ( opcion != 'n' && opcion != 'N' && _propiedadesOfertadas.size() < _emisor->get_numPropiedades () );
      // Si se tienen tarjetas, se pregunta si se quieren aadir y se aaden tantas veces como se quiera.
      if ( _emisor->tieneTarjetasCarcel () > 0 )
	{
	  incluido = 0;
	  cout << "Deseas incluir alguna tarjeta para salir de la crcel en la operacion? (S/n)";	   
	  do 
	    {	  
	      opcion = getchar();
	      cin.clear();
	      cin.ignore( 2000, '\n' );
	      if ( opcion != 'n' && opcion !='N' )
		 // incluir tarjetas
		{
		  incluido ++;
		  incluirTarjetasOfertadas( incluido );
		  if ( incluido < _emisor->tieneTarjetasCarcel () )
		    cout << "Deseas incluir alguna tarjeta ms en la operacion? (S/n)";
		}	  
	    }
	  while ( opcion != 'n' && opcion != 'N' && incluido < _emisor->tieneTarjetasCarcel () );
	}	
    }
  return 0;
}

/**
 * Seleccionar todo lo que se quiere perdir a cambio en una negociacin. Debe incluir tarjetas, 
 * propieades y dinero.
 */
int negociacion::pedir ()
{
  long cantidad = 0;
  cout << _emisor->get_nombre () << " indica la cantidad de dinero que quieres pedir a " << _receptor->get_nombre() << ". Si no quieres ofrecer dinero efectivo, introduce 0." << endl;
  cout << "\tDinero en efectivo: ";
  string dinero;
  int intento = 0;
  // Introducir una cantidad de dinero para ofrecer.
  do 
    {
      if ( intento > 0 )
	{
	  cout << _receptor->get_nombre() << " no tiene tanto dinero, introduce un valor menor o igual a " << _receptor->get_dinero_total () << endl;
	  cout << "\tDinero en efectivo: ";
	}
      cin >> dinero;
      cin.clear();
      cin.ignore( 2000, '\n' );
      cantidad = atol ( dinero.c_str() );
      intento ++;
    }
  while ( ( cantidad > _receptor->get_dinero_total() ) && ( intento <= 3 ) );  
  
  // Si la cantidad no es vlida, salir con error.
  if ( ( cantidad > _receptor->get_dinero_total () ) || ( intento > 3 ) )
    return -1;
  else 
    {
      int incluido = 0;
      set_dineroEfectivoPedido ( cantidad );
      //Propiedades ofertadas
      cout << "Deseas pedir alguna propiedad en la operacion? (S/n)";
      char opcion;

      // Incluir tantas propiedades como se quiera. SE muestra una lista y se recoge el nmero de propiedad
      do 
	{
	  opcion = getchar();
	  cin.clear();
	  cin.ignore( 2000, '\n' );
	  if ( opcion != 'n' && opcion !='N' )
	    // incluir propiedades
	    {
	      incluido ++;
	      incluirPropiedadesPedidas( );  
	      if ( incluido < _receptor->get_numPropiedades () )		
		cout << "Deseas pedir alguna propiedad ms? (S/n)";
	    }	  
	}
      while ( opcion != 'n' && opcion != 'N' && _propiedadesPedidas.size() < _receptor->get_numPropiedades () );

      // Si el receptor tiene tarjetas, se pregunta si las quiere pedir en la operacin
      if ( _receptor->tieneTarjetasCarcel () > 0 )
	{
	  incluido = 0;
	  cout << "Deseas adquirir alguna tarjeta para salir de la crcel con esta operacion? (S/n)";	   
	  do 
	    {	  
	      opcion = getchar();
	      cin.clear();
	      cin.ignore( 2000, '\n' );
	      if ( opcion != 'n' && opcion !='N' )
		 // incluir tarjetas
		{
		  incluido ++;
		  incluirTarjetasPedidas( incluido );  
		  if ( incluido < _receptor->tieneTarjetasCarcel () )
		    cout << "Deseas pedir otra tarjeta de crcel? (S/n)";
		}	  
	    }
	  while ( opcion != 'n' && opcion != 'N' && incluido < _receptor->tieneTarjetasCarcel () );
	}	
    }
  return 0;
}

/**
 * Las propiedades incluidas en la oferta por el emisor son adjudicadas al receptor y viceversa
 */
void negociacion::distribuirBienes ()
{
  long saldoReceptor = _receptor->get_dinero_total ();
  long capitalReceptor = _receptor->get_capital ();
  long saldoEmisor = _emisor->get_dinero_total ();
  long capitalEmisor = _emisor->get_capital ();
  
  // El receptor cobra del emisor
  _emisor->pagar ( get_dineroEfectivo() );
  _receptor->pagar ( 0 - get_dineroEfectivo () );

  //El receptor paga al emisor
  _receptor->pagar ( get_dineroEfectivoPedido() );
  _emisor->pagar ( 0 - get_dineroEfectivoPedido () );
  
  //Se aaden las propiedades ofrecidas por el emisor al receptor.
  _receptor->anyadirListaPropiedades ( _propiedadesOfertadas );
  _emisor->eliminarListaPropiedades ( _propiedadesOfertadas );

  //Se aaden las propiedades pedidas por el emisor al receptor en la lista de props del emisor.
  _emisor->anyadirListaPropiedades ( _propiedadesPedidas );
  _receptor->eliminarListaPropiedades ( _propiedadesPedidas );

  //Se intercambian las tarjetas
  _receptor->anyadirListaTarjetas ( _tarjetasOfertadas );
  _emisor->eliminarListaTarjetas ( _tarjetasOfertadas );
  
  _emisor->anyadirListaTarjetas ( _tarjetasPedidas );
  _receptor->eliminarListaTarjetas ( _tarjetasPedidas );

}

/**
 * Mtodo que lista todas las propiedades incluidas en una negociacion como oferta
 */
string negociacion::verPropiedadesOfertadas ()
{
  string aux = "";
  list <propiedad*>::iterator listaP; 
  for ( listaP = _propiedadesOfertadas.begin(); listaP != _propiedadesOfertadas.end(); listaP++ )
    {
      aux += (*listaP)->infoString() + "\n";
    }
  return aux;
}

/**
 * Mtodo que lista todas las propiedades incluidas en una negociacion como peticin
 */
string negociacion::verPropiedadesPedidas ()
{
  string aux = "";
  list <propiedad*>::iterator listaP; 
  for ( listaP = _propiedadesPedidas.begin(); listaP != _propiedadesPedidas.end(); listaP++ )
    {
      aux += (*listaP)->infoString() + "\n";
    }
  return aux;
}

/**
 * Comrpueba si la propiedad 'p' ha sido ya incluida en la oferta realizada
 */
bool negociacion::propiedadOfertadaIncluida ( propiedad *p )
{
  list <propiedad*>::iterator listaP; 
  // Recorrer toda lalista de propiedades ofertadas comprobando si existe
  for ( listaP = _propiedadesOfertadas.begin(); listaP != _propiedadesOfertadas.end(); listaP++ )
    {
      if ( *listaP == p )
	return true;
    }  
  return false;
}

/**
 * Comrpueba si la propiedad 'p' ha sido ya incluida en la oferta realizada como peticin
 */
bool negociacion::propiedadPedidaIncluida ( propiedad *p )
{
  list <propiedad*>::iterator listaP; 
  // Recorrer toda lalista de propiedades pedidas comprobando si existe
  for ( listaP = _propiedadesPedidas.begin(); listaP != _propiedadesPedidas.end(); listaP++ )
    {
      if ( *listaP == p )
	return true;
    }  
  return false;
}

/**
 * Mtodo que permite al jugador introducir las propiedades que quiere aadir a la oferta.
 * Se muestra una lista y el jugador deber seleccionar de esa lista aquellas propiedades que
 * quiera adjuntar identificandolas por su nmero de propiedad.
 */
void negociacion::incluirPropiedadesOfertadas ( )
{
  // Muestra el listado de propiedades del emisor
  cout << "LISTADO DE PROPIEDADES:" << endl;
  cout << _emisor->get_infoPropiedades ();
  cout << "(C) Cancelar " << endl;

  //Indicar el nmero de propiedad que quiere incluir
  cout << "Indica el nmero de propiedad que quieres incluir. " << endl;
  int intento = 0;
  string opcion;
  int opcionI;
  do 
    {
      opcion = "";
      if ( intento > 0 )
	cout << "[Valor no vlido] " << endl;
      cout << "Elige: ";
      cin >> opcion;
      cin.clear();
      cin.ignore( 2000, '\n' );
      opcionI = atoi ( opcion.c_str() );
      intento ++;    
    } 
  while ( ( _emisor->tienePropiedadInt ( opcionI ) == false && opcion.substr(0,1) != "c" && opcion.substr(0,1) != "C" ) );

  // Adjuntar la propiedad a la lista
  if ( opcion != "c" && opcion !="C" )
    {
      propiedad *p = _emisor->get_propiedad ( opcionI );
      if ( _emisor->get_propiedad ( opcionI ) != NULL )
	int ok = nuevaPropiedadOfertada ( p );
      else
	cout << "Propiedad no encontrada" << endl;
    }
  else
    {
      cout << "No se incluirn ms propiedades" << endl;
    }
  
  // Mostrar las propiedades incluidas.
  cout << "Propiedades incluidas: " << endl;
  list <propiedad*>::iterator listaP; 
  for ( listaP = _propiedadesOfertadas.begin(); listaP != _propiedadesOfertadas.end(); listaP++ )
    {
      cout << (*listaP)->get_nombre() << endl;      
    }  
  return;
}

 /**
  * Mtodo que permite al jugador introducir las propiedades que quiere pedir al receptor..
  * Se muestra una lista y el jugador deber seleccionar de esa lista aquellas propiedades que
  * quiera adjuntar identificandolas por su nmero de propiedad.
  */
void negociacion::incluirPropiedadesPedidas ( )
{
  // Muestra el listado de propiedades del receptor para poder elegir
  cout << "LISTADO DE PROPIEDADES DE " << _receptor->get_nombre() << ":" << endl;
  cout << _receptor->get_infoPropiedades ();
  cout << "(C) Cancelar " << endl;
  cout << "Indica el nmero de propiedad que quieres pedir. " << endl;
  int intento = 0;
  string opcion;
  int opcionI;
  // Leera el valor que introduzca el jugador
  do 
    {
      opcion = "";
      if ( intento > 0 )
	cout << "[Valor no vlido] " << endl;
      cout << "Elige: ";
      cin >> opcion;
      cin.clear();
      cin.ignore( 2000, '\n' );
      opcionI = atoi ( opcion.c_str() );
      intento ++;    
    } 
  while ( ( _receptor->tienePropiedadInt ( opcionI ) == false && opcion != "c" && opcion != "C" ) );
  //Si el valor es correcto, se aadir a la lista
  if ( opcion != "c" && opcion !="C" )
    {
      propiedad *p = _receptor->get_propiedad ( opcionI );
      if ( _receptor->get_propiedad ( opcionI ) != NULL )
	int ok = nuevaPropiedadPedida ( p );
      else
	cout << "Propiedad no encontrada" << endl;
    }
  else
    {
      cout << "No se incluirn ms propiedades" << endl;
    }
  
  // Se muestran las propiedades incluidas.
  cout << "Propiedades incluidas: " << endl;
  list <propiedad*>::iterator listaP; 
  for ( listaP = _propiedadesPedidas.begin(); listaP != _propiedadesPedidas.end(); listaP++ )
    {
      cout << (*listaP)->get_nombre() << endl;      
    }  
  return;
}

/**
 * Mtodo que prueba la factibilidad de incluir una propiedad en la negociacin como oferta.
 * Comprobar que la calle no este construida y si lo est, deber ofrecer incluir todo el monopolio
 * con casas y hoteles incluidas.
 * Finalmente, aadir la propiedad a la lista de ofertadas.
 */
int negociacion::nuevaPropiedadOfertada ( propiedad *p )
{
  int ok = 0;
  // Si es ofertable, se incluye en la lista
  if ( p->esOfertable () )
    {
       if ( ! propiedadOfertadaIncluida ( p ) )
	{
	  anyadirPropiedadOfertada ( p );
	  cout << "Propiedad incluida" << endl;
	}
      else
	{
	  cout << "Esta propiedad ya ha sido aadida a la lista, no se vuelve a incluir" << endl;
	  ok = -1;
	}      
    }
  // Si tiene edificios construidos, entonces se ofrece incluir todo el monopolio
  else    
    {
      cout << "No puedes incluir esta propiedad porque tiene edificios construidos." << endl;
      cout << "Quieres aadir todo el monopolio con sus edificaciones en la oferta? (s/N)";
      char opcion = cin.get();      
      // Si se quiere incluir todo el monopolio, aaden todas las propiedades a la lista
      if ( ( opcion == 's' ) || ( opcion =='S' ) )
	{	  
	  set <propiedad*> propiedadesMonopolio = p->get_monopolioCompleto ( p);
	  set <propiedad*>::iterator listaP; 
	  listaP = propiedadesMonopolio.begin();
	  while ( ( listaP != propiedadesMonopolio.end() ) )
	    {	      
	      anyadirPropiedadOfertada (  *listaP );
	      listaP ++;
	    }
	}
      else
	{
	  cout << "No se ha aadido la propiedad" << endl;
	  ok = -1;
	}
    }
  return ok;
}

/**
 * Mtodo que prueba la factibilidad de incluir una propiedad en la negociacin como peticin.
 * Comprobar que la calle no este construida y si lo est, deber ofrecer incluir todo el monopolio
 * con casas y hoteles incluidas.
 * Finalmente, aadir la propiedad a la lista de peticiones.
 */
int negociacion::nuevaPropiedadPedida ( propiedad *p )
{
  int ok = 0;
  // Sis e puede incluir se aade a la lista
  if ( p->esOfertable () )
    {
      if ( ! propiedadPedidaIncluida ( p ) )
	{
	  anyadirPropiedadPedida ( p );
	  cout << "Propiedad incluida" << endl;
	}
      else
	{
	  cout << "Esta propiedad ya ha sido aadida a la lista, no se vuelve a incluir" << endl;
	  ok = -1;
	}
    }
  // Si no se puede incluir, se pregunta si se quiere incluir el monopolio completo. 
  // En caso afirmativo, se incluirn todas las casillas.
  else
    {
      cout << "No puedes incluir esta propiedad porque tiene edificios construidos." << endl;
      cout << "Quieres aadir todo el monopolio con sus edificaciones en la oferta? (s/N)";
      char opcion = cin.get();      
      if ( ( opcion == 's' ) || ( opcion =='S' ) )
	{	  
	  set <propiedad*> propiedadesMonopolio = p->get_monopolioCompleto ( p);
	  set <propiedad*>::iterator listaP; 
	  listaP = propiedadesMonopolio.begin();
	  while ( ( listaP != propiedadesMonopolio.end() ) )
	    {	      
	      anyadirPropiedadPedida (  *listaP );
	      listaP ++;
	    }
	}
      else
	{
	  cout << "No se ha aadido la propiedad" << endl;
	  ok = -1;
	}
    }
  return ok;
}

/**
 * Aade ordenadamente la propiedad p a la lista de propiedades ofertadas
 */
void negociacion::anyadirPropiedadOfertada ( propiedad *p )
{  
  list <propiedad*>::iterator listaP; 
  listaP = _propiedadesOfertadas.begin();
  while ( ( *listaP < p ) && ( listaP != _propiedadesOfertadas.end() ) )
    {
      listaP ++;
    }
  _propiedadesOfertadas.insert ( listaP, p );
  return;
}

/**
 * Aade ordenadamente la propiedad p a la lista de propiedades pedidas
 */
void negociacion::anyadirPropiedadPedida ( propiedad *p )
{  
  list <propiedad*>::iterator listaP; 
  listaP = _propiedadesPedidas.begin();
  while ( ( *listaP < p ) && ( listaP != _propiedadesPedidas.end() ) )
    {
      listaP ++;
    }
  _propiedadesPedidas.insert ( listaP, p );
  return;
}

/**
 * Aade una tarjeta a las tarjetas ofertadas. La recoge de la lista de tarjetas del emisor
 */
void negociacion::incluirTarjetasOfertadas ( int i )
{  
  
  tarjeta *tjt = _emisor->get_tarjeta ( i ); 
  anyadirTarjetaOfertada ( tjt );
  cout << "Numero de tarjetas incluidas: " << _tarjetasOfertadas.size() << endl;
  return;
}

/**
 * Aade una tarjeta a las tarjetas pedidas. La recoge de la lista de tarjetas del receptor
 */
void negociacion::incluirTarjetasPedidas ( int i )
{  
  
  tarjeta *tjt = _receptor->get_tarjeta ( i ); 
  anyadirTarjetaPedida ( tjt );
  cout << "Numero de tarjetas incluidas: " << _tarjetasPedidas.size() << endl;
  return;
}

/**
 * Insertar una tarjeta en la lista de tarjetas ofertadas
 */
void negociacion::anyadirTarjetaOfertada ( tarjeta *t )
{  
  _tarjetasOfertadas.push_front ( t );
  return;
}

/**
 * Insertar una tarjeta en la lista de tarjetas pedidas
 */
void negociacion::anyadirTarjetaPedida ( tarjeta *t )
{  
  _tarjetasPedidas.push_front ( t );
  return;
}
