/**********************************************************************************
			propiedad.cpp
	
Programa: 	9
Nombre:		Javier Aragn Zabalegui		100039831@alumnos.uc3m.es
Fecha_inicio:	2.Noviembre.2005
Descripcion:	Clase propiedad para Street Master's
			
Comentarios:	Hereda de casilla
**********************************************************************************/

/**********************************************************************************
Listado de contenidos:
	include "propiedad.h"

**********************************************************************************/

#include "propiedad.h"
#include "tablero.h"


// Constructor por defecto
propiedad::propiedad ()
  : casilla (),
    _precio ( 0 ),
    _hipoteca ( 0 ),
    _hipotecado ( false ),
    _hipotecable ( false ),
    _esPropiedad ( NULL )
{
  // No se hace nada!
}

// constructor explcito
propiedad::propiedad ( int numero, string nombre, string nombreCorto, long precio, long hipoteca, bool hipotecado, bool hipotecable, propietario *p )
  : casilla ( numero, nombre, nombreCorto ),    
    _precio ( precio ),
    _hipoteca ( hipoteca ),
    _hipotecado ( hipotecado ),
    _hipotecable ( hipotecable ),
    _esPropiedad ( p )
{
  // No se hace nada!
}
 
// Copy constructor
propiedad::propiedad (const propiedad& right):casilla(right)
{
  _precio	= right._precio;
  _hipoteca	= right._hipoteca;
  _hipotecado	= right._hipotecado;
  _hipotecable	= right._hipotecable;
  _esPropiedad  = right._esPropiedad;
}

// Assignment operator
propiedad& propiedad::operator= (const propiedad& right)
{
  casilla::operator= (right);
  _precio	= right.get_precio ();
  _hipoteca	= right.get_hipoteca ();
  _hipotecado	= right.get_hipotecado ();
  _hipotecable	= right.get_hipotecable ();
  _esPropiedad  = right._esPropiedad;
  return ( *this );
}

bool propiedad::operator< ( propiedad *compara )
{
  if ( get_numero() < compara->get_numero() )
    return true;
  else
    return false;
}

ostream& operator<< (ostream &os, propiedad *c)
{  
  os << c->mostrar ();
  return os;
  //...Aqu como se muestra una casilla
}

// Destructor
propiedad::~propiedad ()
{
  // No se hace nada!
}

/**
 * Mtodo que comprueba si el monopolio al que pertenece esta calle es el mismo que el
 * que se indica como parmetro (color)
 */
bool propiedad::esMonopolio ( string color )
{
  monopolio *mp; 
  mp = get_perteneceMonopolio();  
  if ( mp->get_color () != color ) 
    return false;    
  else
    return true;    
}

/**
 * Mtodo que crea un monopolio con el color pasado como parmetro. Se dar valor 1 al 
 * atributo numPropiedades de la clase monopolio porque si se crea el monopolio es porque
 * no exisita previamente y por lo tanto el nmero de propiedades actual es ese (1).
 */
void propiedad::crearMonopolio ( string color )
{ 
  monopolio *mp = new monopolio ( color, 1 );
  set_perteneceMonopolio ( mp );
  mp -> anyadirPropiedad ( this );
}

/**
 * Mtodo que asigna una calle al mismo monopolio al que pertenece la calle pasada como
 * parmetro.
 */
void propiedad::asignarMonopolio ( propiedad* p )
{ 
  monopolio* mp = p->get_perteneceMonopolio ();  
  int numP = mp->get_numPropiedades();
  mp->set_numPropiedades( ++numP );
  set_perteneceMonopolio ( mp );
  mp -> anyadirPropiedad ( this );
  
}

/**
 * Mtodo que devuelve el color del monopolio al que pertenece la propiedad
 */
string propiedad::verColor ()
{
  monopolio *mp;
  mp = get_perteneceMonopolio();
  return ( mp->get_color() );
}

/**
 * Comprueba si la propiedad tiene dueo o est disponible para ser comprada
 */
bool propiedad::disponible ()
{
  //Si no pertenece a un jugador esta disponible
  if ( !dynamic_cast <jugador *> ( get_esPropiedad() ) )  
    return true;
  else
    return false;
}


//Mtodos virtuales: Polimorfismo

// MTODOS PARA DIBUJAR LAS CASILLAS

/**
 * Mtodo que muestra la informacin de una propiedad
 */
string propiedad::mostrar ()
{
  return "propiedad";
}

/**
 * Mtodo que dibuja la fila indicada de la casilla correspondiente (en las filas de arriba y abajo del tablero).
 * Se indicar la fila que se tiene que dibujar y el ancho con el que se tiene que pintar. Como resultado se
 * devolver una cadena de caracteres. En este caso se muestra la informacin propia de una propiedad.
 */
string propiedad::dibujarCasillaHorizontal ( int fila, int anchoCasilla )
{
  string sFila;  
  
  switch ( fila )
    {
    case 0:
      sFila = " ";				//Primer caracter
      for ( int i = 1; i < anchoCasilla ; i++ )
	sFila += "_";
      break;
    
    // Dibujar el nmero de casilla
    case 1:					
      {
	char numAlpha[10];
	sprintf( numAlpha, "%d", get_numero() );
	sFila = string ( numAlpha );       
	while ( sFila.size() <= ( anchoCasilla - 2 ) )
	  sFila += " ";      
	sFila += "|";
	break;
      }

    // Aade el nombre corto
    case 2:
      sFila = get_nombreCorto();
      while ( sFila.size() <= ( anchoCasilla - 2 ) )
	sFila += " ";      
      sFila += "|";
      break;
      
    // Aade los jugadores que se encuentran en la casilla y el monopolio
    case 3:					
      {
	int i = 0;
	string aux = "";
	int numJugadores = estaJugador.size ();
	while ( ( numJugadores > 0 ) && ( i < numJugadores ) && ( i < 3 ) )
	  {
	    string color = estaJugador[i]->get_color();
	    aux += color.substr(0,2); 
	    i++;
	    if ( i < 3 ) 
	      aux += " ";
	  }
	cout << i << endl;
	while (aux.size() < anchoCasilla - 1 )
	  aux += " ";
	sFila += aux + '|';
	break;
      }

    // Indica los jugadores si hay ms de 3
    case 4:
      {
	int i = 3;
	string aux = "";
	int numJugadores = estaJugador.size ();
	while ( ( numJugadores > 3 ) && ( i < numJugadores ) && ( i < 6 ) )
	  {	   
	    string color = estaJugador[i]->get_color();
	    aux += color.substr(0,2);
	    i++;
	    if ( i < 6 ) 
	      aux += " ";	
	  }
	cout << i << endl;
	while (aux.size() < anchoCasilla - 1 )
	  aux += " ";
	sFila += aux + '|';
	break;
      }

    case 5:
      while ( sFila.size() <= ( anchoCasilla - 2 ) )
	sFila += "_";	 	
      sFila += "|";
      break;    
    }
  
  return sFila;
}

/**
 * Mtodo que dibuja la fila indicada de la casilla correspondiente (en las columnas derecha e izquierda del tablero).
 * Se indicar la fila que se tiene que dibujar y el ancho con el que se tiene que pintar. Como resultado se
 * devolver una cadena de caracteres. En este caso se muestra la informacin propia de una propiedad
 */
string propiedad::dibujarCasillaVertical ( int fila, int anchoCasilla )
{
  string sFila;  
  
  sFila = "|";
  switch ( fila )
    {    
    // Nmero de casillas y jugadores que estn sobre ella
    case 0:
      {
	string aux = "";
	char numAlpha[10];
	sprintf( numAlpha, "%d", get_numero() );
	aux += string ( numAlpha ) + "    ";
	int i = 0;	
	cout << "Jugadores en la casilla: " << estaJugador.size () << endl;
	int numJugadores = estaJugador.size ();
	while ( ( numJugadores > 0 ) && ( i < numJugadores ) && ( i < 3 ) )
	  {
	    string color = estaJugador[i]->get_color();
	    aux += color.substr(0,2);	    
	    i++;
	    if ( i < 3 ) 
	      aux += " ";	
	  }
	cout << i << endl;
	while (aux.size() < anchoCasilla - 1 )
	  aux += " ";
	sFila += aux + '|';      
	break;
      }

    // Jugadores en la casilla y nombre corto de la misma
    case 1:
      {
	string aux = "";
	aux += get_nombreCorto();      
	while ( aux.size() < 6 ) aux += " ";
	int i = 3;	
	int numJugadores = estaJugador.size ();
	while ( ( numJugadores > 3 ) && ( i < numJugadores ) && ( i < 6 ) )
	  {
	    string color = estaJugador[i]->get_color();
	    aux += color.substr(0,2);
	    i++;
	    if ( i < 6 ) 
	      aux += " ";	
	  }
	cout << i << endl;
	while (aux.size() < anchoCasilla - 1 )
	  aux += " ";
	sFila += aux + '|';
	break;
      }
      
    case 2:      
      while ( sFila.size() <= ( anchoCasilla - 1) )
	sFila += "_";      
      sFila += "|";
      break;  
      
      return sFila;
    }
}

/**
 * Mtodo que devuelve una cadena de caracteres que representa como es la casilla en su 
 * imgen en el tablero, indicando aquellos datos de inters para un jugador que caiga 
 * sobre ella:
 *  -Numero
 *  -Nombre
 *  -Precio
 *  -Hipoteca
 */
string propiedad::dibujarCasillaEnTablero ()
{
  const int anchoDibujado = 27;
  string aux;
  aux = " ___________________________\n";
  aux +="|                           |\n";
  aux +="|                           |\n";  
  
  // Linea del nombre 
  string linea = "|";
  string nombre = get_nombre ();
  aux += centrarCadena ( nombre, anchoDibujado + 2, 2, "|" );

  // Informacin sobre la edificacin en la calle 
  aux +="|                           |\n";
  aux +="|                           |\n";
  aux +="|                           |\n";

  // Precio de la propiedad
  string precio = "Precio ";
  char precioAlpha[30];
  sprintf( precioAlpha, "%d", get_precio() );
  precio += precioAlpha;
  aux += centrarCadena ( precio, anchoDibujado + 2, 1, "|" );

  // Nmero de casilla 
  int numero = get_numero();
  aux +="|                         ";
  if ( numero < 10 )
    aux += " ";
  char numAlpha[10];
  sprintf( numAlpha, "%d", numero );
  aux += numAlpha;
  aux += "|\n";

  // Base del rectngulo
  aux +="|___________________________|\n";
  return aux;
}

/**
 * Mtodo que calcula el alquiler de una propiedad
 */
long propiedad::calcularAlquiler ( jugador *j )
{
  //En el caso de la clase base no es aplicable
  return 0;
}

/**
 * Se adjudica la propiedad a un propietario genrico dado, indpendientemente de que se trate de 
 * la banca o de un jugador
 */
int propiedad::adjudicarA ( propietario* j )
{
  //Si es un jugador, este debe pagar por ella (invertir)
  if ( jugador *jug = dynamic_cast <jugador *> ( j ) ) 
    jug->invertir ( get_precio() );    

  //Se aade la propiedad al propietario
  j->set_posee ( this );
  set_esPropiedad ( j );
  return 0;
}

/**
 * El jugador paga el alquiler que se ha calculado para la propiedad dada. Si el jugador no
 * tiene dinero quedar como un jugador moroso. El propietario de la propiedad cobrar dicha
 * cantidad.
 */
int propiedad::pagarAlquiler ( jugador* j )
{
  int ok = 0;
  if ( jugador *propieta = dynamic_cast <jugador *> ( get_esPropiedad() ) )
    {      
      cout << j->get_nombre() << " debes pagar a " << propieta->get_nombre() << " por caer en su propiedad." << endl;
      if ( j->get_multiPago() > 1 )
	cout << "La cantidad estipulada se multiplicar por " << j->get_multiPago() << endl;
      long cantidad = calcularAlquiler ( j );
      int multi = j->get_multiPago();
      cantidad = cantidad * multi;
      if ( multi > 1 )
	cout << "Se ha multiplicado el valor por " << multi << endl;
      cout << "\t+Total a pagar: " << cantidad  << endl;
      //Si hay dinero suficiente, se paga y el jugador propietario, cobra,
      //si no, el jugador propietario no cobrar, pero se quedar en estado de acreedor.      
      if ( j->pagar( cantidad ) == -1  )
	{
	  propieta->set_estado ( cantidad );
	  ok = 1;	 
	}
      else
	{
	  propieta->pagar ( 0 - cantidad );
	  ok = 0;
	}
    }
  else
    ok = -1;
  return ok;
}

/**
 * Mtodo polimrfico de la clase casilla. En el caso de las propiedades, el mtodo tiene dos 
 * posibles actuaciones. Si la casilla tiene dueo, se debe pagar el alquiler al propietario.
 * Si no tiene dueo, se ofrece al jugador la posibilidad de comprar la propiedad. Si acepta,
 * pagar el precio y se aadir a su lista de propiedades, si no, la casilla ser subastada.
 */
int propiedad::accion ( jugador* player )
{
  int ok = 0;
  if ( disponible () )
    {      
      if ( ! player->decidirComprar ( this ) )
	ok = tablero::subastar ( this );
      else
	ok = comprar ( player );	   
    }
  //Si pertenece a un jugador y no esta hipotecada, se paga por caer
  else if ( player == get_esPropiedad () )
    cout << "Es una casilla de tu propiedad" << endl;
  else if ( !get_hipotecado() )
    {
      ok = pagarAlquiler ( player );
    }
  //Si esta hipotecada, no se paga, y contina el juego
  else
    {
      cout << "Propiedad hipotecada" << endl;
      ok = 0;
    }
  return ok;
}

/**
 * El jugador compra la propiedad a cambio del precio establecido. Se descontar la cantidad de 
 * dinero del dinero total del jugador y se aadir dicha propiedad a su lista de propiedades.
 */
int propiedad::comprar ( jugador* j )
{
  int ok = 0;
  //Si no tiene dinero, no se compra
  if ( get_precio() > j->get_dinero_total () )
    {
      cout << "No tienes dinero suficiente para completar la operacin. La propiedad ser subastada." << endl; 
      ok = tablero::subastar ( this ); //Valor de subasta
    }
  // Si se tiene dinero para comprar una propiedad, se adjudica la casilla al jugador  
  else if ( get_precio() <= j->get_dinero_total () )
    {
      ok = adjudicarA ( j );
      cout << "Enhorabuena, has adquirido la propiedad: " << get_nombre() << endl;     
    }
  return ( ok );
}

/**
 * Comrpueba si una propiedad tiene el mismo propietario que otra.
 */
bool propiedad::mismoPropietario ( propiedad* p )
{
  bool iguales;
  if ( get_esPropiedad() == p->get_esPropiedad() )
    iguales = true;
  else
    iguales = false;
  return iguales;
}

/**
 * Devuelve la informacin de una casilla en forma de cadena de caracteres (string).
 */
string propiedad::infoString ()
{
  string aux = "";
  char numAlpha[10];
  sprintf ( numAlpha, "%d", get_numero() );
  aux += "(" + string ( numAlpha ) + ") " + get_nombre(); 
  for ( int i = aux.size() ; i <= 35; i++ )
    aux += " ";
  aux += "[" + verColor() + "]";
  if ( get_hipotecado() )
    {
      for ( int i = aux.size() ; i <= 55; i++ )
	aux += " ";
      aux += "(HIPOTECADA)";
    }
 
  return ( aux );
}

/**
 * Mtodo que hipoteca la propiedad. El propietario recibir la cantidad de dinero indicada en
 * ese caso y la propiedad pasar a tener estado hipotecado (y no hipotecable).
 */
int propiedad::hipotecar ()
{
  int ok = 0;
  // PAra ser hipotecada debe pertenecer a un jugador
  if ( jugador *j = dynamic_cast <jugador *> ( get_esPropiedad() ) )
    {
      //Comrpueba que sea hipotecable
      if ( get_hipotecable() )
	{
	  ok = 0;
	  //Indico que no se puede hipotecar otra vez hasta que se deshipoteque
	  set_hipotecable ( false );
	  set_hipotecado ( true );
	  //Se paga al jugador la cantidad estipulada
	  long cantidadHipoteca = get_hipoteca ();	  
	  j->pagar ( 0 - cantidadHipoteca );
	  j->set_capital ( j->get_capital() - get_precio() );
	}
      else
	{
	  ok = -1;
	}
    }
  else
    ok = -1;
  return ok;
}

/**
 * Mtodo que deshipoteca la propiedad, indicando dicho estado a la propiedad y cobrando al jugador la
 * cantidad indicada.
 */
int propiedad::deshipotecar ()
{
  int ok = 0;
  if ( jugador *j = dynamic_cast <jugador *> ( get_esPropiedad() ) )
    {  
      long cantidadHipoteca = get_deshipoteca ();
      if ( get_hipotecado() )
	if ( j->get_dinero_total() >= cantidadHipoteca )
	  {
	    ok = 0;
	    //Indico que se podr volver a hipotecar y que no est hipotecado
	    set_hipotecable ( true );
	    set_hipotecado ( false );
	    //Se cobra al jugador la cantidad estipulada (hipoteca + 10% de inters)     
	    j->pagar ( cantidadHipoteca );
	    j->set_capital ( j->get_capital () + get_precio () );
	  }
	else
	  ok = -2;
      else
	{
	  ok = -1;
	}
    }
  else
    ok = -1;
  return ok;
}

/**
 * Indica si una propiedad se puede ofertar en una negociacin
 */
bool propiedad::esOfertable ()
{
  return true;
}

/**
 * Obtiene todas las propiedades que pertenecen al mismo monopolio
 */
set <propiedad*> propiedad::get_monopolioCompleto ( propiedad *p )
{
  monopolio *mp = get_perteneceMonopolio ();
  return ( mp->get_contienePropiedad () );
}

/**
 * Asigna la propiedad al propietario p sin cobrar ni preguntar. 
 * Este propietario suele ser la banca.
 */
void propiedad::liberar ( propietario *p )
{
  set_hipotecado ( false );
  set_hipotecable ( true );
  _esPropiedad = p;
  return;
}

/**
 * Devuelve una cadena de caracteres con el formato necesario para guardarla en un fichero en el que
 * se est salvando una partida en curso
 */
string propiedad::guardar ()
{
  string aux = "PROPIEDAD\n";
  char numAlpha[30];
  sprintf ( numAlpha, "%d", get_numero() );
  aux += "NUMERO= " + string ( numAlpha );
  aux += "\n";
  sprintf ( numAlpha, "%d", get_hipotecado() );
  aux += "HIPOTECADO = " + string ( numAlpha );
  aux += "\n";
  sprintf ( numAlpha, "%d", get_hipotecable () );
  aux += "HIPOTECABLE= " + string ( numAlpha );
  aux += ";\n";
  return aux;
}

/**
 * Mtodo que indica si est completamente libre el monopolio al que pertenece la propiedad.
 */
bool propiedad::monopolioLibre ()
{
  monopolio *mp = get_perteneceMonopolio ();
  return ( mp->estaLibre () );
}

/**
 * Mtodo que indica si esta es la ltima propiedad que le falta a el jugador pasado como argumento
 * para completar el monopolio.
 */
bool propiedad::ultimaPropiedadMonopolio ( jugador *j )
{
  monopolio *mp = get_perteneceMonopolio ();
  return ( mp->ultimaPropiedad ( j ) );
}

/**
 * Devuelve verdadero si el jugador j es el nico que tiene propiedades del monopolio al que pertence
 * la propiedad actual
 */
bool propiedad::unicoPoseedorMonopolio ( jugador *j )
{
  monopolio *mp = get_perteneceMonopolio ();
  return ( mp->unicoPoseedor ( j ) );
}

/**
 * Mtodo que comprueba si el jugador j posee parte del monopolio al que pertence esta propiedad.
 */
bool propiedad::poseeParteMonopolio ( jugador *j )
{
  monopolio *mp = get_perteneceMonopolio ();
  return ( mp->poseeParte ( j ) );
}

/**
 * Mtodo que indica si esta es la ltima propiedad que le falta a un jugador distinto al pasado 
 * como argumento para completar el monopolio.
 */
bool propiedad::ultimaPropiedadMonopolioOtroJugador ( jugador *j )
{
  monopolio *mp = get_perteneceMonopolio ();
  return ( mp->ultimaPropiedadOtroJugador ( j ) );
}
