/***********************************************************************************
 *  CLASE PROPIETARIO
 *  Clase que representa cualquiera de los posibles propietarios de las propiedades
 *  del juego Street Master's. Es la clase base de una jerarqua en la que se deriva
 *  en banca y jugador.  
 *
 *  Requiere:
 *     - clase string
 *     - clase list
 *
 *  Autor: Javier Aragn Zabalegui (PFC para la uc3m: street Master's)
 *  Contacto: 100039831@alumnos.uc3m.es
 *  version 1.0
 **********************************************************************************/

#ifndef __PROPIETARIO__
#define __PROPIETARIO__

#include <list>
#include <string>
using namespace std;

class propiedad;  //Fordward declaration

class propietario {

 protected:   //Se usaran en jugador y banca
  list<propiedad*> _posee;

 public:
  //Constructores
  propietario ();
  propietario ( list<propiedad*> posee );
  
  //Copy constructor
  propietario ( const propietario& right);

  //Operador de asignacin
  propietario& propietario::operator= (const propietario& right);
  
  //Destructor
  ~propietario ();

  // Mtodos GET
  list<propiedad*> get_posee () const;
  
  // Mtodos SET
  void set_posee ( propiedad *p );

  /**
   * OTROS MTODOS
   */
  
  /**
   * Comrpueba si el propietario tiene la propiedad identificada por el nmero pasado como argumento
   * del mtodo
   */
  bool tienePropiedadInt ( int numero );

  /**
   * Se obtiene la propiedad del propietario cuyo numero coincide con el que se pasa como argumento
   */
  propiedad *get_propiedad ( int numero );

  /**
   * Aade una lista completa de propiedades al propietario. Se aadir una a una cada propiedad.
   */
  virtual void anyadirListaPropiedades ( list <propiedad*> listaPropiedades );

  /**
   * Elimina una propiedad dada de las posesiones del propietario.
   */
  void eliminarPropiedad ( propiedad *p );

  /**
   * Elimina las coincidencias de la lista dada de la lista de propiedades del propietario. Es
   * diferente si el propietario es un jugador o la banca, por eso es virtual.
   */
  virtual void eliminarListaPropiedades ( list <propiedad*> listaPropiedades );

};

inline list<propiedad*> propietario::get_posee () const
{
  return ( _posee ); 
}


inline void propietario::set_posee ( propiedad* p )
{
  list <propiedad*>::iterator listaP; 
  listaP = _posee.begin();
  while ( ( *listaP < p ) && ( listaP != _posee.end() ) )
    listaP ++;
  _posee.insert ( listaP, p );
}

#endif
