/***********************************************************************************
 *  CLASE SALIDA
 *  Clase que representa la casilla de salida del juego del Monopoly. Tiene un atributo
 *  _sueldo que indica cuanto se debe pagar a un jugador al pasar sobre ella.
 *
 *  Requiere:
 *     - clase casilla (herencia)
 *
 *  Autor: Javier Aragn Zabalegui (PFC para la uc3m: street Master's)
 *  Contacto: 100039831@alumnos.uc3m.es
 *  version 1.0
 **********************************************************************************/

#ifndef __SALIDA__
#define __SALIDA__

#include "cadenas.h"
#include "casilla.h"

#include <string>
using namespace std;

class salida : public casilla {

 private:

  //Atributos
  long _sueldo; 
  
  //Asociaciones
 public:

  // Constructor por defecto
  salida ();
  
  // Constructor explcito
  salida ( int numero, string nombre, string nombreCorto, long sueldo );
  
  // Copy constructor
  salida (const salida& right);

  // Operador de asignacin
  salida& operator= (const salida& right);

  // Destructor
  ~salida ();

  // Mtodos GET
  long get_sueldo () const;   
  
  // Mtodos SET
  void set_sueldo ( long sueldo );

  
  /**
   * Polimorfismo de la clase casilla. En el caso de las casilla se debe pagar al 
   * jugador la cantidad de dinero que se indica en el atributo _sueldo.
   */
  int accion ( jugador* player );

  /**
   * Mtodo que dibuja la fila indicada de la casilla correspondiente (en las filas de arriba y abajo del tablero).
   * Se indicar la fila que se tiene que dibujar y el ancho con el que se tiene que pintar. Como resultado se
   * devolver una cadena de caracteres.
   */
  string dibujarCasillaHorizontal ( int fila, int anchoCasilla );

  /**
   * Mtodo que dibuja la fila indicada de la casilla correspondiente (en las columnas derecha e izquierda del tablero).
   * Se indicar la fila que se tiene que dibujar y el ancho con el que se tiene que pintar. Como resultado se
   * devolver una cadena de caracteres.
   */
  string dibujarCasillaVertical ( int fila, int anchoCasilla );
  
  /**
   * Mtodo que devuelve una cadena de caracteres que representa como es la casilla en su 
   * imgen en el tablero, indicando aquellos datos de inters para un jugador que caiga 
   * sobre ella:
   *  -Numero
   *  -Nombre
   *  -Sueldo
   */
  string dibujarCasillaEnTablero ();

  /**
   * Devuelve la informacin de la casilla de salida en modo texto para mostrar por pantalla la informacin
   * en forma de tabla. 
   */
  string leyendaCasilla ( int num, int nombreCorto, int tipo, int nombre, int estado );

};

inline long salida::get_sueldo () const
{
  return _sueldo;
}

inline void salida::set_sueldo ( long sueldo )
{
  _sueldo = sueldo;
}


#endif
