
#include "servicio.h"

// Constructor por defecto
servicio::servicio ()
  : propiedad (),
    _multiplicador1 ( 0 ),
    _multiplicador2 ( 0 )
{
  // No se hace nada!
}

// constructor explcito
servicio::servicio ( int numero, string nombre, string nombreCorto, long precio, long hipoteca, 
		     bool hipotecado, bool hipotecable, int multiplicador1, int multiplicador2,
		     propietario *p )
  : propiedad ( numero, nombre, nombreCorto, precio, hipoteca, hipotecado, hipotecable, p ),    
    _multiplicador1 ( multiplicador1 ),
    _multiplicador2 ( multiplicador2 )
{
  // No se hace nada!
}
 
// Copy constructor
servicio::servicio (const servicio& right)
{
  propiedad servicio(right);			
  _multiplicador1	= right._multiplicador1;
  _multiplicador2	= right._multiplicador2;
}

// Assignment operator
servicio& servicio::operator= (const servicio& right)
{  
  propiedad::operator= ( right );
  _multiplicador1	= right.get_multiplicador1 ();
  _multiplicador2	= right.get_multiplicador2 ();
  return ( *this );
}

ostream& operator<< ( ostream& os, servicio *s )
{
  os << "info estacion";
  return os;
}


// Destructor
servicio::~servicio ()
{
  // No se hace nada!
}


// POLIMORFISMOS

/**
 * Mtodo que devuelve en una cadena de caracteres toda la informacin necesaria, con un
 * formato preparado para mostrarse por pantalle, toda la informacin de un servicio, que 
 * incluye:
 *    -Nombre
 *    -Estado (hipotecado, libre ...)
 *    -Propietario
 *    -Nmero de casilla
 *    -Informacin sobre los precios de alquiler
 *    -Precio de compra e hipoteca
 * Representa la tarjeta de propiedad de una propiedad de tipo servicio.
 */
string servicio::mostrar ()
{  
  const int ancho = 42;
  string aux =  " __________________________________________\n";
  aux += "|                                          |\n";
  aux += "|                                          |\n";
  aux += centrarCadena ( get_nombre(), ancho + 2, 2, "|" );

  // Informacin sobre el propietario.
  if ( jugador *j = dynamic_cast <jugador *> ( get_esPropiedad() ) )
    {      
      if ( j != NULL )
	{
	  aux += "|                                          |";
	  if ( get_hipotecado() )
	    {
	      aux += "             PROPIETARIO: ";
	      aux += j->get_nombre() + " (" + j->get_color() + ")    HIPOTECADO\n";
	    }
	  else
	    {
	      aux += "             PROPIETARIO: ";
	      aux += j->get_nombre() + " (" + j->get_color() + ")\n";
	    }
	}
      else
	aux += "|                                          |\n"; 
    }
  else
    aux += "|                                          |\n";
  
  aux += "|                                          |\n";
  
  // Informacin sobre el precio de la propiedad
  char precioAlpha[30];
  sprintf ( precioAlpha, "%d", get_precio () );
  string aux2 = "Precio = " + string ( precioAlpha );
  aux += centrarCadena ( aux2, ancho + 2, 1, "|" ); 
  aux += "|                                          |\n";
  
  // Informacin sobre los multiplicadores
  sprintf ( precioAlpha, "%d", get_multiplicador1 () );
  string texto1 = "Si se posee UNA carta de Compaa de Servicio Pblico el alquiler es " + string ( precioAlpha ) + " veces el nmero salido en los dados.";  
  aux += centrarCadena ( texto1 , ancho + 2, 4, "|" );

  aux += "|                                          |\n";

  sprintf ( precioAlpha, "%d", get_multiplicador2 () );
  string texto2 = "Si se poseen dos cartas de Comas de Servicio Pblico, el Alquiler es " + string ( precioAlpha ) + " veces el nmero salido en los dados.";
  aux += centrarCadena ( texto2, ancho + 2, 4, "|" );
  
  //Informacin sobre las hipotecas
  sprintf ( precioAlpha, "%d", get_hipoteca () );
  aux2 = "Hipoteca = " + string ( precioAlpha );
  aux += centrarCadena ( aux2, ancho + 2, 1, "|" );

  aux += "|                                          |\n";
  aux += "|                                          |\n";

  // Nmero de casilla
  char numAlpha[20];
  sprintf ( numAlpha, "%d", get_numero() );
  if ( get_numero() >= 10 )
    aux += "|                                       " + string ( numAlpha )  + " |\n";
  else
    aux += "|                                        " + string ( numAlpha )  + " |\n";    

  // Cerrar el rectngulo
  aux += "|__________________________________________|\n";

  return aux;
}

/**
 * Mtodo que dibuja la fila indicada de la casilla correspondiente (en las filas de arriba y abajo del tablero).
 * Se indicar la fila que se tiene que dibujar y el ancho con el que se tiene que pintar. Como resultado se
 * devolver una cadena de caracteres.
 */
string servicio::dibujarCasillaHorizontal ( int fila, int anchoCasilla )
{
  string sFila;  
  const int MaxCasillasAncho = 13;
  
  // Se dibujara segn la fila que se indique lo siguiente:
  switch ( fila )
    {

    // Inicio del rectangulo
    case 0:
      sFila = " ";				//Primer caracter
      for ( int i = 1; i < anchoCasilla ; i++ )
	sFila += "_";
      break;
    
    // Nmero de casilla e icono de la misma (en este caso una S)
    case 1:
      {
	char numAlpha[20];
	sprintf( numAlpha, "%d", get_numero() );
	sFila = string ( numAlpha );       
	while ( sFila.size() <= ( anchoCasilla - 3 ) )
	  sFila += " ";	
	sFila += "S|";
	break;
      }

    // Nombre corto y estado d la casilla.
    case 2:
      // Nombre corto
      sFila = get_nombreCorto();

      // Si no pertenece a un jugador, esta libre (L), si no, esta comprada (C)
      if ( ! dynamic_cast <jugador *> ( get_esPropiedad() ) )	
	sFila +="(L)";
      else
	sFila +="(C)";       

      while ( sFila.size() <= ( anchoCasilla - 2 ) )
	sFila += " ";      
      sFila += "|";
      break;
    
    // Primeros tres jugadores que esten en la casilla.
    case 3:
      {
	int i = 0;
	string aux = "";	
	int numJugadores = estaJugador.size ();
	// Recorre los primeros 3 elementos de la lista de jugadores
	while ( ( numJugadores > 0 ) && ( i < numJugadores ) && ( i < 3 ) )
	  {
	    string color = estaJugador[i]->get_color();
	    string colorAbreviado = color.substr(0,2);
	    //Convierte a mayusculas el color que se muestra
	    transform (colorAbreviado.begin(), colorAbreviado.end(), colorAbreviado.begin(), (int(*)(int))toupper);
	    aux += colorAbreviado;	    
	    i++;
	    if ( i < 3 ) 
	      aux += " ";
	  }
	while (aux.size() < anchoCasilla - 1 )
	  aux += " ";
	sFila += aux + '|';
	break;
      }

    // Si hay ms de tres jugadores, se indicar aqu
    case 4:      
      {
	int i = 3;
	string aux = "";	
	int numJugadores = estaJugador.size ();
	//Recorrer la lista de jugadores empezando en el tercero
	while ( ( numJugadores > 3 ) && ( i < numJugadores ) && ( i < 6 ) )
	  {	   
	    string color = estaJugador[i]->get_color();
	    string colorAbreviado = color.substr(0,2);
	    //Convertir a mayusculas
	    transform (colorAbreviado.begin(), colorAbreviado.end(), colorAbreviado.begin(), (int(*)(int))toupper);
	    aux += colorAbreviado;	    
	    i++;
	    if ( i < 6 ) 
	      aux += " ";	
	  }	
	while (aux.size() < anchoCasilla - 1 )
	  aux += " ";
	sFila += aux + '|';
	break;
      }

    // Cerrar el rectangulo dibujado
    case 5:
      while ( sFila.size() <= ( anchoCasilla - 2 ) )
	sFila += "_";	 	
      sFila += "|";
      break;    
    }
  
  return sFila;
}

/**
 * Mtodo que dibuja la fila indicada de la casilla correspondiente (en las columnas derecha e izquierda del tablero).
 * Se indicar la fila que se tiene que dibujar y el ancho con el que se tiene que pintar. Como resultado se
 * devolver una cadena de caracteres.
 */
string servicio::dibujarCasillaVertical ( int fila, int anchoCasilla )
{
  string sFila;  
  int const MaxCasillasAncho = 13;
  int const MaxCasillasAlto = 8;
  
  sFila = "|";
  switch ( fila )
    {    
    // Muestra el nmero, el icono de servicio y los primeros 3 jugadores que estn en la casilla
    case 0:
      {	
	string aux = "";

	// Numero
	char numAlpha[20];
	sprintf( numAlpha, "%d", get_numero() );
	aux += string ( numAlpha );	
	// Icono
	aux += " S  ";
	if ( get_numero() < 10 )
	  aux +=  " ";	
	int i = 0;
	
	//Recorre los primeros 3 jugadores de la lista mostrando su ficha
	int numJugadores = estaJugador.size ();
	while ( ( numJugadores > 0 ) && ( i < numJugadores ) && ( i < 3 ) )
	  {
	    string color = estaJugador[i]->get_color();
	    aux += color.substr(0,2);
	    i++;
	    if ( i < 3 ) 
	      aux += " ";	
	  }
	while (aux.size() < anchoCasilla - 1 )
	  aux += " ";
	sFila += aux + '|';      
	break;
      }

    // Muestra el nombre corto de la casilla, el estado y los jugadores por detrs del tercero si los hay
    case 1:
      {
	string aux = "";
	
	// Nombre corto
	aux += get_nombreCorto();

	// Estado de la casilla (con o sin propietario)
	if ( ! dynamic_cast <jugador *> ( get_esPropiedad() ) )	  
	  aux +="(L)";
	else
	  aux +="(C)";     
	
	while ( aux.size() < 6 ) aux += " ";
	
	// Recorre la lista de jugadores a partir del tercero
	int i = 3;
	int numJugadores = estaJugador.size ();
	while ( ( numJugadores > 3 ) && ( i < numJugadores ) && ( i < 6 ) )
	  {
	    string color = estaJugador[i]->get_color();
	    string colorAbreviado = color.substr(0,2);
	    // Convertir a maysculas
	    transform (colorAbreviado.begin(), colorAbreviado.end(), colorAbreviado.begin(), (int(*)(int))toupper);
	    aux += colorAbreviado;	    
	    i++;
	    if ( i < 6 ) 
	      aux += " ";	
	  }	
	while (aux.size() < anchoCasilla - 1 )
	  aux += " ";
	sFila += aux + '|';
	break;
      }
      
    case 2:      
      while ( sFila.size() <= ( anchoCasilla - 1) )
	sFila += "_";      
      sFila += "|";
      break;  
      
      return sFila;
    }
}

/**
 * Devuelve la informacin de la casilla en modo texto para mostrar por pantalla la informacin
 * en forma de tabla. 
 */
string servicio::leyendaCasilla ( int num, int nombreCorto, int tipo, int nombre, int estado )
{
  string aux;

  // Numero de casilla
  int numeroCasilla = get_numero();
  char numAlpha[30];
  sprintf (numAlpha, "%d", numeroCasilla );
  string numero = numAlpha;
  aux = numero.substr ( 0, num );
  for ( int i = numero.size(); i<=num; i++ )
    aux += " ";

  // Nombre corto
  string sNombreCorto = get_nombreCorto();
  aux += sNombreCorto.substr ( 0, nombreCorto );
  for ( int i = sNombreCorto.size (); i<=nombreCorto; i++ )
    aux += " ";

  // Icono de casilla
  string sTipo = "S";
  aux += sTipo.substr ( 0, tipo );
  for ( int i = sTipo.size (); i<=tipo; i++ )
    aux += " ";
  string sNombre = get_nombre();
  aux += sNombre.substr ( 0, nombre );
  for ( int i = sNombre.size (); i<=nombre; i++ )
    aux += " ";

  // Estado actual
  string sEstado;
  if ( ! dynamic_cast <jugador *> ( get_esPropiedad() ) )
    sEstado +="L";
  else
    sEstado +="C";     
  aux += sEstado;

  for ( int i = sEstado.size (); i<=estado; i++ )
    aux += " ";

  return aux;
}

/**
 * Mtodo polimrfico de la clase propiedad. Calcula el alquiler para el jugador dado
 * a partir de su ultima tirada y del nmero de propiedades de este tipo que tenga.
 */
long servicio::calcularAlquiler ( jugador *j )
{
  monopolio *mp = get_perteneceMonopolio();
  
  // Calcula el numero de servicios que posee el propietario
  int numServicios = mp->contarPropiedadesConMismoPropietario ( this );  
  int multiplica = 0;
  
  // Obtiene el multiplicador
  switch ( numServicios )
      {
      case 1:
	multiplica = get_multiplicador1 ();
	break;
      case 2:
	multiplica = get_multiplicador2 ();
	break;      
      default:
	multiplica = get_multiplicador1() * numServicios;
      }  
   
  // Obtiene la ltima tirada del dado del jugador.
  int tirada = j->ultimaTirada();

  // Calcula el alquiler
  int precio = tirada * multiplica;

  return precio;
}
