/***********************************************************************************
 *  CLASE SERVICIO
 *  Clase que representa una casilla del tipo servicio del juego de Monopoly. Esta 
 *  casilla es una clase derivada de propiedad ya que se puede comprar, hipotecar
 *  y cobrar un alquiler.
 *  La carcateristica fundamental de las propiedades de este tipo es que su alquiler
 *  se calcula a partir de un multiplicador y el valor obtenido al lanzar los dados.
 *
 *  Requiere:
 *     - clase propiedad ( jerarquia )
 *     - clase string (STL) 
 *
 *  Autor: Javier Aragn Zabalegui (PFC para la uc3m: street Master's)
 *  Contacto: 100039831@alumnos.uc3m.es
 *  version 1.0
 **********************************************************************************/


#ifndef __SERVICIO__
#define __SERVICIO__

#include "propiedad.h"

#include <string>
using namespace std;


class servicio : public propiedad {

 private:

  /**
   * Multiplicadores de dinero para calcular el alquiler.
   */
  int _multiplicador1;
  int _multiplicador2;  

 public:

  // Constructor por defecto
  servicio ();
  
  // Constructor explcito
  servicio ( int numero, string nombre, string nombreCorto, long precio, long hipoteca, bool hipotecado, 
	     bool hipotecable, int multiplicador1, int multiplicador2, propietario *p );
  
  // Copy constructor
  servicio (const servicio& right);

  // Operador de asignacin
  servicio& operator= (const servicio& right);

  // Sobrecarga del operador de salida
  friend ostream& operator<< ( ostream& os, servicio *s );

  // Destructor
  ~servicio ();

  // Mtodos GET
  int get_multiplicador1 () const;
  int get_multiplicador2 () const;
  
  // Mtodos SET
  void set_multiplicador1 ( int multiplicador1 );
  void set_multiplicador2 ( int multiplicador2 );

  /**
   * Mtodo que devuelve en una cadena de caracteres toda la informacin necesaria, con un
   * formato preparado para mostrarse por pantalle, toda la informacin de un servicio, que 
   * incluye:
   *    -Nombre
   *    -Estado (hipotecado, libre ...)
   *    -Propietario
   *    -Nmero de casilla
   *    -Informacin sobre los precios de alquiler
   *    -Precio de compra e hipoteca
   * Representa la tarjeta de propiedad de una propiedad de tipo servicio.
   */
  string mostrar ();

  /**
   * Mtodo que dibuja la fila indicada de la casilla correspondiente (en las filas de arriba y abajo del tablero).
   * Se indicar la fila que se tiene que dibujar y el ancho con el que se tiene que pintar. Como resultado se
   * devolver una cadena de caracteres.
   */
  string dibujarCasillaHorizontal ( int fila, int anchoCasilla );

  /**
   * Mtodo que dibuja la fila indicada de la casilla correspondiente (en las columnas derecha e izquierda del tablero).
   * Se indicar la fila que se tiene que dibujar y el ancho con el que se tiene que pintar. Como resultado se
   * devolver una cadena de caracteres.
   */
  string dibujarCasillaVertical ( int fila, int anchoCasilla );
 
  /**
   * Devuelve la informacin del servicio en modo texto para mostrar por pantalla la informacin
   * en forma de tabla. 
   */
  string leyendaCasilla ( int num, int nombreCorto, int tipo, int nombre, int estado );

  /**
   * Mtodo polimrfico de la clase propiedad. Calcula el alquiler para el jugador dado
   * a partir de su ultima tirada y del nmero de propiedades de este tipo que tenga.
   */
  long calcularAlquiler ( jugador *j );
};

inline int servicio::get_multiplicador1 () const
{
  return _multiplicador1;
}

inline int servicio::get_multiplicador2 () const
{
  return _multiplicador2;
}
  
  //Set operators

inline void servicio::set_multiplicador1 ( int multiplicador1 )
{
  _multiplicador1 = multiplicador1;
}

inline void servicio::set_multiplicador2 ( int multiplicador2 )
{
  _multiplicador2 = multiplicador2;
}

#endif
