
#include "tablero.h"

vector<casilla*> tablero::contiene;
vector<jugador*> tablero::juega;
int tablero::_numCasillas = 0;
int tablero::_salida = 0;
int tablero::_turno = 0;
banca* tablero::_banca = NULL;
bool tablero::_subastas = false;

/**
 * IMPLEMENTACIN DE MTODOS
 */

/**
 * Constructor por defecto
 */
tablero::tablero ()
  : _nombrePartida ( "desconocido" ), 
    _ciudad ( "Madrid" ),
    _moneda ( "pesetas" ),
    _fichCasillas ( "" ),
    _fichSuerte ( "" ),
    _fichCajaComunidad ( "" ),
    _dineroInicial ( 250000),
    _numJugadores ( 0 ),
    //_numCasillas ( 0 ),
    _partidaAbierta ( false ),
    _jugadorActual ( 0 )    
    //_banca ( NULL )
{
  //Atributos estaticos
  //_turno = 0;
  //_salida = -1;
  _banca = NULL;
  _subastas = false;
  _numCasillas = 0;
}

/**
 * Constructor explcito
 */
tablero::tablero ( string nombrePartida, string ciudad, string moneda, string fichCasillas, 
		   string fichSuerte, string fichCajaComunidad, int numJugadores, int numCasillas,
		   bool partidaAbierta, int salida, int turno, int jugadorActual, long dineroInicial,
		   bool subastas )
  : _nombrePartida ( nombrePartida ), 
    _ciudad ( ciudad ),
    _moneda ( moneda),
    _fichCasillas ( fichCasillas ),
    _fichSuerte ( fichSuerte ),
    _fichCajaComunidad ( fichCajaComunidad ),
    _dineroInicial ( dineroInicial),
    _numJugadores ( numJugadores ),    
    //_numCasillas ( numCasillas ),
    _partidaAbierta ( partidaAbierta),
    _jugadorActual ( jugadorActual )    
{
  //Atributos estaticos
  tablero::_salida = salida;
  _turno =turno;
  _numCasillas = numCasillas;
  _subastas = subastas;
}

/**
 * Copy constructor
 */
tablero::tablero (const tablero& right)
{
  _nombrePartida	= right._nombrePartida;
  _ciudad		= right._ciudad;
  _moneda		= right._moneda;
  _fichCasillas		= right._fichCasillas;
  _fichSuerte		= right._fichSuerte;
  _fichCajaComunidad	= right._fichCajaComunidad;
  _dineroInicial	= right._dineroInicial;
  _numJugadores		= right._numJugadores;
  _numCasillas		= right._numCasillas;
  _partidaAbierta	= right._partidaAbierta;
  _salida	        = right._salida;
  _turno		= right._turno;
  _jugadorActual	= right._jugadorActual;
  _banca                = right._banca;
  _subastas             = right._subastas;
}

/**
 * Assignment operator
 */
tablero& tablero::operator= ( const tablero& right )
{
  _nombrePartida	= right._nombrePartida;
  _ciudad		= right._ciudad;
  _moneda		= right._moneda;
  _fichCasillas		= right._fichCasillas;
  _fichSuerte		= right._fichSuerte;
  _fichCajaComunidad	= right._fichCajaComunidad;
  _dineroInicial	= right._dineroInicial;
  _numJugadores		= right._numJugadores;
  _numCasillas		= right._numCasillas;
  _partidaAbierta	= right._partidaAbierta;
  _salida       	= right._salida;
  _turno		= right._turno;
  _jugadorActual	= right._jugadorActual;
  _banca                = right._banca;
  _subastas             = right._subastas;
  return ( *this );
}

/**
 * Destructor
 */
tablero::~tablero ()
{
  // No se hace nada!
}

/**
 * Mtodo que retorna el nombre del jugador que ocupa la posicion indicada en el argumento 
 * 'numeroJugador'
 */
string tablero::obtenerNombreJugador ( int numeroJugador )
{
  return juega[ numeroJugador ]->get_nombre();
}


/**
 *-------------------------CARGAR TABLERO------------------------------------------
 *
 * Mtodos que sirven para cargar las casillas de un tablero
 */

/** 
 * Toma una linea e identifica de que tipo de casilla se trata. 
 */
int identificar_casilla ( string linea )
{
  //Linea vaca
  if ( linea.size () == 0 ) 
    return 0;
  if ( linea.substr ( 0, 5 ) == Ccalle )
    return 1;
  else if ( linea.substr( 0, 8 ) == Cservicio )
    return 2;
  else if ( linea.substr ( 0, 8 ) == Cestacion )
    return 3;
  else if ( linea.substr ( 0, 7 ) == Cparking )    
    return 4;
  else if ( linea.substr ( 0, 17 ) == Cimpuesto_especial )
    return 5;
  else if ( linea.substr ( 0, 8 ) == Cimpuesto )
    return 6;
  else if ( linea.substr ( 0, 7 ) == Ctarjeta )
    return 7;  
  else if ( linea.substr ( 0, 6 ) == Csalida )
    return 8;
  else if ( linea.substr ( 0, 11 ) == Ca_la_carcel )
    return 9;
  else if ( linea.substr ( 0, 6 ) == Ccarcel )
    return 10;
  else     
    return -1;
}

/** 
 * Mtodos que obtienen los diferentes atributos de cada tipo de casilla. Los atributos 
 * vendrn dados por una cadena de caracteres (de la clase string de C++), y sern 
 * aislados mediante los mtodos de la familia obtener_substring_delimitado incluidos 
 * en la clase cadenas.    
 * Todos estos mtodos seguirn el mismo proceso:
 *  	- Se indica el valor en el que comienzan a leerse los atributos (tipicamente 
 *	la longitud del identificador del tipo de casilla + 1)
 *	- Se leer el primer atributo iniciandose en el valor indicado en el punto 
 *	anterior y se devolver la posicin donde comenzar el siguiente atributo.
 *	-En la siguiente lectura el inicio ser el final del anterior. Por ello en todos 
 *	las llamadas se irn 
 *	intercambiando las variables pos1 y pos2.
 */

/**
 * Lee del fichero del tablero los datos referentes a una calle. Cada uno de estos
 * campos pasaran a formar parte de una instancia de casilla-calle 
 */
int tablero::casilla_tipo_calle ( string linea )
{    
  string::size_type pos1, pos2;  //Posiciones de inicio y fin de cada substring. Se rotarn para leer cada substring
  
  //NMERO DE CASILLA
  int numero_casilla = obtener_substring_delimitado_int ( linea, "|", 6, pos1 );  

  //NOMBRE
  string nombre = obtener_substring_delimitado_st ( linea, "|", pos1, pos2 ); 

  //NOMBRE CORTO
  string nombreCorto = obtener_substring_delimitado_st ( linea, "|", pos2, pos1 );
  nombreCorto = nombreCorto.substr (0,3);

  //COLOR
  string color = obtener_substring_delimitado_st ( linea, "|", pos1, pos2 );

  //PRECIO
  long precio = obtener_substring_delimitado_long ( linea, "|", pos2, pos1 ); 

  //HIPOTECA
  long hipoteca = obtener_substring_delimitado_long ( linea, "|", pos1, pos2 ); 

  //ALQUILERES:
  //0 casas
  long alquiler0 = obtener_substring_delimitado_long ( linea, "|", pos2, pos1 ); 

  //1 casa
  long alquiler1 = obtener_substring_delimitado_long ( linea, "|", pos1, pos2 ); 

  //2 casas
  long alquiler2 = obtener_substring_delimitado_long ( linea, "|", pos2, pos1 ); 

  //3 casas
  long alquiler3 = obtener_substring_delimitado_long ( linea, "|", pos1, pos2 ); 

  //4 casas
  long alquiler4 = obtener_substring_delimitado_long ( linea, "|", pos2, pos1 ); ;

  //hotel
  long alquiler_h = obtener_substring_delimitado_long ( linea, "|", pos1, pos2 ); 

  //precios de casas y hoteles
  //precio_casa
  int precio_casa = obtener_substring_delimitado_int ( linea, "|", pos2, pos1 ); 
  
  //precio_hotel
  int precio_hotel = obtener_substring_delimitado_int ( linea, "|", pos1, pos2 ); 
  
  //Crear una calle con todos los datos anteriores
  incluir_calle ( numero_casilla, nombre, nombreCorto, color, precio, hipoteca, 
			  alquiler0, alquiler1, alquiler2, alquiler3, alquiler4, 
			  alquiler_h, precio_casa, precio_hotel );
 
  return ( 1 );
}

/** 
 * Lee del fichero del tablero los datos referentes a una casilla de propiedad de un servicio.
 * Cada uno de estos campos pasarn a formar parte de una instancia de casilla-propiedad-servicio 
 */
int tablero::casilla_tipo_servicio ( string linea )
{
  string::size_type pos1, pos2;  //Posiciones de inicio y fin de cada substring. Se rotarn para leer cada substring

  //NUMERO
  int numero_casilla = obtener_substring_delimitado_int ( linea, "|", 9, pos1 ); 
  
  //NOMBRE
  string nombre = obtener_substring_delimitado_st ( linea, "|", pos1, pos2 ); 

  //NOMBRE CORTO
  string nombreCorto = obtener_substring_delimitado_st ( linea, "|", pos2, pos1 );
  nombreCorto = nombreCorto.substr (0,3);
  
  //CANTIDAD
  int precio = obtener_substring_delimitado_int ( linea, "|", pos1, pos2 );

  //HIPOTECA
  long hipoteca = obtener_substring_delimitado_long ( linea, "|", pos2, pos1 );

  //MULTIPLICADOR CUANDO SE POSEE 1 SERVICIO
  int multiplicador1 = obtener_substring_delimitado_int ( linea, "|", pos1, pos2 );
  
  //MULTIPLICADOR CUANDO SE POSEEN 2 SERVICIOS
  int multiplicador2 = obtener_substring_delimitado_int ( linea, "|", pos2, pos1 );

  incluir_servicio ( numero_casilla, nombre, nombreCorto, precio, hipoteca, multiplicador1, 
			     multiplicador2 );

  return ( 1 );
}


/** 
 * Lee del fichero del tablero los datos referentes a una estacion. Cada uno de estos
 * campos pasaran a formar parte de una instancia de casilla-estacion 
 */
int tablero::casilla_tipo_estacion ( string linea )
{    
  string::size_type pos1, pos2;  //Posiciones de inicio y fin de cada substring. Se rotarn para leer cada substring
  
  //NMERO DE CASILLA
  int numero_casilla = obtener_substring_delimitado_int ( linea, "|", 9, pos1 );  

  //NOMBRE
  string nombre = obtener_substring_delimitado_st ( linea, "|", pos1, pos2 ); 

  //NOMBRE CORTO
  string nombreCorto = obtener_substring_delimitado_st ( linea, "|", pos2, pos1 );
  nombreCorto = nombreCorto.substr (0,3);

  //PRECIO
  long precio = obtener_substring_delimitado_long ( linea, "|", pos1, pos2 ); 

  //HIPOTECA
  long hipoteca = obtener_substring_delimitado_long ( linea, "|", pos2, pos1 ); 

  //ALQUILERES:

  //1 estacion
  long alquiler1 = obtener_substring_delimitado_long ( linea, "|", pos1, pos2 ); 

  //2 estaciones
  long alquiler2 = obtener_substring_delimitado_long ( linea, "|", pos2, pos1 ); 

  //3 estaciones
  long alquiler3 = obtener_substring_delimitado_long ( linea, "|", pos1, pos2 ); 

  //4 estaciones
  long alquiler4 = obtener_substring_delimitado_long ( linea, "|", pos2, pos1 );
  
  //Crear una calle con todos los datos anteriores
  incluir_estacion ( numero_casilla, nombre, nombreCorto, precio, hipoteca, 
			     alquiler1, alquiler2, alquiler3, alquiler4 );
 
  return ( 1 );
}


/**
 * Lee del fichero del tablero los datos referentes a una casilla de impuestos.
 * Cada uno de estos campos pasaran a formar parte de una instancia de casilla-impuesto 
 */
int tablero::casilla_tipo_impuesto ( string linea )
{
  string::size_type pos1, pos2;  //Posiciones de inicio y fin de cada substring. Se rotarn para leer cada substring
  //NUMERO
  int numero_casilla = obtener_substring_delimitado_int ( linea, "|", 9, pos1 ); 
  
  //NOMBRE
  string nombre = obtener_substring_delimitado_st ( linea, "|", pos1, pos2 ); 

  //NOMBRE CORTO
  string nombreCorto = obtener_substring_delimitado_st ( linea, "|", pos2, pos1 );
  nombreCorto = nombreCorto.substr (0,3);
  
  //CANTIDAD
  int precio = obtener_substring_delimitado_int ( linea, "|", pos1, pos2 );
    
  incluir_impuesto ( numero_casilla, nombre, nombreCorto, precio );
  return ( 1 );
}

/** 
 * Lee del fichero del tablero los datos referentes a un impuesto especial (con %). 
 * Cada uno de estos campos pasaran a formar parte de una instancia de 
 * casilla-impuesto-impuesto5 
 */
int tablero::casilla_tipo_impuesto_especial ( string linea )
{
  string::size_type pos1, pos2;  //Posiciones de inicio y fin de cada substring. Se rotarn para leer cada substring
  //NUMERO
  int numero_casilla = obtener_substring_delimitado_int ( linea, "|", 18, pos1 ); 
  
  //NOMBRE
  string nombre = obtener_substring_delimitado_st ( linea, "|", pos1, pos2 ); 

  //NOMBRE CORTO
  string nombreCorto = obtener_substring_delimitado_st ( linea, "|", pos2, pos1 );
  nombreCorto = nombreCorto.substr (0,3);
  
  //TANTO POR CIENTO
  int tpc = obtener_substring_delimitado_int ( linea, "|", pos1, pos2 );

  //PRECIO FIJO
  int precio = obtener_substring_delimitado_int ( linea, "|", pos2, pos1 );
  
  incluir_impuesto_especial ( numero_casilla, nombre, nombreCorto, precio, tpc );
  return ( 1 );
}

/** 
 * Lee del fichero del tablero los datos referentes a una casilla de tarjetas. 
 * Cada uno de estos campos pasaran a formar parte de una instancia de 
 * casilla-casilla_tarjetas 
 */
int tablero::casilla_tipo_tarjetas ( string linea )
{
  string::size_type pos1, pos2;  //Posiciones de inicio y fin de cada substring. Se rotarn para leer cada substring
  //NUMERO
  int numero_casilla = obtener_substring_delimitado_int ( linea, "|", 8, pos1 ); 
  
  //NOMBRE
  string nombre = obtener_substring_delimitado_st ( linea, "|", pos1, pos2 ); 

  //NOMBRE CORTO
  string nombreCorto = obtener_substring_delimitado_st ( linea, "|", pos2, pos1 );
  nombreCorto = nombreCorto.substr (0,3);
  
  //MAZO RELACIONADO
  string mazo = obtener_substring_delimitado_st ( linea, "|", pos1, pos2 );
  
  incluir_casilla_tarjetas ( numero_casilla, nombre, nombreCorto, mazo );
  return ( 1 );
}


/** 
 * Lee del fichero del tablero los datos referentes a una casilla de salida. 
 * Cada uno de estos campos pasaran a formar parte de una instancia de 
 * casilla-salida 
 */
int tablero::casilla_tipo_salida ( string linea )
{
  string::size_type pos1, pos2;  //Posiciones de inicio y fin de cada substring. Se rotarn para leer cada substring
  //NUMERO
  int numero_casilla = obtener_substring_delimitado_int ( linea, "|", 7, pos1 ); 
  
  //NOMBRE
  string nombre = obtener_substring_delimitado_st ( linea, "|", pos1, pos2 ); 
  
  //NOMBRE CORTO
  string nombreCorto = obtener_substring_delimitado_st ( linea, "|", pos2, pos1 );
  nombreCorto = nombreCorto.substr (0,3);

  //SUELDO
  long sueldo = obtener_substring_delimitado_long ( linea, "|", pos1, pos2 );  
  
  incluir_salida ( numero_casilla, nombre, nombreCorto, sueldo );
  return ( 1 );
}


/** 
 * Lee del fichero del tablero los datos referentes a una casilla de tipo parking. 
 * Las casillas de parking libre de pago, no tienen caractersticas propias, por 
 * lo que sern instancias directas de la clase CASILLA.
 */
int tablero::casilla_tipo_parking ( string linea )
{
  string::size_type pos1, pos2;  //Posiciones de inicio y fin de cada substring. Se rotarn para leer cada substring
  //NUMERO
  int numero_casilla = obtener_substring_delimitado_int ( linea, "|", 8, pos1 ); 
  
  //NOMBRE
  string nombre = obtener_substring_delimitado_st ( linea, "|", pos1, pos2 ); 

  //NOMBRE CORTO
  string nombreCorto = obtener_substring_delimitado_st ( linea, "|", pos2, pos1 );
  nombreCorto = nombreCorto.substr (0,3);
  
  incluir_casilla_generica ( numero_casilla, nombre, nombreCorto );
  return ( 1 );
}


/** 
 * Lee del fichero del tablero los datos referentes a una casilla de tipo "a la carcel".
 * Creara una instancia de la clase encarceladora y se incluir en el tablero
 */
int tablero::casilla_tipo_encarceladora ( string linea )
{
  string::size_type pos1, pos2;  //Posiciones de inicio y fin de cada substring. Se rotarn para leer cada substring
  //NUMERO
  int numero_casilla = obtener_substring_delimitado_int ( linea, "|", 12, pos1 ); 
  
  //NOMBRE
  string nombre = obtener_substring_delimitado_st ( linea, "|", pos1, pos2 ); 

  //NOMBRE CORTO
  string nombreCorto = obtener_substring_delimitado_st ( linea, "|", pos2, pos1 );
  nombreCorto = nombreCorto.substr (0,3);
 
  incluir_encarceladora ( numero_casilla, nombre, nombreCorto );
  return ( 1 );
}


/** 
 * Lee del fichero del tablero los datos referentes a una casilla de tipo carcel. 
 * Con los datos obtenidos se crea una casilla de tipo carcel que se insetar
 * en el tablero.
 */
int tablero::casilla_tipo_carcel ( string linea )
{
  string::size_type pos1, pos2;  //Posiciones de inicio y fin de cada substring. Se rotarn para leer cada substring
  //NUMERO
  int numero_casilla = obtener_substring_delimitado_int ( linea, "|", 7, pos1 ); 
  
  //NOMBRE
  string nombre = obtener_substring_delimitado_st ( linea, "|", pos1, pos2 ); 

  //NOMBRE CORTO
  string nombreCorto = obtener_substring_delimitado_st ( linea, "|", pos2, pos1 );
  nombreCorto = nombreCorto.substr (0,3);

  //MXIMO DE TURNOS
  int maxTurnos = obtener_substring_delimitado_int ( linea, "|", pos1, pos2 ); 
  
  //MULTA
  int multa = obtener_substring_delimitado_long ( linea, "|", pos2, pos1 ); 
  
  incluir_carcel ( numero_casilla, nombre, nombreCorto, maxTurnos, multa );
  return ( 1 );
}


/** 
 * Mtodo que lee del fichero de casillas todas las casillas y crear la instancia de casillas
 * correspondiente 
 *  1- Leer del fichero una linea (casilla)
 *  2- Identificar que tipo de casilla es
 *  3- Crear la instancia 
 */
int tablero::cargar_casillas ()
{
  int ok;  
  string ftablero = get_fichCasillas ();
  /* Abrir fichero para lectura */
  fstream ifs ( ftablero.c_str (), ios::in );
  if ( !ifs )
    {
      cerr << "Fichero de tablero no disponible. Compruebe que exista " << ftablero << endl;
      return -1;
    }  
  //leer datos del fichero
  //1 linea = 1 casilla
  const int N=200;  
  string linea;
  int ncars=0, i, cont_linea = 0;  
  do
    {
      cont_linea++;
      getline ( ifs, linea, '\n' );     //lee una linea
      if ( ifs.eof() ) break;		//fin de fichero      
      /* Tratamos la linea leida */
      /* Elimina espacios en blanco y comentarios de una linea */
      i=0;
      while ( ( linea[i] != '#') && (i < linea.size () ) )
	i++;	 
      /* Limpia los comentarios detectados y los espacios en blanco */
      linea = linea.substr ( 0, i );
      linea = stripwhite_st ( linea );
      
      // Carga la ruta indicada en el fichero de configuracion en la variable correspondiente
      int tipo_casilla = identificar_casilla ( linea );
      switch ( tipo_casilla )
	{
	case 1: //calle
	  ok = casilla_tipo_calle ( linea );	  
	  break;
	case 2: //servicio
	  ok = casilla_tipo_servicio ( linea );
	  break;
	case 3: //estacion
	  ok = casilla_tipo_estacion ( linea );
	  break;
	case 4: //parking
	  ok = casilla_tipo_parking ( linea );
	  break;
	case 5: //impuesto_especial
	  ok = casilla_tipo_impuesto_especial ( linea );
	  break;
	case 6: //impuesto
	  ok = casilla_tipo_impuesto ( linea );
	  break;
	case 7: //casillas de tarjetas (suerte y caja_comunidad)
	  ok = casilla_tipo_tarjetas ( linea );
	  break;	
	case 8: //salida
	  ok = casilla_tipo_salida ( linea );
	  break;
	case 9: //a_la_carcel
	  ok = casilla_tipo_encarceladora ( linea );
	  break;
	case 10: //carcel
	  ok = casilla_tipo_carcel ( linea  );
	  break;
	case -1: //error
	  cerr << "Error en la linea " << cont_linea << " del fichero " << ftablero << endl;
	  cerr << "LINEA: " << linea << endl;
	  cerr << "Que desea hacer? (O)mitir (C)ancelar ";
	  char opcion = getchar ();	  
	  if ( toupper(opcion) != 'O' )
	    return -1;
	  cin.clear(); //limpio las banderas de cin. Colocarlas en estado de bueno
	  cin.ignore( 2000, '\n' ); // vacia el buffer de la entrada ya que contiene "basura"
	}
      if ( ( ncars = ifs.gcount() ) == N-1 ) //se activo el failbit
	ifs.clear();     
    } while ( true );
  
  // LLama a verificar para comprobar que las casillas estn colocadas en orden correcto
  if ( verificar () == false )
    {
      char preguntarRecolocar;
      cout << endl;
      cout << "Se han detectado que aparecen numeros de casilla vacios o repetidos" << endl;
      cout << "Desea que Street Master's las recoloque automticamente?(recomendado)(s/n)";
      preguntarRecolocar = getchar();
      if ( toupper ( preguntarRecolocar ) != 'N' )
	{	  
	  cin.clear();
	  cin.ignore( 2000, '\n' );
	  cout << "Recolocando las casillas...";
	  recolocar ();
	  cout << "HECHO" << endl;
	}
      else
	{
	  cerr << "El proceso no puede continuar, repare el fichero de casillas o seleccione uno nuevo y vuelva a iniciar el proceso" << endl;
	  cin.clear();
	  cin.ignore( 2000, '\n' );
	  return -1;
	}
    }
  if ( !ifs.eof() && ifs.fail() )
    {
      cerr << "Error al leer el fichero de ayuda\n";
      return -1;
    }
  ifs.clear();
  return 0;
}

/**
 * Mtodo que guarda la informacin sobre una partida en el fichero indicado. Se guardarn los datos
 * generales de la partida y los de los jugadores y casillas que se incluyen en ella.
 */
int tablero::guardar ( string fichero )
{
  ofstream fs ( fichero.c_str() );
  // 1- DATOS DE LA PARTIDA
  fs << "NOMBRE=" << fichero << endl;
  fs << "CIUDAD= " << get_ciudad () << endl;
  fs << "MONEDA= " << get_moneda () << endl;
  fs << "SUERTE= " << get_fichSuerte () << endl;
  fs << "CAJA COMUNIDAD= " << get_fichCajaComunidad () << endl;
  fs << "TABLERO= " << get_fichCasillas () << endl;
  fs << "SALIDA= " << get_salida () << endl;
  fs << "TURNO= " << get_turno () << endl;
  fs << "JUGADOR ACTUAL= " << get_jugadorActual () << endl;
  fs << "SUBASTAS= " << get_subastas () << endl;
  fs << "CASAS DISPONIBLES= " << _banca->get_casasDisponibles () << endl;
  fs << "HOTELES DISPONIBLES= " << _banca->get_hotelesDisponibles () << endl << endl; 

  //2- DATOS DE LOS JUGADORES
  for ( int i = 0; i < get_numJugadores (); i++ )
    {
      if ( dynamic_cast <jugador_reglas*> ( juega[i] ) )
	{
	  jugador *j = juega[i];
	  fs << "JUGADOR AUTO=" << endl;
	  fs << j->guardar () << endl;      
	}
      else
	{
	  jugador *j = juega[i];
	  fs << "JUGADOR=" << endl;
	  fs << j->guardar () << endl;  
	}
    }
  
  //3- DATOS DE LAS PROPIEDADES
  //   -Adems de los datos que se introducen en el fichero de casillas, se debe indicar su estado,
  //    si estan hipotecadas, as como el nmero de casas y hoteles.
  for ( int i = 0; i < get_numCasillas (); i++ )
    {
      casilla *c = contiene[i];
      if ( propiedad *p = dynamic_cast <propiedad*> ( contiene [ i ] ) )
	fs << p->guardar () << endl;
    }
  fs.close();
  return 0;
}

/**
 * Carga la liste de propiedades en un jugador. La lista viene dada con el formato:
 *     numeroPropiedad, numeroPropiedad, numeroPropiedad...
 */
void tablero::cargarPropiedades ( jugador *j, string lista )
{
  string::size_type pos1 = 0, pos2;
  list <propiedad*> listaPropiedades;
  int i = 0;
  do
    {      
      string num = "";
      pos2 = lista.find ( ",", 0 );
      if ( pos2 != string::npos )
	num = lista.substr ( pos1, pos2 );
      else
	  num = lista.substr ( pos1, lista.size() );
      num = stripwhite_st ( num );
      lista = lista.substr ( pos2 + 1, lista.size() );
      lista = stripwhite_st ( lista );      
      if ( propiedad *p = dynamic_cast <propiedad*> ( contiene[ atoi( num.c_str() ) ] ) )
	listaPropiedades.push_front ( p );     
      pos1 = 0;
      i++;
    }
  while ( pos2 != string::npos );
  j->anyadirListaPropiedades ( listaPropiedades );
  return;
}

/**
 * Carga la liste de tarjetas en un jugador. La lista viene dada con el formato:
 *     numeroTarjeta (mazo), numeroTarjeta (mazo)...
 */
void tablero::cargarTarjetas ( jugador *j, string lista )
{  
  string::size_type pos1 = 0, pos2, posParentesis, posParentesisCerrado;
  list <tarjeta*> listaTarjetas;
  int i = 0;   
  do
    {      
      string num = "";
      pos2 = lista.find ( ",", 0 );
      posParentesis = lista.find ( "(", 0 );
      posParentesisCerrado = lista.find ( ")", 0 );
      string maz = "";
      num = lista.substr ( pos1, posParentesis );
      num = stripwhite_st ( num );      //siguiente tarjeta
      if ( atoi ( num.c_str() ) > 9 )
      	maz = lista.substr ( posParentesis + 1, posParentesisCerrado - 3 );
      else
	maz = lista.substr ( posParentesis + 1, posParentesisCerrado - 2 );      
      lista = lista.substr ( pos2 + 1, lista.size() );
      lista = stripwhite_st ( lista );    
      tarjeta *tjt = _mazo [ maz ]->obtenerTarjeta ( atoi ( num.c_str() ) );
      _mazo [ maz ]->eliminarTarjeta ( atoi ( num.c_str() ) );
      listaTarjetas.push_back ( tjt );
      pos1 = 0;
      i++;
    }
  while ( ( pos2 != string::npos ) );
  j->set_tieneTarjeta ( listaTarjetas );
  return;
}

/**
 * Mtodo que carga una partida ya comenzada. Se cargan los datos generales de la partida, adems de los datos 
 * de los jugadores, las propiedades y las tarjetas. Se debe indicar el fichero del que se carga y se devolver
 * el prompt del sistema en el estado que se qued cuando se guard la partida
 */
int tablero::cargar ( string fichero, string &prompt )
{
  bool cargado = false;
  set_numJugadores ( 0 );
  fstream ifs ( fichero.c_str (), ios::in );
  if ( !ifs )
    {
      cerr << "No existe ninguna partida guardada con ese nombre. Compruebe que exista el fichero " << fichero << endl;
      return -1;
    }
  banca *b = new banca ();
  set_banca ( b );
  //leer datos del fichero  
  const int N = 100;
  string::size_type pos;
  string linea;
  int ncars=0, i;
  do
    {
      getline ( ifs, linea, '\n' );     //lee una linea      
      if ( ifs.eof() ) break;		//fin de fichero
      /* Tratamos la linea leida */
      
      /* Elimina espacios en blanco y comentarios de una linea */
      i=0;
      while ( ( linea[i] != '%') && (i < linea.size() ) )
        i++;
      /* Limpia los comentarios detectados y los espacios en blanco */
      linea = linea.substr ( 0, i );
      linea = stripwhite_st ( linea );
      
      /* Nombre de la partida */
      if ( linea.substr ( 0, 6 ) == "NOMBRE" )
	{
	  pos = linea.find ( "=", 0 );
	  string aux = stripwhite_st ( linea.substr ( pos+1, linea.size () ) );
	  set_nombrePartida ( aux );	
	}
      /* Posicin de la casilla de salida */
      else if ( linea.substr ( 0, 6 ) == "SALIDA" )
	{
	  pos = linea.find ( "=", 0 );
	  string aux = stripwhite_st ( linea.substr ( pos+1, linea.size () ) );
	  int salida = atoi ( aux.c_str() ); 
	  set_salida ( salida );	
	}
      /* Turno */
      else if ( linea.substr ( 0, 5 ) == "TURNO" )
	{
	  pos = linea.find ( "=", 0 );
	  string aux = stripwhite_st ( linea.substr ( pos+1, linea.size () ) );
	  int turno = atoi ( aux.c_str() );
	  set_turno ( turno );	
	}
      /* Nombre de la partida */
      else if ( linea.substr ( 0, 14 ) == "JUGADOR ACTUAL" )
	{
	  pos = linea.find ( "=", 0 );
	  string aux = stripwhite_st ( linea.substr ( pos+1, linea.size () ) );
	  int jugadorAc = atoi ( aux.c_str() );
	  set_jugadorActual ( jugadorAc );	
	}
      /* Carga la ruta indicada en el fichero de configuracion en la variable correspondiente*/
      else if ( linea.substr ( 0, 7 ) == "TABLERO" )
	{
	  pos = linea.find ( "=", 0 );
	  string aux = stripwhite_st ( linea.substr ( pos+1, linea.size () ) );
	  set_fichCasillas ( aux );	
	}
      else if ( linea.substr ( 0, 6 ) == "SUERTE" )
	{
	  pos = linea.find ( "=", 0 );
	  string aux = stripwhite_st ( linea.substr ( pos+1, linea.size () ) );
	  set_fichSuerte ( aux );
	}
      else if ( linea.substr ( 0, 14 ) == "CAJA COMUNIDAD" )
	{
	  pos = linea.find ( "=", 0 );
	  string aux = stripwhite_st ( linea.substr ( pos+1, linea.size () ) );
	  set_fichCajaComunidad ( aux );
	}
      else if ( linea.substr ( 0, 6 ) == "MONEDA" )
	{
	  pos = linea.find ( "=", 0 );
	  string aux = stripwhite_st ( linea.substr ( pos+1, linea.size () ) );
	  set_moneda( aux );
	}
      else if ( linea.substr ( 0, 6 ) == "SUBASTAS" )
	{
	  pos = linea.find ( "=", 0 );
	  string aux = stripwhite_st ( linea.substr ( pos+1, linea.size () ) );
	  set_subastas( atoi ( aux.c_str() ) );
	}
      else if ( linea.substr ( 0, 14 ) == "DINERO INICIAL" )
	{
	  pos = linea.find ( "=", 0 );
	  string aux = stripwhite_st ( linea.substr ( pos+1, linea.size () ) );	  
	  set_dineroInicial( atol ( aux.c_str() ) );
	}
      else if ( linea.substr ( 0, 17 ) == "CASAS DISPONIBLES" )
	{
	  pos = linea.find ( "=", 0 );
	  string aux = stripwhite_st ( linea.substr ( pos+1, linea.size () ) );	  
	  _banca->set_casasDisponibles ( atoi ( aux.c_str() ) );
	}
      else if ( linea.substr ( 0, 19 ) == "HOTELES DISPONIBLES" )
	{
	  pos = linea.find ( "=", 0 );
	  string aux = stripwhite_st ( linea.substr ( pos+1, linea.size () ) );
	  _banca->set_hotelesDisponibles ( atoi ( aux.c_str() ) );
	}

       else if (  linea.substr ( 0, 12 ) == "JUGADOR AUTO" )
	{	 
	  if ( cargado == false )
	    {
	      cerr << "Error en el formato del fichero, se deben indicar los ficheros de casillas y tarjetas antes que los jugadores";
	      return -1;
	    }
	  if ( linea.substr ( linea.size() - 1, linea.size() ) != ";" )
	    {	      
	      bool finJugador = false;	      
	      jugador_reglas *j = new jugador_reglas ( );
	      j->set_reglas ( 1 );
	      string nombre = "", color = "";
	      long dinero_total = 0, capital = 0, estado = 0;
	      int turnosEncarcelado = 0, tiradasDobles = 0;
	      do
		{
		  getline ( ifs, linea, '\n' );     //lee una linea
		  /* Elimina espacios en blanco y comentarios de una linea */
		  i=0;
		  while ( ( linea[i] != '%') && (i < linea.size() ) )
		    i++;
		  /* Limpia los comentarios detectados y los espacios en blanco */
		  linea = linea.substr ( 0, i );
		  linea = stripwhite_st ( linea );
		  if ( ifs.eof() ) break;
		  if ( linea.substr ( linea.size() - 1, linea.size() ) == ";" )
		    {
		      linea = linea.substr ( 0,  linea.size() - 1 );
		      finJugador = true;
		    }
		  if ( linea.substr ( 0, 6 ) == "NOMBRE" )
		    {
		      pos = linea.find ( "=", 0 );
		      string aux = linea.substr ( pos+1, linea.size () );		      
		   
		      if ( buscar_jugador_nombre_int ( aux ) == -1 )
			j->set_nombre ( aux );
		      else
			{
			  cerr << aux << ": Nombre de jugador repetido" << endl;
			  return -1;
			}
		    }
		  else if ( linea.substr ( 0, 5 ) == "COLOR" )
		    {
		      pos = linea.find ( "=", 0 );
		      string aux = stripwhite_st ( linea.substr ( pos+1, linea.size () ) );
		      if ( buscar_jugador_color ( aux ) == NULL )
			j->set_color ( aux );
		      else
			{
			  cerr << aux << ": Color de ficha repetido" << endl;
			  return -1;
			}
		    }
		  else if ( linea.substr ( 0, 6 ) == "REGLAS" )
		    {
		      pos = linea.find ( "=", 0 );
		      string aux = stripwhite_st ( linea.substr ( pos+1, linea.size () ) );
		      int regla = atoi ( aux.c_str() );
		      if ( regla > 0 && regla < 3 )
			j->set_reglas ( regla );
		      else
			{
			  cerr << aux << ": Jugador automtico no vlido (reglas)" << endl;
			  return -1;
			}
		    }
		  else if ( linea.substr ( 0, 6 ) == "DINERO" )
		    {
		      pos = linea.find ( "=", 0 );
		      string aux = stripwhite_st ( linea.substr ( pos+1, linea.size () ) );
		      j->set_dinero_total ( atol ( aux.c_str() ) );
		    }
		  else if ( linea.substr ( 0, 7 ) == "CAPITAL" )
		    {
		      pos = linea.find ( "=", 0 );
		      string aux = stripwhite_st ( linea.substr ( pos+1, linea.size () ) );
		      j->set_capital ( atol ( aux.c_str() ) );
		    }
		  else if ( linea.substr ( 0, 18 ) == "TURNOS ENCARCELADO" )
		    {
		      pos = linea.find ( "=", 0 );
		      string aux = stripwhite_st ( linea.substr ( pos+1, linea.size () ) );
		      j->set_turnosEncarcelado ( atoi ( aux.c_str() ) );
		    }
		  else if ( linea.substr ( 0, 14 ) == "TIRADAS DOBLES" )
		    {
		      pos = linea.find ( "=", 0 );
		      string aux = stripwhite_st ( linea.substr ( pos+1, linea.size () ) );
		      j->set_tiradasDobles ( atoi ( aux.c_str() ) );
		    }
		  else if ( linea.substr ( 0, 6 ) == "ESTADO" )
		    {
		      pos = linea.find ( "=", 0 );
		      string aux = stripwhite_st ( linea.substr ( pos+1, linea.size () ) );
		      j->set_estado ( atol ( aux.c_str() ) );
		    }
		  else if ( linea.substr ( 0, 8 ) == "POSICION" )
		    {
		      pos = linea.find ( "=", 0 );
		      string aux = stripwhite_st ( linea.substr ( pos+1, linea.size () ) );
		      colocarJugador ( j, atoi ( aux.c_str() ) );
		    }
		   else if ( linea.substr ( 0, 11 ) == "PROPIEDADES" )
		    {
		      pos = linea.find ( "=", 0 );
		      string aux = stripwhite_st ( linea.substr ( pos+1, linea.size () ) );		      
		      cargarPropiedades ( j, aux );
		    }
		  else if ( linea.substr ( 0, 8 ) == "TARJETAS" )
		    {
		      pos = linea.find ( "=", 0 );
		      string aux = stripwhite_st ( linea.substr ( pos+1, linea.size () ) );
		      cargarTarjetas ( j, aux );
		    }
		}
	      while ( ( linea.substr ( linea.size() - 1, linea.size() ) != ";" ) && ( finJugador == false ) );	      
	      anyadir_jugador ( j );	      
	      set_numJugadores ( juega.size() );	    
	    }
	  else
	    {
	      cerr << "ERROR: No hay datos para un jugador" << endl;
	      return -1;
	    }
	}

      else if (  linea.substr ( 0, 7 ) == "JUGADOR" )
	{	 
	  if ( cargado == false )
	    {
	      cerr << "Error en el formato del fichero, se deben indicar los ficheros de casillas y tarjetas antes que los jugadores";
	      return -1;
	    }
	  if ( linea.substr ( linea.size() - 1, linea.size() ) != ";" )
	    {	      
	      bool finJugador = false;	      
	      jugador *j = new jugador ( );	  
	      string nombre = "", color = "";
	      long dinero_total = 0, capital = 0, estado = 0;
	      int turnosEncarcelado = 0, tiradasDobles = 0;
	      do
		{
		  getline ( ifs, linea, '\n' );     //lee una linea
		  /* Elimina espacios en blanco y comentarios de una linea */
		  i=0;
		  while ( ( linea[i] != '%') && (i < linea.size() ) )
		    i++;
		  /* Limpia los comentarios detectados y los espacios en blanco */
		  linea = linea.substr ( 0, i );
		  linea = stripwhite_st ( linea );
		  if ( ifs.eof() ) break;
		  if ( linea.substr ( linea.size() - 1, linea.size() ) == ";" )
		    {
		      linea = linea.substr ( 0,  linea.size() - 1 );
		      finJugador = true;
		    }
		  if ( linea.substr ( 0, 6 ) == "NOMBRE" )
		    {
		      pos = linea.find ( "=", 0 );
		      string aux = linea.substr ( pos+1, linea.size () );		      
		   
		      if ( buscar_jugador_nombre_int ( aux ) == -1 )
			j->set_nombre ( aux );
		      else
			{
			  cerr << aux << ": Nombre de jugador repetido" << endl;
			  return -1;
			}
		    }
		  else if ( linea.substr ( 0, 5 ) == "COLOR" )
		    {
		      pos = linea.find ( "=", 0 );
		      string aux = stripwhite_st ( linea.substr ( pos+1, linea.size () ) );
		      if ( buscar_jugador_color ( aux ) == NULL )
			j->set_color ( aux );
		      else
			{
			  cerr << aux << ": Color de ficha repetido" << endl;
			  return -1;
			}
		    }
		  else if ( linea.substr ( 0, 6 ) == "DINERO" )
		    {
		      pos = linea.find ( "=", 0 );
		      string aux = stripwhite_st ( linea.substr ( pos+1, linea.size () ) );
		      j->set_dinero_total ( atol ( aux.c_str() ) );
		    }
		  else if ( linea.substr ( 0, 7 ) == "CAPITAL" )
		    {
		      pos = linea.find ( "=", 0 );
		      string aux = stripwhite_st ( linea.substr ( pos+1, linea.size () ) );
		      j->set_capital ( atol ( aux.c_str() ) );
		    }
		  else if ( linea.substr ( 0, 18 ) == "TURNOS ENCARCELADO" )
		    {
		      pos = linea.find ( "=", 0 );
		      string aux = stripwhite_st ( linea.substr ( pos+1, linea.size () ) );
		      j->set_turnosEncarcelado ( atoi ( aux.c_str() ) );
		    }
		  else if ( linea.substr ( 0, 14 ) == "TIRADAS DOBLES" )
		    {
		      pos = linea.find ( "=", 0 );
		      string aux = stripwhite_st ( linea.substr ( pos+1, linea.size () ) );
		      j->set_tiradasDobles ( atoi ( aux.c_str() ) );
		    }
		  else if ( linea.substr ( 0, 6 ) == "ESTADO" )
		    {
		      pos = linea.find ( "=", 0 );
		      string aux = stripwhite_st ( linea.substr ( pos+1, linea.size () ) );
		      j->set_estado ( atol ( aux.c_str() ) );
		    }
		  else if ( linea.substr ( 0, 8 ) == "POSICION" )
		    {
		      pos = linea.find ( "=", 0 );
		      string aux = stripwhite_st ( linea.substr ( pos+1, linea.size () ) );
		      colocarJugador ( j, atoi ( aux.c_str() ) );
		    }
		   else if ( linea.substr ( 0, 11 ) == "PROPIEDADES" )
		    {
		      pos = linea.find ( "=", 0 );
		      string aux = stripwhite_st ( linea.substr ( pos+1, linea.size () ) );		      
		      cargarPropiedades ( j, aux );
		    }
		  else if ( linea.substr ( 0, 8 ) == "TARJETAS" )
		    {
		      pos = linea.find ( "=", 0 );
		      string aux = stripwhite_st ( linea.substr ( pos+1, linea.size () ) );
		      cargarTarjetas ( j, aux );
		    }
		}
	      while ( ( linea.substr ( linea.size() - 1, linea.size() ) != ";" ) && ( finJugador == false ) );
	      anyadir_jugador ( j );	     
	      set_numJugadores ( juega.size() );	    
	    }
	  else
	    {
	      cerr << "ERROR: No hay datos para un jugador" << endl;
	      return -1;
	    }
	  
	}   

      else if ( ( linea.substr ( 0, 9 ) == "PROPIEDAD" ) )
	{
	  if ( cargado == false )
	    {
	      cerr << "Error en el formato del fichero, se deben indicar los ficheros de casillas y tarjetas antes que los jugadores";
	      return -1;
	    }
	  if ( linea.substr ( linea.size() - 1, linea.size() ) != ";" )
	    {
	      bool finCasilla = false;
	      int numero = 0, numCasas = 0, numHoteles = 0;
	      bool hipotecado = 0, hipotecable = 0, permiso = 0;	      
	      do
		{
		  getline ( ifs, linea, '\n' );     //lee una linea
		  /* Elimina espacios en blanco y comentarios de una linea */
		  i=0;
		  while ( ( linea[i] != '%') && (i < linea.size() ) )
		    i++;
		  /* Limpia los comentarios detectados y los espacios en blanco */
		  linea = linea.substr ( 0, i );
		  linea = stripwhite_st ( linea );
		  if ( ifs.eof() ) break;
		  if ( linea.substr ( linea.size() - 1, linea.size() ) == ";" )
		    {
		      linea = linea.substr ( 0,  linea.size() - 1 );
		      finCasilla = true;
		    }
		  if ( linea.substr ( 0, 6 ) == "NUMERO" )
		    {
		      pos = linea.find ( "=", 0 );
		      string aux = stripwhite_st ( linea.substr ( pos+1, linea.size () ) );
		      numero = atoi ( aux.c_str() );
		    }
		  else if ( linea.substr ( 0, 10 ) == "HIPOTECADO" )
		    {
		      pos = linea.find ( "=", 0 );
		      string aux = stripwhite_st ( linea.substr ( pos+1, linea.size () ) );
		      hipotecado = ( atoi ( aux.c_str() ) );
		    }
		  else if ( linea.substr ( 0, 11 ) == "HIPOTECABLE" )
		    {
		      pos = linea.find ( "=", 0 );
		      string aux = stripwhite_st ( linea.substr ( pos+1, linea.size () ) );
		      hipotecable = ( atoi ( aux.c_str() ) );
		    }
		  else if ( linea.substr ( 0, 5 ) == "CASAS" )
		    {
		      pos = linea.find ( "=", 0 );
		      string aux = stripwhite_st ( linea.substr ( pos+1, linea.size () ) );
		      numCasas = ( atoi ( aux.c_str() ) );
		    }
		  else if ( linea.substr ( 0, 7 ) == "HOTELES" )
		    {
		      pos = linea.find ( "=", 0 );
		      string aux = stripwhite_st ( linea.substr ( pos+1, linea.size () ) );
		      numHoteles = ( atol ( aux.c_str() ) );
		    }		  
		  else if ( linea.substr ( 0, 7 ) == "PERMISO CONSTRUCCION" )
		    {
		      pos = linea.find ( "=", 0 );
		      string aux = stripwhite_st ( linea.substr ( pos+1, linea.size () ) );
		      permiso = ( atol ( aux.c_str() ) );
		    }		  
		}
	      while ( ( linea.substr ( linea.size() - 1, linea.size() ) != ";" ) && ( finCasilla == false ) );	      
	      casilla *c = contiene [ numero ];
	      if ( propiedad *p = dynamic_cast <propiedad*> ( contiene[ numero ] ) )
		{
		  p->set_hipotecado ( hipotecado );
		  p->set_hipotecable ( hipotecable );
		  if ( calle *cal = dynamic_cast <calle*> ( contiene[ numero ] ) )
		    {
		      cal->set_numCasas ( numCasas );
		      cal->set_numHoteles ( numHoteles);
		    }
		}
	     
	    }
	}



      if ( get_fichCasillas() != "" && get_fichSuerte() != "" && get_fichCajaComunidad() !="" && cargado == false )
	{
	  crearMazo ( tarjetasSuerte );
	  crearMazo ( tarjetasCaja );
	  cargar_casillas ();
	  cargado = true;
	}
      if ( ( ncars = ifs.gcount() ) == N-1 ) //se activo el failbit
	ifs.clear();
    } while ( true );
  set_partidaAbierta ( true );
  prompt = "(" + juega[ get_turno() ]->get_nombre() + ")";
  if ( get_jugadorActual () != get_turno () )
    {
      prompt += "[~";
      prompt += juega[ get_jugadorActual() ]->get_nombre ();
      prompt += "]";
    }
  return 0;
}
 
/**
 * Mtodo que comprueba uno por uno cada uno de los jugadores para comprobar que no
 * tenga ninguna deuda con otro jugador o con la banca, mediante el atributo estado,
 * que indica el estado financiero actual (debe o haber pendiente de pago/cobro).
 */
bool tablero::existenDeudas ()
{
  int i = 0;
  while ( i < juega.size() && juega[i]->get_estado() >= 0 )
      i++;
  if ( i < juega.size() )
    return true;
  else
    return false;
}

/**
 * Mtodo derivado del anterior que comprueba slo si existen deudas para un jugador
 * concreto, que se pasar como parmetro indicando su numero de turno en el juego.
 */
bool tablero::existenDeudas ( int idJugador)
{
  if ( juega[ idJugador ]->get_estado() < 0 )
    return true;
  else
    return false; 
}

/**
 * Mtodo que muestra una lista de morosos en la partida. Para ser moroso un jugador debe
 * tener su atributo _estado negativo.
 */
string tablero::verMorosos ()
{
  string aux = "";
  for ( int i = 0; i < get_numJugadores(); i++ )
    if ( juega[i]->get_estado() < 0 )
      {
	aux += juega[i]->get_nombre();
	char numAlpha[30];
	sprintf ( numAlpha, "%d", juega[i]->get_estado() );
	aux += " ( saldo = " + string ( numAlpha ) + " )\n";
      }
  return ( aux );
}

/**
 * Mtodo que salda las deudas de juego que hay en la partida. El proceso consta de dos
 * partes: pagar deudas y cobrar deudas.
 *
 *	1. Pagar: Todos los jugadores con estado deudor (<0) pagarn la cantidad correspondiente.
 *		  Si no dipone de dinero suficiente no pagar, dejndole tiempo para que 
 *		  obtenga el dinero lquido necesario.
 *	2. Cobrar: Los jugadores con estado acreedor (>0) cobrarn la cantidad indicada.
 *
 * Se puede dar el caso de que los deudores no puedan pagar sus deudas. En este caso, los
 * cobradores evidentemente no podrn cobrar.
 */
bool tablero::pagarDeudas ()
{
  bool ok;
  long pagado = 0;
  long pendienteDeCobro = 0;
  int numCobradores = 0;
  // 1. Los deudores pagan.
  for ( int i = 0; i < juega.size(); i++ )
    {
      if ( juega[i]->get_estado() < 0 ) 
	{
	  if ( juega[i]->get_dinero_total() + juega[i]->get_estado() >= 0 )
	    {
	      juega[i]->pagar ( 0 - juega[i]->get_estado() );
	      pagado += juega[i]->get_estado();
	      juega[i]->set_estado ( 0 );
	      cout << "Bien hecho, has cabado con tus deudas!" << endl;
	    }
	  else
	    cout << juega[i]->get_nombre() << " te veo en apuros, o consigues dinero antes de que se tire el dado de nuevo, o tendrs que abandonar la partida" << endl;
	}
      else
	{
	  pendienteDeCobro += juega[i]->get_estado();
	  numCobradores ++;
	}
    }
  
  // 2. Los cobradores cobran sus deudas.
  //  2.1. Si todos los dudores han pagado, debe haber dinero suficiente -> cobran todo.
  //	   ( pendiente de cobro <= pagado )
  //  2.2. Si no hay dinero, se repartir el dinero pagado entre los jugadores y el resto
  //	   se quedar pendiente.
  if ( (0 - pagado) >= pendienteDeCobro )
    {
      ok = true;
      for ( int i = 0; i < juega.size(); i++ )
	{
	  if ( juega[i]->get_estado() > 0 ) 
	    {
	      juega[i]->pagar ( 0 - juega[i]->get_estado() );
	      juega[i]->set_estado ( 0 );
	    }
	}
    }
  else
    {
      ok = false;
    }
  return ok;
}

/**
 * Mtodo que distribuye los bienes de un jugador eliminado entre el resto de los jugadores.
 * Para ello se subastarn los bienes del jugador eliminado. Si el acreedor es un unico
 * jugador y no es la banca, se la atribuyen los bienes directamente.
 */
void tablero::redistribuirBienes ( jugador * j )
{
  vector <jugador *> acreedores;

  cout << "Redistribuyendo los bienes de " << j->get_nombre () << " ... " << endl;

  // Busca los acreedores
  for ( int i = 0; i < get_numJugadores(); i++ )
    {
      if ( juega[i]->get_estado() > 0 )	
	{
	  acreedores.push_back ( juega[ i ] );	
	  juega[i]->set_estado ( 0 );
	}
    }

  //REGLAS: El banco se queda con las casas y hoteles pagando por ellos la mitad de su coste. Este dinero
  // se le entregar al acreedor.
  long cantidadEdificios = j->expropiarEdificios ();
  if ( cantidadEdificios > 0 )
    cout << "Se han expropiado los edificios por un total de : " << cantidadEdificios << endl;

  //Devolver a la banca los edificios exporpiados
  int numCasas = j->get_casasConstruidas ();
  int numHoteles = j->get_hotelesConstruidos ();
  _banca->set_casasDisponibles ( _banca->get_casasDisponibles () + numCasas );
  _banca->set_hotelesDisponibles ( _banca->get_hotelesDisponibles () + numHoteles );

  //Si hay un acreedor se le asignarn las propiedades del jugador en quiebra.
  if ( acreedores.size() == 1 )
    {
      acreedores[0]->anyadirListaPropiedades ( j->get_posee () );
      acreedores[0]->set_dinero_total ( acreedores[0]->get_dinero_total () + cantidadEdificios + j->get_dinero_total () );
      acreedores[0]->set_tieneTarjeta ( j->get_tieneTarjeta () );
      cout << acreedores[0]->get_nombre () << ", se han aadido a tus posesiones las propiedades y el dinero de " << j->get_nombre () << endl;    
    }
  //Si solo queda un jugador, la partida ha terminado
  else if ( get_numJugadores () <= 2 )
    {
      cout << "No hay subasta, ya tenemos un ganador" << endl;
    }
  //Subastar
  else
    {
      //Si el acreedor es la banca, se subasta entre todos
      if ( acreedores.size() == 0 )
	{
	  cout << "Se subastarn las propiedades de " << j->get_nombre() << " entre todos los jugadores" << endl;
	  subastarLista ( juega, j->get_posee (), j );
	}
      //Subastar solo entre los acreedores
      else 
	{
	  cout << "Se subastarn las propiedades de " << j->get_nombre() << " entre sus acreedores" << endl;
	  subastarLista ( acreedores, j->get_posee (), j );
	}
    }
}

/**
 * Mtodo que subasta una serie de propiedades que viene dada con una lista entre los acreedores indicados.
 * Se iran asignando las propiedades al jugador ganador en cada puja.
 */
void tablero::subastarLista ( vector <jugador*> acreedores, list <propiedad*> listaPropiedades, jugador *propietario )
{
  cout << "Se inicia la subasta para adquirir las propiedades del jugador eliminado" << endl;
  cout << "Los siguientes jugadores podrn pujar por las propiedades que les interesen: " << endl;
  int i = 0;
  //Muestra los jugadores que pujan
  while ( i != acreedores.size() )
    {      
      if ( acreedores[ i ] == propietario )
	acreedores.erase ( acreedores.begin() + i );
      else
	{
	  cout << acreedores[ i ]->get_nombre () << endl;
	  i++;
	}
    }
  cout << "Si quieres pasar en una ronda de pujas, pulsa 0 o escribe 'paso', si deseas salir de la puja tienes que escribir 'salir'" << endl;
  //Subastar una a una las propiedades recorriendo la lista y llamando al metodo subastarPropiedad ( .. )
  cout << endl << "SE INICIA LA SUBASTA" << endl;
  list <propiedad*>::iterator listaP;
  for ( listaP = listaPropiedades.begin(); listaP != listaPropiedades.end(); listaP++ )
    { 
      vector <jugador*> acreedoresIntervienen = acreedores;
      subastarPropiedad ( (*listaP), acreedoresIntervienen );
    } //fin_for
}

/**
 * Mtodo que subasta una propiedad entre todos los jugadores interesados. Al final del proceso
 * la propiedad ser asignada a la banca si nadie la quiere o al jugador que ms alto puje.
 * Las pujas seguirn un orden secuencial, y un jugador podr pujar, pasar o salir de la subasta
 * del objeto
 */
void tablero::subastarPropiedad ( propiedad *p, vector<jugador*> interesados )
{
  //Muestra la informacin del objeto
  cout << "OBJETO PARA PUJAR: " << p->get_nombre() << endl;
  cout << p;
  long max = 0;
  int i = 0;	
  cin.clear();
  bool adjudicado = false;
  jugador *ganador = NULL;
  int paso = 0;

  // Permite pujar mientras no se cumpla un ciclo completo sin pujar o no quede ningun interesado.
  while ( !adjudicado && paso < interesados.size() * 2 && !interesados.empty() )
    {
      long puja;	 
      // Si el jugador pasa, se incrementa paso para comprobar el ciclo
      puja = interesados[i]->pujar( p, max, ganador );
      if ( puja == 0 )
	paso ++;

      //Si puja, se reinicia el ciclo de paso, y se reajusta el ganador y el mximo
      else if ( puja > max )
	{
	  paso = 0;
	  max = puja;
	  ganador = interesados[ i ];
	}
      //Si devuelve -1 : Salir de la subasta
      else if ( puja < 0 )	
	interesados.erase ( interesados.begin() + i );	  
      
      // Colocar el puntero al siguiente jugador que puja
      if ( interesados.size () > 0 )
	{	  
	  if ( puja >= 0 )		
	    i = ( i + 1 ) % interesados.size();	    
	  if ( i >= interesados.size() )	      
	      i = 0;	    
	  if ( ( interesados[ i ] == ganador ) || ( ( ganador == NULL ) && ( interesados.size() == 0 ) ) )	    
	    adjudicado = true;	    	    
	}
      else
	adjudicado = true;    
    }
  
  //Si nadie a pujado, se devuelve la propiedad a la banca.
  if ( ( ganador == NULL ) && ( interesados.size() == 0 ) )
    {
      cout << "Nadie ha querido la propiedad, por lo que sta queda libre." << endl;
      p->liberar( get_banca () );
    }

  // Si gana alguien se le aade al jugador correspondiente
  else if ( ganador != NULL )
    {
      cout << p->get_nombre() << " ADJUDICADA A " << ganador->get_nombre() << endl;
      list<propiedad*> aux;
      ganador->set_dinero_total ( ganador->get_dinero_total() - max );
      ganador->set_capital ( ganador->get_capital() - max + p->get_precio() );
      aux.push_back ( p );
      ganador->anyadirListaPropiedades ( aux );
    }
  else
    {
      cout << "Nadie ha querido la propiedad, por lo que sta queda libre." << endl;
      p->liberar( get_banca () );
    }
}

/**
 * Mtodo que elimina un jugador de la partida. Para ello redistribuye sus bienes
 * y lo borra del tablero. A continuacin se retorna el nmero de jugadores que
 * quedan en la partida.
 */
int tablero::declararBancarrota ( int numeroJugador)
{  
  redistribuirBienes ( juega [ numeroJugador] ); 
  
  //Guardar el log del jugador
  if ( juega.size() >= 2 )
    juega [numeroJugador]->cerrarLogs();
  juega [numeroJugador]->guardarLogs();

  //Elimina el jugador tanto de la lista de jugadores como de la casilla en la que est.  
  eliminarJugador ( juega[ numeroJugador ] );
  set_numJugadores ( juega.size() );

  //Establece el turno a quien le corresponde y le da el poder sobre la consola.
  set_turno ( get_turno () % get_numJugadores () );
  set_jugadorActual ( get_turno() );
  
  cout << "Jugador eliminado" << endl;
  return juega.size();
}

/**
 * Mtodo que elimina un jugador de la partida. Para ello redistribuye sus bienes
 * y lo borra del tablero. A continuacin se retorna el nmero de jugadores que
 * quedan en la partida.
 */
int tablero::declararBancarrota ( jugador *j)
{  
  redistribuirBienes ( j ); 
  
  //Guardar el log del jugador
  if ( juega.size() >= 2 )
    j->cerrarLogs();
  j->guardarLogs();
  
  //Elimina el jugador tanto de la lista de jugadores como de la casilla en la que est.
  eliminarJugador ( j );
  set_numJugadores ( juega.size() );

  //Establece el turno a quien le corresponde y le da el poder sobre la consola.
  set_turno ( get_turno () % get_numJugadores () );
  set_jugadorActual ( get_turno() );

  cout << endl << endl << j->get_nombre() << " eliminado" << endl;
  return juega.size();
}

/**
 * Mtodo que calcula el reparto de las casillas en el tablero en funcin del nmero total
 * de casillas insertadas en el tablero.
 * Las proporciones del tablero en este caso sern
 *	-arriba y abajo: 30%
 *	-izqda y dcha: 20%
 * Para ver un ejemplo de tablero estandar mirar el fichero ejemplo_tablero.txt
 * -IN: numCasillas (nmero total de casillas en el tablero).
 * -OUT: casillasArriba, casillasAbajo, casillasDcha, casillasIzqda (nmero de casillas en 
 *	cada lado del tablero ).
 */
void calcularRepartoCasillas (int numCasillas, int &casillasArriba, int &casillasAbajo, int &casillasDcha, int &casillasIzqda)
{
  //Si hay menos de 9 casillas se definen en una sla fila
  if ( numCasillas <= 8 ) 
    {
      casillasArriba = numCasillas;
      casillasAbajo = 0;
      casillasDcha = 0;
      casillasIzqda = 0;
      return;
    }
  else
    {
      //Calculo del numero de casillas inicialmente, en caso de que el nmero de casillas
      //sea el estandar
      casillasArriba = numCasillas*3/10;
      casillasAbajo = casillasArriba;
      casillasDcha = numCasillas*2/10;
      casillasIzqda = casillasDcha;
    }  

  //Si hay menos de 4 casillas arriba o abajo, el tablero se descuadra,
  //  ->obligamos a que al menos haya 4 arriba y abajo.  
  if ( casillasArriba < 4 )
    {
      //Caso especial de 9 casillas lo coloco a mano
      if ( numCasillas == 9 )
	{
	  casillasArriba = 3;
	  casillasAbajo = 4;
	  casillasDcha = 1;
	  casillasIzqda = 1;
	}
      else
	{
	  //Las casillas que vayamos a reasignar, se cambian de las franjas horizontales
	  //(arriba y abajo) a las verticales (izq y dcha).
	  casillasIzqda -= 4 - casillasArriba; 
	  casillasDcha = casillasIzqda;
	  casillasArriba = 4;				//Arriba y dcha se fuerzan a 4.
	  casillasAbajo = 4;
	}
    }
  //Si no es un tablero estandar (sobran casillas) -> Redistribuir el tablero
  int casillasAcumuladas = ( casillasDcha + casillasIzqda + casillasArriba + casillasAbajo );
  if ( casillasAcumuladas != numCasillas )
    {          
      int resto = numCasillas - casillasAcumuladas;	//casillas que faltan por redistribuir
      casillasArriba += resto/2;			//Distribuyo la mitad arriba ...
      casillasAbajo = casillasArriba;			//... y la mitad abajo ->siguen iguales
      //Si aun quedan casillas por distribuir (si eran impares quedar 1)
      if ( numCasillas != casillasAcumuladas + (resto/2 * 2) )
	casillasAbajo++;				//Aado una abajo (abajo = arriba+1)      
    }
}

/**
 * OPERADOR <<
 * Se imprimirn por pantalla todos aquellos tableros que estn formados por menos de 42
 * casillas. El resto mostrarn un mensaje para indicar que no es posible mostrarse (por
 * motivos de espacio.
 * Secuencia:
 *	1. Comprobar dimensiones del tablero: 
 *		- <=42: Dibujo el tablero
 *		- >42: No se dibuja porque no entra en la pantalla
 *	2. Dimensiones del tablero (por filas):
 *		1 fila: -> De lado a lado
 *			1 casilla (10x6 caracteres)
 *			resto (9x6 caracteres)
 *		de la fila 2 a la n-1
 *			casillas de (17x3 caracteres): una acoplada a cada esquina
 *			Fila n-1 dibuja el largo de la ltima fila.
 *		Fila n
 *			Igual que la 1 fila 
 */
ostream& operator<< ( ostream &os, tablero &tab )
{
  int casillasArriba, casillasAbajo;		//n casillas de las filas horizontales
  int casillasDcha, casillasIzqda;		//n casillas de las columnas (izq y dcha)
  string dibujoTablero;				//string que guarda el tablero
  if ( tab.get_numCasillas()<=42 )
    {
      //Calculo las dimensiones del tablero
      calcularRepartoCasillas ( tab.get_numCasillas(), casillasArriba, casillasAbajo, 
				casillasDcha, casillasIzqda );
      dibujoTablero = tab.dibujarTablero ( casillasArriba, casillasAbajo, casillasDcha, 
					   casillasIzqda );    
    }
  else
    {
      cerr << "Las dimensiones del tablero no permiten que se dibuje en pantalla." << endl;
      cerr << "Para que esto sea posible, el nmero de casillas debe ser menor o igual a 42" << endl;
    }
  os << dibujoTablero << endl;
  return os;
}

/**
 * DIBUJO TABLERO
 * Mtodo que crea un tablero a partir de unas dimensiones dadas.
 * El resultado ser el tablero completo en una cadena de caracteres.
 * Cada casilla tendr el formato que se indica en el fichero ejemplo_casilla.txt
 */
string tablero::dibujarTablero ( int casillasArriba, int casillasAbajo, int casillasDcha, int casillasIzqda )
{  
  // Casillas de los lados superior e inferior
  const int anchoCasillaV = 16;
  const int altoCasillaV = 2; //0 .. 2
  // Casillas de los lados dcho y izqdo
  const int anchoCasilla = 9;
  const int altoCasilla = 5; //0 .. 5

  string aux;  

  //Casillas de la fila superior
  for ( int i = 0; i <= altoCasilla; i++)
    {      
      if ( i != 0) aux += "|"; //inicio de fila
      //Dibujo cada fila
      for ( int j = 1; j < casillasArriba*anchoCasilla; j = j + anchoCasilla )
	{	
	  int numero = j / anchoCasilla; //Casilla actual
	  //Si el tablero esta descuadrado la primera casilla ser doble...
	  if ( (casillasArriba != casillasAbajo ) && ( j == 1 ) )
	    aux += contiene[numero] -> dibujarCasillaHorizontal ( i, anchoCasilla*2 );
	  else //... y si no se decuadra, sern todas iguales
	    aux += contiene[numero] -> dibujarCasillaHorizontal ( i, anchoCasilla );
	}
      aux += "\n";
    }

  //Casillas de las columnas derecha e izquierda   
  int incNumero = 0;
  //Dibujo cada casilla de la columna
  for ( int i = 1; i < casillasDcha*altoCasillaV; i = i + altoCasillaV )
    {
      incNumero++;
      int max;
      if ( i == casillasDcha*altoCasillaV - 1 )
	max = altoCasillaV - 1;
      else 
	max = altoCasillaV;
      for ( int j = 0; j <= max; j++ )
	{
	  //Casilla de la columna izquierda
	  int numero = get_numCasillas() - incNumero;			
	  aux += tablero::contiene[numero] -> dibujarCasillaVertical ( j, anchoCasillaV );
	  for ( int k = anchoCasillaV; k < anchoCasilla*casillasAbajo - anchoCasillaV -1; k++ )
	    aux += " ";
	  //Casilla de la columna derecha
	  numero = casillasArriba + incNumero - 1;
	  aux += tablero::contiene[numero] -> dibujarCasillaVertical ( j, anchoCasillaV );
	  aux += "\n";
	}
    }
  //Casillas de la fila inferior
  for ( int i = 0; i <= altoCasilla; i++)
    { 
      incNumero = casillasDcha + 1;
      aux += "|";  //inicio de fila
      if ( i == 0 )
	{
	  for ( int k = 1; k != anchoCasillaV; k++ )
	    aux += '_';
	  aux += "|";
	  for ( int k = anchoCasillaV; k < anchoCasilla*casillasAbajo - anchoCasillaV - 1; k++ )
	    aux += '_';
	  aux += "|";
	  for ( int k = 1; k < anchoCasillaV; k++ )
	    aux += '_';
	  aux += "|\n";
	}
      else
	{
	  //Dibujo cada fila
	  for ( int j = 1; j < casillasAbajo*anchoCasilla; j = j + anchoCasilla )
	    {	
	      int numero = get_numCasillas() - incNumero; //Casilla que estoy dibujando
	      aux += tablero::contiene[numero] -> dibujarCasillaHorizontal ( i, anchoCasilla );
	      incNumero++;
	    }
	  aux += "\n";
	}
    }
  
  return aux;
}

/**
 * Aade un jugador al vector de jugadores.
 * Recibe como parmetro un jugador y lo inserta en la ltima posicion del vector
 */
void tablero::anyadir_jugador ( jugador* nuevo )
{
  juega.push_back ( nuevo );
}

/**
 * Mtodo que elimina un jugador del vector de jugadores que participan en una partida.
 * El jugador eliminado ser el indicado en la variable player
 */
void tablero::eliminarJugador ( jugador* player )
{
  int i = 0;
    
  if ( juega.size() > 0 )
    {
      while ( ( juega[i] != player ) && ( i <= juega.size() ) )	
	i++;      
      juega[i]->desvincularDeCasilla();
      juega.erase ( juega.begin()+i );
    }
  
  // Reducimos el nmero de jugadores.
  int numJugadores = get_numJugadores();
  set_numJugadores ( --numJugadores );
}

/**
 * Aada un puntero a una casilla al vector de casillas (contiene) del tablero.
 * La insercin se realiza en funcin del numero de la casilla, de forma que el
 * tablero queda ordenado.
 * Finalmente, se incrementa el nmero de casillas. 
 */
int tablero::anyadir_casilla ( casilla *nuevo )
{
  int i=0;
  
  while ( ( i < tablero::contiene.size() ) && ( tablero::contiene[i]->get_numero() < nuevo->get_numero() ) )
    {      
      i++;
    }
  tablero::contiene.insert ( tablero::contiene.begin()+i, 1, nuevo );
  int numCasillas = get_numCasillas ();
  set_numCasillas ( ++numCasillas );
  return ( i );		//Devuelve la posicin en la que se ha insertado
}

/**
 * Comrpueba si el jugador esta introduciendo un nombre repetido
 */
bool tablero::repetido ( string nombre )
{
  bool repe = false;
  if ( juega.size() > 0 )
    for ( int i = 0; i < juega.size(); i++ )
      if ( juega[i]->get_nombre() == nombre )
	repe =true;
  return repe;
}

/**
 * Mtodo que crear el jugador con los datos conocidos y llamar al mtodo que lo inserta
 * en el vector de jugadores.
 */
int tablero::incluir_jugador ( int id, string nombre, string color, int jugadorAuto )
{
  if ( jugadorAuto == 0 )
    {
      jugador* player = new jugador ( nombre, color, get_dineroInicial(), get_dineroInicial(), 0, 0, 0, 1 );
      anyadir_jugador ( player );
    }
  else
    {
      jugador_reglas* player = new jugador_reglas ( nombre, color, get_dineroInicial(), get_dineroInicial(), 0, 0, 0, 1, jugadorAuto );
      anyadir_jugador ( player );
    }
  return 0;
}

/**
 * Aade una instancia de la clase banca a la partida, con los hoteles y casas iniciales determinadas
 * en los argumentos numCasas y numHoteles
 */
int tablero::incluirBanca ( int numCasas, int numHoteles )
{
  int ok = 0;
  banca *b = new banca ();
  if ( numCasas > 0 )
    b->set_casasDisponibles ( numCasas );
  else
    ok = -1;
  if ( numHoteles > 0 )
    b->set_hotelesDisponibles ( numHoteles );
  else
    ok = -1;
  set_banca ( b );
  return ok;    
}


/**
 *  Mtodo que recorre la lista de jugadores del tablero buscando aquel
 *  que responde al patron de color pasado como parmetro. Devuelve todos los
 *  datos del mismo. Devuelve la instancia de jugador localizada.
 */
jugador* tablero::buscar_jugador_color ( string color ) const
{
  int i = 0;

  while ( ( i < get_numJugadores () ) && ( juega[i]->get_color () != color ) )
    {      
      i++;
    }
  if ( i < get_numJugadores () )
    if ( juega[i]->get_color () == color )
      {
	return juega[i];
      }
  return ( NULL );
}

/**
 *  Mtodo que recorre la lista de jugadores del tablero buscando aquel
 *  que responde al patron de color pasado como parmetro. Devuelve todos los
 *  datos del mismo.  Devuelve la posicin del jugador localizado en el vector
 *  de jugadores.
 */
int tablero::buscar_jugador_color_int ( string color ) const
{
  int i = 0;

  while ( ( i < get_numJugadores () ) && ( juega[i]->get_color () != color ) )
    {      
      i++;
    }
  if ( i < get_numJugadores () )
    if ( juega[i]->get_color () == color )
      {
	return ( i) ;
      }
  return ( -1 );
}

/**
 *  Mtodo que recorre la lista de jugadores del tablero buscando aquel
 *  que responde al patron de nombre pasado como parmetro. Devuelve la posicin del 
 *  jugador localizado en el vector de jugadores.
 */
int tablero::buscar_jugador_nombre_int ( string nombre ) const
{
  int i = 0;

  while ( ( i < get_numJugadores () ) && ( juega[i]->get_nombre () != nombre ) )
    {      
      i++;
    }
  if ( i < get_numJugadores () )
    if ( juega[i]->get_nombre () == nombre )
      {
	return ( i) ;
      }
  return ( -1 );
}

/**
 * Metodo que crea una calle y la incluye dentro del tablero.
 * Adems relacionar la calle creada con su monopolio:
 *	- Si existe el monopolio, se relacionarn ambos objetos.
 *	- Si no existe, se crear el monopolio con la informacin existente.
 */
void tablero::incluir_calle ( int numero_casilla, string nombre, string nombreCorto, 
			      string color, long precio, long hipoteca, 
			      long alquiler0, long alquiler1, long alquiler2, 
			      long alquiler3, long alquiler4, long alquiler_h, 
			      int precio_casa, int precio_hotel )
{
  //Construye la calle con los datos recibidos
  calle *street = new calle ( numero_casilla, nombre, nombreCorto, precio, hipoteca, false, true, 
			      alquiler0,alquiler1, alquiler2, alquiler3, alquiler4, 
			      alquiler_h, false, precio_casa, precio_hotel, 0, 0, get_banca(), get_banca() );  

  //Inserta la calle en la lista de casillas
  int posInsercion = anyadir_casilla ( street );
  
  //Asociar la calle al monopolio correspondiente. No se hace en el constructor porque
  //desde la clase calle no se puede saber si ya existe ese monopolio porque no puedo recorrer
  //todas las calles
  int numCasilla = 0;
  bool existeMonopolio = false;

  // Busca la posicin en la que la insertara y comprueba si ya existe el monopolio
  while ( ( numCasilla < tablero::contiene.size() )  && 
	  ( existeMonopolio == false ) )
    {
      if ( calle* c = dynamic_cast <calle*> ( contiene[ numCasilla ] ) )      
	//Aseguramos que no entre en la casilla actual
	if ( posInsercion != numCasilla )
	  if ( c -> esMonopolio ( color ) == true )
	    existeMonopolio = true;	    	

      if ( existeMonopolio == false )
	numCasilla++;      
    }

  //Si ya existe ese monopolio, se le aade esta calle.
  if ( existeMonopolio == true )
    ( ( calle* ) street )->asignarMonopolio ( ( ( calle*) tablero::contiene[ numCasilla ] ) );
  //Si no existe el monopolio, lo creo y se lo asigno.
  else
    street->crearMonopolio ( color );
}

/**
 * Metodo que crea un servicio y la incluye dentro del tablero.
 * Adems relacionar el servicio creado con su monopolio:
 *	- Si existe el monopolio, se relacionarn ambos objetos.
 *	- Si no existe, se crear el monopolio con la informacin existente.
 */
void tablero::incluir_servicio ( int numero_casilla, string nombre, string nombreCorto, 
				 long precio, long hipoteca, int multiplicador1, 
				 int multiplicador2 )
{
  servicio *service = new servicio ( numero_casilla, nombre, nombreCorto, precio, hipoteca, 
				     false, true, multiplicador1, multiplicador2, get_banca () );

  int posInsercion = anyadir_casilla ( service );  
  int numCasilla = 0;  
  bool existeMonopolio = false;
  // Buscar la posicin en la que la insertar y busca si existe el monopolio
  while ( ( numCasilla < tablero::contiene.size() )  && 
	  ( existeMonopolio == false ) )
    {
      if ( servicio* s = dynamic_cast <servicio*> ( contiene[ numCasilla ] ) )      
	//Aseguramos que no entre en la casilla actual
	if ( posInsercion != numCasilla )
	  if ( s->esMonopolio ( "Servicio" ) == true )
	    existeMonopolio = true;
      
      if ( existeMonopolio == false )
	numCasilla++;      
    }
  
  //Si ya existe ese monopolio, se le aade esta calle.
  if ( existeMonopolio == true )
    ( ( servicio* ) service ) -> asignarMonopolio ( ( ( servicio*) tablero::contiene[ numCasilla ] ) );
  //Si no existe el monopolio, lo creo y se lo asigno.
  else
    service -> crearMonopolio ( "Servicio" );  
}

/**
 * Metodo que crea una estacin y la incluye dentro del tablero.
 * Adems relacionar la estacin creada con su monopolio:
 *	- Si existe el monopolio, se relacionarn ambos objetos.
 *	- Si no existe, se crear el monopolio con la informacin existente.
 */
void tablero::incluir_estacion ( int numero_casilla, string nombre, string nombreCorto, 
				 long precio, long hipoteca, long alquiler1, long alquiler2, 
				 long alquiler3, long alquiler4 )
{  
  //Construye la estacion con los datos recibidos
  estacion *station = new estacion ( numero_casilla, nombre, nombreCorto, precio, hipoteca, 
				     false, true, alquiler1, alquiler2, alquiler3, alquiler4, get_banca() );  
  //Inserta la estacion en la lista de casillas

  int posInsercion = anyadir_casilla ( station );  
  int numCasilla = 0;  
  bool existeMonopolio = false;
  // Busca la posicin en la que se insertar la casilla y adems comprobar si existe el monopolio
  while ( ( numCasilla < tablero::contiene.size() )  && 
	  ( existeMonopolio == false ) )
    {
      if ( estacion *e = dynamic_cast <estacion*> ( contiene[ numCasilla ] ) )      
	//Aseguramos que no entre en la casilla actual
	if ( posInsercion != numCasilla )
	  if ( e->esMonopolio ( "Estacion" ) == true )
	    existeMonopolio = true;
      
      if ( existeMonopolio == false )
	numCasilla++;      
    }
  
  //Si ya existe ese monopolio, se le aade esta calle.
  if ( existeMonopolio == true )
    ( ( estacion* ) station ) -> asignarMonopolio ( ( ( estacion*) tablero::contiene[ numCasilla ] ) );
  //Si no existe el monopolio, lo creo y se lo asigno.
  else
    station -> crearMonopolio ( "Estacion" );
  
}


/**
 * Mtodo que crea una casilla de impuestos y la incluye dentro del tablero 
 *  - IN: Todos los datos de una casilla de impuestos (precio, nombre, numero) 
 */
void tablero::incluir_impuesto ( int numero_casilla, string nombre, string nombreCorto, 
				 int precio )
{
  impuesto *imp = new impuesto ( numero_casilla, nombre, nombreCorto, precio );
  anyadir_casilla ( imp );
}


/** Mtodo que crea una casilla de impuestos especiales y la incluye dentro del tablero 
 *	- IN:	Todos los datos de una casilla de impuestos especiales (precio, nombre, 
 *		numero, tanto por ciento) 
 */
void tablero::incluir_impuesto_especial ( int numero_casilla, string nombre, string nombreCorto, 
					  int precio, int tpc )
{
  impuesto5 *imp5 = new impuesto5 ( numero_casilla, nombre, nombreCorto, precio, tpc );
  anyadir_casilla ( imp5 );
}

/**
 * Mtodo que crea una casilla de tipo tarjetas y la incluye en en su posicin del tablero.
 * Adems asociar la casilla a uno de los mazos creados en el tablero.
 */
void tablero::incluir_casilla_tarjetas ( int numero_casilla, string nombre, string nombreCorto, 
					 string mazo )
{
  casilla_tarjetas *cTarjetas = new casilla_tarjetas ( numero_casilla, nombre, nombreCorto );
  //Asignar la casilla a un mazo!!!
  anyadir_casilla ( cTarjetas );
  cTarjetas -> set_mazo ( _mazo[ mazo ] );
}

/**
 * Metodo que crea una casilla de tipo salida y la incluye dentro del tablero.
 */
void tablero::incluir_salida ( int numero_casilla, string nombre, string nombreCorto, 
			       long sueldo )
{
  salida *cSalida = new salida ( numero_casilla, nombre, nombreCorto, sueldo );
  anyadir_casilla ( cSalida );
  //Se indica cual es el numero de casilla de la salida en el atributo del tablero.
  set_salida ( numero_casilla );
}

/**
 * Metodo que crea una casilla genrica y la incluye dentro del tablero.
 */
void tablero::incluir_casilla_generica ( int numero_casilla, string nombre, string nombreCorto )
{
  casilla *casillas = new casilla ( numero_casilla, nombre, nombreCorto );
  anyadir_casilla ( casillas );
}

/**
 * Metodo que crea una casilla de tipo crcel y la incluye dentro del tablero.
 */
void tablero::incluir_carcel ( int numeroCasilla, string nombre, string nombreCorto, 
			       int maxTurnos, long multa )
{
  carcel *cCarcel = new carcel ( numeroCasilla, nombre, nombreCorto, maxTurnos, multa );
  anyadir_casilla ( cCarcel );
}

/**
 * Metodo que crea una casilla del tipo 'vaya a la crcel' y la incluye dentro del tablero.
 */
void tablero::incluir_encarceladora ( int numeroCasilla, string nombre, string nombreCorto )
{
  encarceladora *cEncarceladora = new encarceladora ( numeroCasilla, nombre, nombreCorto );
  anyadir_casilla ( cEncarceladora );
}

/**
 * Mtodo que cierra la partida actual liberando memoria y estableciendo la partida como cerrada.
 * Una vez ejecutado, no podr seguir jugndose una partida.
 */
void tablero::cerrar ( void )
{  
  tablero::contiene.clear ();
  juega.clear ();
  set_nombrePartida ( "desconocido" );
  set_ciudad ( "Madrid" );
  set_moneda ( "pesetas" );
  set_fichCasillas ( "" );
  set_fichSuerte ( "" );
  set_fichCajaComunidad ( "" );
  set_numJugadores ( 0 );
  set_numCasillas ( 0 );
  set_partidaAbierta ( false );
  set_salida ( -1 );
  set_turno ( 0 );
}

/**
 * Mtodo que reorganiza el tablero para que coincida el nmero de casilla con su
 * posicin en el vector.
 */
int tablero::recolocar ( void )
{
  //Modifica los numeros de casilla para que coincidan
  for ( int i=0; i < tablero::contiene.size(); i++ )
    {
      if ( ( tablero::contiene[i]->get_numero() ) != i )
	tablero::contiene[i]->set_numero(i);
    }
}

/** 
 *  Mtodo que verifica que la insercin de casillas ha sido correcta, es decir,
 *  coinciden los numeros de casilla con su orden en el vector 
 *  -Si estn bien colocadas = true
 *  -Si estn mal colocadas = false
 */
bool tablero::verificar ( void )
{
  bool colocadas = true;

  //Recorre la lista comprobando si estn todas bien colocadas
  for ( int i=0; i < tablero::contiene.size(); i++ )
    {
      if ( ( tablero::contiene[i]->get_numero() ) != i )
	colocadas = false;
    }
  
  return colocadas;
}

/** 
 * Mtodo que coloca un jugador en una casilla en funcin de los parmetros que se pasan:
 * La variable numJugador indica su orden dentro del array de jugadores y 'posicion'es el 
 * numero de casilla en la que se debe colocar.   
 */
void tablero::colocarJugador ( int numJugador, int numCasilla )
{  
  //Relaciona casilla con jugador y jugador con casilla.
  tablero::contiene[ numCasilla ] -> situarJugadorEnCasilla ( juega[ numJugador ] ); 
  juega[ numJugador ]->set_estaEn ( tablero::contiene[ numCasilla ] );
}


/** 
 *  Mtodo que establece que jugador iniciara la partida. Para ello lanzarn los dados cada
 *  uno de los jugadores dados de alta en la misma, lanzando una vez cada uno, y se seleccionar
 *  como primer jugador aquel que lance el dado y obtenga un nmero ms elevado. En caso de 
 *  empate, volvern a lanzar aquellos jugadores que hayan empatado en la tirada anterior.
 *  MTODO:
 *	1 - Recorre el vector de jugadores juega, haciendo que cada jugador lance el dado.
 *	2 - Comprueba si el valor es mximo entre los ya lanzados (se almacena el mximo en
 *	    la variable max (int).
 *	    2.1 - Si el valor es mximo: se cambiar la variable max por el nuevo valor y se
 *	          reinicia el vector de jugadores que han lanzado ese valor (maximo). Se borra
 *		  su contenido y se inserta el entero 'i' que identifica a ese jugador dentro
 *		  del vector juega.
 *	    2.2 - Si NO es mximo: no se hace nada
 *	    2.3 - Si el valor es igual que el mximo actual, se inserta el identificador entero
 *	          'i' en el vector de jugadores que han lanzado ese valor.
 *	3 - Se coprueba si ha habido varios jugadores que han lanzado el valor mximo (variable
 *	    entera 'repetido = maximo.size()')
 *	    3.1 - Si NO existen ms de 1 jugador en la lista mximo: FIN (el primero es el 
 *	          jugador que se encuentra en el vector juega en la posicion que indica 
 *		  maximo[0].
 *	    3.2 - Si existe ms de uno, se repite la operacion pero slo entre los jugadores
 *	          que aparecen en el vector 'maximo', hasta que slo quede 1 jugador.
 */
string tablero::elegirOrdenTirada ( void )
{
  vector <int> maximo;	//Jugadores que han lanzado 'max'
  int max = 0;		//Valor mximo obtenido hasta ahora
  int i = 0;		//Iterador
  int resultado;	//Resultado obtenido de lanzar el dado.
  // Lanzan todos los jugadores y se buscan los mximos
  while ( i < get_numJugadores() )
    {
       cout << juega[ i ]->get_nombre() << " pulse 'INTRO' para lanzar su dado: ";
       if ( dynamic_cast <jugador_reglas *> ( juega[ i ] ) )
	 cout << " (Jugador automtico)" << endl;
       else
	 getchar();
      
      int dado1, dado2;
      bool tiradaDoble;
      resultado = juega[i]->tirarDado ( tiradaDoble, dado1, dado2 );
      cout << "\t-Total lanzado = " << resultado << " ( " << dado1 << " + " << dado2 << " )" << endl;      
      if ( resultado == max )		//Si iguala el mximo anterior se inserta en el vector de maximos
	  maximo.push_back ( i );
      else 
	if ( resultado > max )		//Si supera el resultado anterior (borra e inserta)
	  {
	    max = resultado;
	    maximo.clear();
	    maximo.push_back ( i );
	  }
      i++;
    }  
  
  int numJugador;			//identifica el orden que ocupa un jugador en juega
  // Si han coincidido dos jugadores con la misma tirada repiten el proceso entre ellos
  // Lo haran hasta que desempaten
  while ( maximo.size() > 1 )
    {
      int repetidos = maximo.size ();
      cout << endl <<  "Varios jugadores han obtenido el mismo valor. Toca desempate entre ellos:" << endl << endl;      
      if ( repetidos > 1 )
	{
	  max = 0;
	  i = 0;
	  while ( i < repetidos )
	    {
	      numJugador = maximo[i];
	      cout << juega[numJugador]->get_nombre() << " pulse 'INTRO' para lanzar su dado: ";
	      if ( dynamic_cast <jugador_reglas *> ( juega[ numJugador ] ) )		
		cout << " (Jugador automtico)" << endl;
	      else
		getchar();
	      int dado1, dado2;
	      bool tiradaDoble;
	      resultado = juega[numJugador]->tirarDado ( tiradaDoble, dado1, dado2 );
	      cout << "\t-Total lanzado = " << resultado << " ( " << dado1 << " + " << dado2 << " )" << endl;
	      if ( resultado == max )
		maximo.push_back( numJugador );
	      else  
		if ( resultado > max )
		  {
		    max = resultado;
		    maximo.clear();
		    maximo.push_back ( numJugador );
		  }
	      i++;
	    }
	}
    }
  
  numJugador = maximo[0];
  cout << endl << ">>>> " << juega[numJugador]->get_nombre() << ", lanzas en primer lugar." << endl;
  set_turno ( numJugador );
  set_jugadorActual ( numJugador );
  return juega[numJugador]->get_nombre();
}

/**
 * Mtodo que dibuja una representacin del movimiento desde la casilla origen hasta
 * la casilla destino. Devolver una cadena de caracteres con este dibujo.
 */
string  tablero::dibujarMovimiento ( int casillaOrigen, int casillaDestino )
{
  string aux;
  aux = "-Casilla de origen\n";
  aux += " -----------------\n";
  aux += tablero::contiene[ casillaOrigen ] -> dibujarCasillaEnTablero ();
  aux += "\n-Casilla de destino\n";
  aux += " ------------------\n";
  aux += tablero::contiene[ casillaDestino ] -> dibujarCasillaEnTablero ();
  return aux;
}

/**
 * Subastar una propiedad indicada como argumento
 */
int tablero::subastar ( propiedad *p)
{
  if ( get_subastas () == true )
    {
      cout << "Se va ha subastar la propiedad entre todos los jugadores" << endl;
      subastarPropiedad ( p, juega );
    }
  return 0;
}

/**
 * Ofrece la libertad a un jugador que est en la crcel. Si tiene tarjetas para salir se le
 * ofrecer salir usandolas. Tambin se ofrece salir pagando la multa. Si el jugador sale se
 * le desencarcelar inmediatamente.
 */
int tablero::ofrecerLibertad ( jugador* j )
{
  int ok = 0;
  bool salir = false;
  //se comprueba que esta en la carcel  
  if ( carcel *car = dynamic_cast <carcel *> ( j-> get_estaEn() ) )
    {
      //Si tiene tarjetas de carcel, se le ofrece salir usandola
      if ( j->tieneTarjetasCarcel () > 0 )
	{	  
	  bool decidirSalir = j->decidirSalirTarjeta ( propiedadesLibres () );	 
	  if ( decidirSalir )
	    {
	      //Si decide salir, se desencarcela al jugador y se devuelve la tarjeta al mazo
	      car->desencarcelar ( j );	    
	      string tipoMazo = j->mazoTarjetaSuerte ();	      
	      ok = _mazo[ tipoMazo ] -> devolverTarjeta ( j );
	      salir = true;
	    }
	}
      if ( salir == false )
	{
	  //Si no decide salir o no tiene tarjetas, se le pregunta si desea salir pagando
	  bool decidirSalir = j->decidirSalirPagando ( propiedadesLibres (), car->get_multa () );	 
	  if ( decidirSalir )
	    //Se le desencarcela
	    ok = car->desencarcelarPagando ( j );
	  else
	    {
	      if ( dynamic_cast <jugador_reglas *> ( j ) )		
		cout << "Entonces se lanzan los dados" << endl;	      
	      else
		{
		  cout << "Entonces se procede a lanzar los dados. Pulse intro para continuar.";
		  getchar();
		}
	    }
	}
    }
  return ok;
   
}

/**
 * Se evalua una accion retornada de las casillas. Es posible que el jugador espere para
 * conseguir dinero (1), no haga nada ms (0) o se declare en bancarrota (-1) 
 */
string tablero::evaluarAccion ( jugador* j, int resultadoAccion )
{
  string prompt;

  //Analizamos y actuamos en consecuencia en función del resultado de la acción:
  // -1: Bancarrota
  //  0: Nada
  //  1: Esperar
  switch ( resultadoAccion )
    {   
    case -1:	//Bancarrota
      //Eliminar al jugador (turno inicial) que ha caido en bancarrota
      declararBancarrota( j );
      //Jugar quien tenga el turno (por asignación)
      set_jugadorActual ( get_turno() );
      prompt = " (" + juega[ get_turno() ]->get_nombre () + ")";
      break;

    case 0:	//Nada
      if ( ( j->get_estado() == 0 )
	   && ( j->get_dinero_total() >= 0 ) )
	{
	  set_jugadorActual ( get_turno() );
	  prompt = " (" + juega[ get_turno() ]->get_nombre () + ")";
	}
      else 
	{
	  cerr << "No puedes continuar hasta no saldar tus deudas" << endl << endl;
	  prompt = " (" + juega[ get_turno() ]->get_nombre () + ")" + " [~" + j->get_nombre () + "]";
	}
      break;

    case 1:	//Esperar
      cout << "Puedes reunir el dinero necesario hasta que des por finalizado tu turno (comando fin)."<< endl;
      prompt = " (" + juega[ get_turno() ]->get_nombre () + ")" + " [~" + j->get_nombre () + "]";
      break;  
   
    default:
      set_jugadorActual ( get_turno() );
      prompt = " (" + juega[ get_turno() ]->get_nombre () + ")";	  
    }
  return prompt;
}

/**
 * Mtodo que mueve un jugador en funcin del resultado obtenido por la tirada de los 
 * dados. Se debe tener en cuenta cuando se lanza una tirada doble ( tambin cuando se
 * enviar al jugador a la crcel ).
 */
string tablero::moverJugador ()
{
  string prompt;	//Valor de retorno
  bool tiradaDoble;
  int resultadoAccion;
  bool permitirMovimiento = true;

  //Comprobar si existen deudas antes de tirar en alguno de los jugadores.
  if ( existenDeudas () )
    {
      cerr << "Existen jugadores con deudas, deben saldar sus deudas o declararse en bancarrota antes de lanzar" << endl;
      cerr << "Morosos:" << endl;
      cerr << verMorosos();
      cerr << "Use el comando 'cambiar' para pasar el poder al jugador moroso" << endl;
      prompt = " (" + juega[ get_turno() ]->get_nombre () + ")";
    }
  //Si no hay deudas se mueve el jugador.
  else
    {
      cout << juega [ get_turno() ]->get_nombre() << ":" << endl;
      // Posicin actual del jugador
      int turnoInicial = get_turno();
      //cout << "turno inicial = " << turnoInicial << endl;
      int numCasillaInicial = juega [ get_turno() ]->obtenerCasilla();  
      
      // Si esta en la crcel, se le debe ofrecer la posibilidad de salir pagado
      if ( juega[ turnoInicial ]->estaEncarcelado () )
	resultadoAccion = ofrecerLibertad ( juega[ turnoInicial ] );
      
      // Tirada de los dados
      int dado1, dado2;
      int totalDados = juega[ turnoInicial ]->tirarDado ( tiradaDoble, dado1, dado2 );
      //cout << endl;
      cout << "DADO 1 = " << dado1 << "    ";
      cout << "DADO 2 = " << dado2 << "    ";
      cout << "TOTAL = " << totalDados << endl;
      // Si no est encarcelado...
      if ( ( tiradaDoble ) && ( ! juega[ turnoInicial ]->estaEncarcelado () ) )
	{
	  cout << "            (tirada doble)";
	  int tiradasDobles = juega[ turnoInicial ]->get_tiradasDobles();
	  juega[ turnoInicial ]->set_tiradasDobles ( tiradasDobles++ );
	  permitirMovimiento = true;
	}
      // Si esta en la crcel y no saca una tirada doble
      else if ( ( ! tiradaDoble ) && ( juega[ turnoInicial ]->estaEncarcelado () ) )
	{
	  cout << "Sigues en la crcel una tirada ms. Aun quedan " << juega[ turnoInicial ]->get_turnosEncarcelado()<< endl;
	  juega[ turnoInicial ]->set_turnosEncarcelado ( juega[ turnoInicial ]->get_turnosEncarcelado() - 1 );
	  permitirMovimiento = false;
	  set_turno( (turnoInicial + 1) % get_numJugadores() );
	}
      //Si saca una tirada doble sale de la carcel
      else if ( ( tiradaDoble ) && ( juega[ turnoInicial ]->estaEncarcelado() ) )
	{
	  tiradaDoble = false;   //No cuenta como tirada doble la salida de la crcel
	  if ( carcel *car = dynamic_cast <carcel *> ( juega [ turnoInicial ]-> get_estaEn() ) )
	    car->desencarcelar( juega[ turnoInicial ] );	  
	  permitirMovimiento = true;
	}
      cout << endl;  
      if ( permitirMovimiento )
	resultadoAccion = mover ( juega[ turnoInicial ], totalDados, tiradaDoble, true );  
      else 
	resultadoAccion = 0;

      prompt = evaluarAccion ( juega[turnoInicial], resultadoAccion );        

      //Se informa que vuelve a tirar, a no ser que el jugador caiga en bancarrota o se encarcele
      if ( tiradaDoble == true && juega[ get_turno() ]->get_tiradasDobles() != 2 && resultadoAccion >=0 && 
	   juega[ get_turno() ]->get_turnosEncarcelado() > 0 )
	cout << "Vuelves a tirar porque has sacado dos números iguales en los dados" << endl;
    }
  return prompt;
}

/**
 * Cada jugador razonar si invierte dinero en algo o no. Para ello seguira el proceso: intentar
 * deshipotecar, intentar construir y si no hay dinero, declarar bancarrota.
 */
void tablero::razonarJugadores ( string &prompt )
{  
  //Si existen deudas, se comprueba si pertenecen a jugadores automticos, y se eliminan directamente,
  //porque ya han hecho todo lo posible para seguir en el juego
  for ( int i = 0; i < juega.size(); i++ )
    {
      if ( juega [ i ]->get_estado () < 0 )	
	if ( dynamic_cast <jugador_reglas*> ( juega[i] ) )
	  declararBancarrota ( juega[ i ] );      
    }
  if ( juega.size() > 1 )
    for ( int i = 0; i < juega.size(); i++ )
      {      
	if ( jugador_reglas *j = dynamic_cast <jugador_reglas*> ( juega[i] ) )
	  {	  
	    //1- Deshipotecar	  
	    j->razonarDeshipotecas ();
	    //2- Construir
	    j->razonarConstrucciones ();
	  }
      }
  // Establece el jugador actual como el turno.
  if ( get_turno() == get_jugadorActual () )
    prompt = "(" + juega[ get_turno() ]->get_nombre() + ")";        
  else
    {
      prompt = "(" + juega[ get_turno() ]->get_nombre() + ") [~" + juega[get_jugadorActual()]->get_nombre () + "]";
      set_jugadorActual ( get_turno() );
    }
}

/**
 * Muestra la leyenda del tablero, indicando todos los atributos de cada casilla y explicando
 * el significado de cada simbolo (se encuentra en un fichero de texto)
 */
string tablero::leyenda ()
{
  const int anchoNum = 4;
  const int anchoNombreCorto = 6;
  const int anchoTipo = 6;
  const int anchoNombre = 30;
  const int anchoEstado = 6;
  string aux; //string donde se devolverá la cadena de caracteres.
  aux = "-CASILLAS:\n";
  //CABECERA
  aux += "Num. Nombre Tipo* Nombre                         Estado  Num. Nombre Tipo* Nombre                         Estado\n";
  aux += "     corto                                                    corto\n";
  aux += "---- ------ ----- ------------------------------ ------  ---- ------ ----- ------------------------------ ------\n";
  int casillasColumna = get_numCasillas() / 2;
  if ( get_numCasillas() % 2 == 1 )
    casillasColumna = casillasColumna + 1;

  //CASILLAS
  // Muestra la informacin de cada casilla en dos columnas.
  for ( int i = 0; i < casillasColumna; i++ )
    {
      //Casilla i
      aux += tablero::contiene[i]->leyendaCasilla ( anchoNum, anchoNombreCorto, anchoTipo, anchoNombre, anchoEstado ); //Se pasan la longitud de cada campo.
      //Casilla i + numero de casillas / 2
      if ( i + casillasColumna < get_numCasillas() )
	aux += tablero::contiene[i + casillasColumna]->leyendaCasilla ( anchoNum, anchoNombreCorto, anchoTipo, anchoNombre, anchoEstado );
      aux += "\n";
    }

  //PIE DE PGINA
  string contenido = "";
  int ok = obtener_fichero ( "leyenda.txt", contenido);
  if ( ok == 0)
    aux += contenido;  
  return aux;
}

/**
 * Crea un mazo de tarjetas y carga todas sus tarjetas desde el fichero correspondiente
 * al tipo de tarjetas que se pasa como argumento. El fichero se recoge de la instancia
 * de clase tablero.
 */
int tablero::crearMazo ( string tipo )
{
  string fichero;

  mazo *mazoTarjetas = new mazo ( tipo );
  
  //Selecciona el fichero (tarjetasSuerte es una constante de configuracion.h)
  if ( tipo == tarjetasSuerte )
    fichero = get_fichSuerte ();
  else if ( tipo == tarjetasCaja )    
    fichero = get_fichCajaComunidad ();
  
  //Carga las tarjetas del mazo en el mazo creado
  int ok = mazoTarjetas -> cargar_tarjetas ( fichero, tipo );
  if ( ok == -1 )    
      cerr << "Error cargando el mazo de " << tipo << endl;  

  // Asigna el mazo al tablero
  set_mazo ( tipo, mazoTarjetas );
  return ok;
}

/**
 * Mtodo que muestra la informacin resumida sobre todos los jugadores 
 * de la partida actual en forma de tabla.
 */
int tablero::ver_lista_jugadores ( )
{
  string nombre, color;
  
  cout << endl;
  cout << "TURNO\tNOMBRE\t\t\t\tCOLOR\t\tDINERO\tCAPITAL\tDEUDAS" << endl;
  cout << "------------------------------------------------------------------------------" << endl;
  for ( int i=0; i < get_numJugadores (); i++ )
    {	      		      
      cout << i;
      // Nombre del jugador
      nombre = juega[i]->get_nombre();
      if ( nombre.size () > 30 )
	cout << "\t" << nombre.substr ( 0, 30 ) << ".";
      else
	cout << "\t" <<  nombre;      
      if ( nombre.length () < 8 )
	cout << "\t\t\t";
      else if ( nombre.length () < 16 )
	cout << "\t\t";
      else if ( nombre.length () < 24 )
	cout << "\t";

      //Color
      color = juega[i]->get_color ();
      cout << "\t" << color;
      if ( color.length () < 8 )
	cout << "\t";
      
      //Dinero total
      cout << "\t" << juega[i]->get_dinero_total();

      //Capital
      cout << "\t" << juega[i]->get_capital();

      //Estado (deudas)
      cout << "\t" << juega[i]->get_estado () << endl;
    } //for
  cout << endl;	
  return ( 0 );	     
}


/** 
 *  Mtodo que muestra toda la informacin sobre el jugador indicado
 *  El identificador principal de los jugadores es el color, luego
 *  se buscar los jugadores por este atributo 
 */
int tablero::ver_jugador ( string color )
{
  //Busca el jugador por el color indicado como argumento
  jugador* j = buscar_jugador_color( color );
  if ( j == NULL )
    {
      cerr << "No existe ningun jugador que responda a una ficha de ese color" << endl << endl;
      return ( -1 );
    }
  //Muestra el jugador encontrado
  else
    {
      cout << j;
      cout << endl;
      cout << endl;
      return ( 0 );      
    }
}

/** 
 * Mtodo que muestra toda la informacin sobre el jugador indicado
 * Se buscar al jugador por su turno (posicion en el vector 'juega')  
 */
int tablero::ver_jugador ( int pos )
{
  if ( pos >= get_numJugadores () )
    {
      cerr << "No existe ningun jugador que responda a una ficha de ese color" << endl << endl;
      return ( -1 );
    }
  else
    {
      cout << juega[ pos ];
      cout << endl;
      cout << endl;
      return ( 0 );      
    }
}

/**
 * Cambia el poseedor del control del juego al jugador con el turno que se indica en el argumento.
 * Este ser el jugador sobre el que recaeran las acciones hasta que se vuelva a cambiar.
 */
string tablero::cambiarJugadorActual ( int nuevoJugadorActual )
{
  string prompt;
  set_jugadorActual ( nuevoJugadorActual % get_numJugadores() );
  
  if ( get_jugadorActual() == get_turno() )    
    prompt = " (" + juega[ get_turno() ]->get_nombre () + ")";    
  else    
    prompt = " (" + juega[ get_turno() ]->get_nombre () + ")" + " [~" + juega[ get_jugadorActual() ]->get_nombre () + "]"; 
  return prompt;
  
}

/**
 * Cambia el poseedor del control del juego al jugador que se pasa como argumento. Se deben
 * indicar el campo por el que se realizar el cambio y el valor de dicho campo.
 */
string tablero::cambiarJugadorActual ( string campo, string valor )
{
  string prompt;
  // En funcion del campo de busqueda
  if ( campo == "-color" )
    {
      //Busca el jugador
      int nuevoJugadorActual = buscar_jugador_color_int ( valor );
      if ( nuevoJugadorActual >= 0 )
	// Se cambia el jugador actual al nuevo encontrado
	prompt = cambiarJugadorActual ( nuevoJugadorActual );
      else
	{
	  cerr << "No existe ningn jugador con una ficha de color " << valor << endl;
	  prompt = " (" + juega[ get_turno() ]->get_nombre () + ")";    
	}
    }
  //Busca por nombre
  else if ( campo == "-nombre" )
    {
      //Busca el nuevo jugador
      int nuevoJugadorActual = buscar_jugador_nombre_int ( valor );
      if ( nuevoJugadorActual >= 0 )
	//Establece el nuevo jugador actual
	prompt = cambiarJugadorActual ( nuevoJugadorActual );
      else
	{
	  cerr << "No existe ningn jugador con el nombre " << valor << endl;
	  prompt = " (" + juega[ get_turno() ]->get_nombre () + ")";    
	}
    }
  return ( prompt );
}

/**
 * Mtodo que muestra todas las propiedades hipotecables del jugador actual
 */
string tablero::verHipotecables ()
{
  return ( juega[ get_jugadorActual() ]->listarPropiedadesHipotecables() );
}

/**
 * Mtodo que muestra todas las propiedades hipotecadas del jugador actual
 */
string tablero::verHipotecados ()
{
  return ( juega[ get_jugadorActual() ]->listarPropiedadesHipotecadas() );
}

/**
 * Mtodo que muestra todas las propiedades edificables del jugador actual
 */
string tablero::verConstruibles ()
{
  return ( juega[ get_jugadorActual() ]->listarCallesConstruibles() );
}

/**
 * Mtodo que muestra todas las calles edificadas del jugador actual
 */
string tablero::verConstruidas ()
{
  return ( juega[ get_jugadorActual() ]->listarCallesConstruidas() );
}

/**
 * Mtodo que hipoteca la propiedad que se indica en la variable valor.
 * En esta versin solo se puede buscar por el atributo "numero".
 * Al hipotecar la propiedad el jugador recibe los ingresos automticamente.
 */
int tablero::hipotecar ( string atributo, string valor )
{  
  int ok = 0;
  if ( atributo == "numero" )
    {
      int numCasilla = atoi ( valor.c_str() );
      //Se dirige a la posicion del vector que se corresponde con la propiedad indicada
      //Para ser hipotecada debe ser una propiedad
      if ( propiedad *p = dynamic_cast <propiedad *> ( tablero::contiene[ numCasilla] ) )
	{
	  //Se comprueba que el jugador que quiere hipotecar (jugadorActual) es el propietario
	  if ( p->get_esPropiedad() == juega[ get_jugadorActual() ] )
	    {
	      //Hipotecar e informar del resultado
	      ok = p->hipotecar();
	      if ( ok == 0 )
		cout << p->get_nombre() << " ha sido hipotacada" << endl;
	      else
		cout << "No se ha podido hipotecar " << p->get_nombre() << ". Es posible que ya se encuentre hipotecada o no pueda hipotecarse por tener ella, o alguna de las propiedades de su grupo, algn edificio construido" << endl;
	    }
	  else
	    {
	      ok = -1;
	      cerr << "No se puede hipotecar una propiedad ajena" << endl;
	    }
	}
      else
	{
	  ok = -1;
	  cerr << "No se puede hipotecar una casilla que no es una propiedad" << endl;
	}
    }
  return ( ok );
}

/**
 * Mtodo que deshipoteca la propiedad que se indica en la variable valor.
 * En esta versin solo se puede buscar por el atributo "numero".
 * Al deshipotecar la propiedad el jugador paga automticamente y su propiedad vuelve
 * a estar disponible para cobrar, construir...
 */
int tablero::deshipotecar ( string atributo, string valor )
{
  int ok = 0;
  if ( atributo == "numero" )
    {
      int numCasilla = atoi ( valor.c_str() );
      if ( numCasilla < get_numCasillas() )
	{
	  //Se obtiene la casilla que se ha solicitado en el atributo valor.
	  if ( propiedad *p = dynamic_cast <propiedad *> ( tablero::contiene[ numCasilla] ) )
	    {
	      //Se comprueba que el jugador que quiere deshipotecar (jugador actual) es el propietario
	      if ( p->get_esPropiedad() == juega[ get_jugadorActual() ] )
		{
		  //Deshipotecar la propiedad
		  ok = p->deshipotecar();
		  if ( ok == 0 )
		    cout << p->get_nombre() << " ha sido deshipotacada" << endl;
		  else if ( ok == -2 )
		    cout << "No se dispone de dinero suficiente para levantar la hipoteca de " << p->get_nombre() << endl;
		  else
		    cout << "No se ha podido deshipotecar " << p->get_nombre() << ". Es probable que no se encuentre hipotecada." << endl;
		}
	      else
		{
		  ok = -1;
		  cerr << "No se puede deshipotecar una propiedad ajena" << endl;
		}
	    }
	  else
	    {
	      ok = -1;
	      cerr << "No se puede deshipotecar una casilla que no es una propiedad" << endl;
	    }
	}
      else
	{
	  cerr << "ERROR: Casilla fuera de rango" << endl;
	  ok = -1;
	}
    }
  return ( ok );
}

/**
 * Mtodo que construye un edificio la propiedad que se indica en la variable valor.
 * En esta versin solo se puede buscar por el atributo "numero".
 * Al construir la propiedad el jugador paga automticamente y se aade el edificio
 * a la propiedad.
 * La propiedad en este caso debe ser de tipo calle.
 */
int tablero::construir ( string atributo, string valor )
{
  int ok = 0;
  if ( atributo == "numero" )
    {
      int numCasilla = atoi ( valor.c_str() );
      if ( numCasilla < get_numCasillas () )
	{
	  //Se situa al jugador en la casilla indicada
	  if ( calle *c = dynamic_cast <calle *> ( tablero::contiene[ numCasilla] ) )
	    {
	      //Se comprueba que el jugador actual es el propietario
	      if ( c->get_esPropiedad() == juega[ get_jugadorActual() ] )
		{
		  ok = c->construir();
		  if ( ok == 0 )
		    cout << c->get_nombre() << " ha sido construida." << endl;
		  else
		    cout << "No se ha podido construir en " << c->get_nombre() << ". Es posible que se encuentre hipotecada, no se pueda construir por no cumplir las reglas del juego en cuestiones de construccin o no queden propiedades en la banca. " << endl;
		}
	      else
		{
		  ok = -1;
		  cerr << "No se puede construir una propiedad ajena" << endl;
		}
	    }
	  else
	    {
	      ok = -1;
	      cerr << "No se puede construir una casilla que no es una calle" << endl;
	    }
	}    
      else
	{
	  cerr << "ERROR: Casilla fuera de rango" << endl;
	  ok = -1;
	}
    }
  return ( ok );
}

/**
 * Mtodo que vende un edificio la propiedad que se indica en la variable valor.
 * En esta versin solo se puede buscar por el atributo "numero".
 * Al vender la propiedad el jugador cobra automticamente y se elimina el edificio
 * de la propiedad.
 * La propiedad en este caso debe ser de tipo calle.
 */
int tablero::vender ( string atributo, string valor )
{
  int ok = 0;
  if ( atributo == "numero" )
    {
      int numCasilla = atoi ( valor.c_str() );
      // Se busca la propiedad indicada en el atributo valor
      // Debe ser una calle
      if ( calle *c = dynamic_cast <calle *> ( tablero::contiene[ numCasilla] ) )
	{
	  // Comprueba si el jugador actual es el propietario
	  if ( c->get_esPropiedad() == juega[ get_jugadorActual() ] )
	    {
	      //Vender
	      ok = c->vender();
	      if ( ok != 0 )	
		cout << "No se ha podido vender un edificio de " << c->get_nombre() << ". Es posible que no se encuentre construida o no queden propiedades en la banca, en caso de venderse un hotel." << endl;
	    }
	  else
	    {
	      ok = -1;
	      cerr << "No se pueden vender edificios de una propiedad ajena" << endl;
	    }
	}
      else
	{
	  ok = -1;
	  cerr << "No se puede vender un edificio de una casilla que no es una calle" << endl;
	}
    }
  return ( ok );
}

/**
 * Mtodo que busca la siguiente propiedad del tipo indicada a partir de la posicion en la 
 * que se encuentra el jugador pasado como argumento del mtodo.
 */
casilla* tablero::buscarSiguienteCasillaTipo ( jugador* j, string tipo )
{
  //Busca la casilla en la que esta
  casilla* c = j->get_estaEn();
  int casillaActual = c->get_numero();  
  int posible = casillaActual;
  bool encontrado = false;
  do
    { 
      //Obtiene el siguiente numero de casilla y comprueba si se corresponde con el tipo pasado como argumento.
      posible= (posible + 1) % contiene.size();
      if ( tipo == "carcel" )
	{
	  if ( dynamic_cast <carcel*> ( contiene[ posible ] ) )      
	    encontrado = true;
	}
      else if ( tipo == "calle" )
	{
	  if ( dynamic_cast <calle*> ( contiene[ posible ] ) )      
	    encontrado = true;
	}
      else if ( tipo == "estacion" )
	{
	  if ( dynamic_cast <estacion*> ( contiene[ posible ] ) )      
	    encontrado = true;
	}
      else if ( tipo == "servicio" )
	{
	  if ( dynamic_cast <servicio*> ( contiene[ posible ] ) )      
	    encontrado = true;
	}
      else if ( tipo == "salida" )
	{
	  if ( dynamic_cast <salida*> ( contiene[ posible ] ) )      
	    encontrado = true;
	}
      else if ( tipo == "impuesto" )
	{
	  if ( dynamic_cast <impuesto*> ( contiene[ posible ] ) )      
	    encontrado = true;
	}
      else if ( tipo == "impuesto5" )
	{
	  if ( dynamic_cast <impuesto5*> ( contiene[ posible ] ) )      
	    encontrado = true;
	}
      else if ( tipo == "casilla_tarjetas" )
	{
	  if ( dynamic_cast <casilla_tarjetas*> ( contiene[ posible ] ) )      
	    encontrado = true;
	}
      else if ( tipo == "encarceladora" )
	{
	  if ( dynamic_cast <encarceladora*> ( contiene[ posible ] ) )      
	    encontrado = true;
	}
      else if ( tipo == "propiedad" )
	{
	  if ( dynamic_cast <propiedad*> ( contiene[ posible ] ) )      
	    encontrado = true;
	}
      else if ( tipo == "casilla" )
	{
	  if ( dynamic_cast <casilla*> ( contiene[ posible ] ) )      
	    encontrado = true;
	}      
    }
  while ( posible != casillaActual && encontrado == false );
  if ( encontrado == true )
    return tablero::contiene[ posible ];
  return ( NULL );
}

/**
 * Mtodo que busca la siguiente propiedad del tipo indicada a partir de la posicion en la 
 * que se encuentra el jugador pasado como argumento del mtodo. Devuelve su posicion en el
 * vector de casilla.
 */
int tablero::buscarSiguienteCasillaTipoInt ( jugador* j, string tipo )
{
  casilla* c = j->get_estaEn();
  int casillaActual = c->get_numero();  
  int posible = casillaActual;
  bool encontrado = false;
  do
    { 
      //Obtiene el siguiente numero de casilla y comprueba si se corresponde con el tipo pasado como argumento.
      posible= (posible + 1) % contiene.size();
      if ( tipo == "carcel" )
	{
	  if ( dynamic_cast <carcel*> ( contiene[ posible ] ) )      
	    encontrado = true;
	}
      else if ( tipo == "calle" )
	{
	  if ( dynamic_cast <calle*> ( contiene[ posible ] ) )      
	    encontrado = true;
	}
      else if ( tipo == "estacion" )
	{
	  if ( dynamic_cast <estacion*> ( contiene[ posible ] ) )      
	    encontrado = true;
	}
      else if ( tipo == "servicio" )
	{
	  if ( dynamic_cast <servicio*> ( contiene[ posible ] ) )      
	    encontrado = true;
	}
      else if ( tipo == "salida" )
	{
	  if ( dynamic_cast <salida*> ( contiene[ posible ] ) )      
	    encontrado = true;
	}
      else if ( tipo == "impuesto" )
	{
	  if ( dynamic_cast <impuesto*> ( contiene[ posible ] ) )      
	    encontrado = true;
	}
      else if ( tipo == "impuesto5" )
	{
	  if ( dynamic_cast <impuesto5*> ( contiene[ posible ] ) )      
	    encontrado = true;
	}
      else if ( tipo == "casilla_tarjetas" )
	{
	  if ( dynamic_cast <casilla_tarjetas*> ( contiene[ posible ] ) )      
	    encontrado = true;
	}
      else if ( tipo == "encarceladora" )
	{
	  if ( dynamic_cast <encarceladora*> ( contiene[ posible ] ) )      
	    encontrado = true;
	}
      else if ( tipo == "propiedad" )
	{
	  if ( dynamic_cast <propiedad*> ( contiene[ posible ] ) )      
	    encontrado = true;
	}
      else if ( tipo == "casilla" )
	{
	  if ( dynamic_cast <casilla*> ( contiene[ posible ] ) )      
	    encontrado = true;
	}      
    }
  while ( posible != casillaActual && encontrado == false );
  if ( encontrado == true )
    return  posible;
  return ( -1 );
}

/**
 * Metodo que mueve a un jugador a la casilla de carcel sin pagar por pasar por la salida.
 * Establece al jugador como encarcelado.
 */
void tablero::encarcelar ( jugador* j )
{
  //Busca la casilla de tipo carcel.
  casilla* casillaDestino = buscarSiguienteCasillaTipo ( j, "carcel" );
  // Si la encuentra encarcela al jugador
  if ( casillaDestino != NULL )
    { 
      moverA( j, casillaDestino->get_numero() );
      cout << "Bienvenido a la crcel" << endl;
      if ( carcel *car = dynamic_cast<carcel *> ( casillaDestino ) )
	{
	  //Encarcela al jugador
	  j->set_turnosEncarcelado ( car->get_maxTurnos() );
	}
    }
  else
    {
      cout << "Ha habido suerte, no hay duda. No existe una casilla de crcel, asique quedas libre." << endl;
    }
  return;
}

/**
 * Mtodo que coloca a un jugador dado en una casilla dada
 */
void tablero::colocarJugador ( jugador* j, int numCasilla )
{  
  //Relaciona casilla con jugador y jugador con casilla.
  tablero::contiene[ numCasilla ] -> situarJugadorEnCasilla ( j ); 
  j->set_estaEn ( tablero::contiene[ numCasilla ] );
}

/**
 * Mtodo que desliga al jugador de la casilla en la que se encuentra y le coloca
 * en la casilla indicada por el numero de casilla final
 */
int tablero::moverA ( jugador* j, int numCasillaFinal )
{
  // Movimiento:
  //	- Desligar jugador de la casilla anterior
  //	- Colocar en la nueva casilla
  casilla* c = j->get_estaEn();
  int numCasillaInicial = c->get_numero();
  tablero::contiene[ numCasillaInicial ] -> eliminarJugador ( j );
  colocarJugador ( j, numCasillaFinal );
  
  // Representa el movimiento que realizara
  cout << dibujarMovimiento ( numCasillaInicial, numCasillaFinal );
  
  return 0;
}

/**
 * Mtodo que mueve un jugador de una casilla a otra
 * Devuelve el resultado de la accin ejecutada en funcin de la casilla destino
 * Si pasa por la casilla de salida, se paga el sueldo
 * En caso de pasar por la casilla de salida se pagará el salario configurado. Un jugador
 * pasara por la casilla de salida si:
 *
 *	1- ( Casilla final < Casilla inicial ) && ( Casilla final >= Casilla salida )
 *      o
 *	2- ( Casilla inicial < Casilla salida) && ( Casilla final >= Casilla salida )
 *
 * que simplificándose queda representado con la siguiente expresion:
 *	( ( Casilla final < Casilla inicial ) || ( Casilla inicial < Casilla salida ) ) 
 *	&& ( Casilla final >= Casilla salida )
 */
int tablero::mover( jugador* j, int totalDados, bool tiradaDoble, bool salario )
{
  casilla* casillaInicial = j->get_estaEn();
  int numCasillaInicial = casillaInicial->get_numero();
  int resultadoAccion = 0;
  int turnoInicial = get_turno();
  int numCasillaFinal = ( numCasillaInicial + totalDados ) % (contiene.size());    
  
  //Mueve el jugador al destino
  moverA (j, numCasillaFinal );
  
  //Establece tiradas dobles
  int tiradasDoblesJugador = j->get_tiradasDobles ();
  if ( tiradaDoble == true )
    j->set_tiradasDobles ( ++tiradasDoblesJugador );
  else
    {
      j->set_tiradasDobles( 0 );
    }

  // Si hay 3 tiradas dobles, encarcela al jugador
  if ( tiradasDoblesJugador == 3 )
    {      
      cout << "No se puede correr tanto, a la cárcel por hacer 3 tiradas dobles seguidas" << endl;
      j->set_tiradasDobles ( 0 );
      encarcelar ( j );
    }
  else 
    {
      // Pagar el sueldo si pasa por la casilla de salida y si est definida dicha casilla.
      // Adems se debe mover para delante al jugador.
      // 1 - Mirar si está definida la casilla de salida
      // 2 - Comprobar que pasa por la salida
      if ( ( (  dynamic_cast <salida*> ( contiene[ get_salida() ] ) ) //Existe casilla salida
	     && ( ( numCasillaFinal < numCasillaInicial ) || ( numCasillaInicial < get_salida() ) ) 
	     && ( numCasillaFinal > get_salida() ) ) && salario == true && totalDados > 0 )
	{
	  //Pagar el salario -> Acción de la casilla de salida.
	  contiene[ get_salida() ]->accion ( j );
	}
      
      resultadoAccion = contiene[ numCasillaFinal ]->accion( j );     
    }
  //Establecer el turno: Avanzar turno si entra en la carcel o si no es una tirada doble
  if ( !tiradaDoble || juega [ get_turno() ]->get_turnosEncarcelado () > 0 )
    {
      set_turno( ( turnoInicial + 1 ) % juega.size() );
      for ( int i = 0; i != juega.size(); i++ )
	juega[i]->actualizarLogs();
    }
  //Establece el turno
  else
    set_turno ( turnoInicial );

  return resultadoAccion;
}

/**
 * Mtodo que mueve el jugador a la casilla indicada en la variable numCasillaFinal
 * Ejecuta la accin de la casilla destino
 */
int tablero::moverAAccion ( jugador* j, int numCasillaFinal )
{
  // Movimiento:
  //	- Desligar jugador de la casilla anterior
  //	- Colocar en la nueva casilla
  casilla* c = j->get_estaEn();
  int numCasillaInicial = c->get_numero();
  tablero::contiene[ numCasillaInicial ] -> eliminarJugador ( j );
  colocarJugador ( j, numCasillaFinal );
  
  // Representa el movimiento que realizará
  cout << dibujarMovimiento ( numCasillaInicial, numCasillaFinal );
  
  int resultadoAccion = tablero::contiene[ numCasillaFinal ]->accion( j );     

  return resultadoAccion;
}

/**
 * Mtodo por el que un jugador paga la cantidad establecida al resto de jugadores.
 * Si se trata de una cantidad negativa cobrar dicha cantidad de los demas.
 * Los jugadores que no puedan pagar sern declarados morosos
 */
int tablero::pagarATodos ( jugador *j, long cantidad )
{
  //1- ver si tendr dinero para pagar a todos:
  //   Si tiene->pagar
  //   Si no -> Ser moroso y deber dinero a todos los dems jugadores
  int numAcreedores = juega.size() - 1;
  long pagoTotal = numAcreedores * cantidad;
  
  //Se paga, si no tiene dinero suficiente dinero, se indica que el resto de jugadores son acreedores
  if ( j->pagar ( pagoTotal ) < 0 )
    {
      cout << j->get_nombre() << " no puedes hacer frente al pago. Debes conseguir dinero o sers eliminado" << endl;
      for ( int i = 0; i < juega.size(); i++ )	
	if ( j != juega[i] )	    
	  juega[i]->set_estado( juega[i]->get_estado() - cantidad );
    }
  //Si hay dinero suficiente, cobrarn
  else
    {     
      for ( int i = 0; i < juega.size(); i++ )	
	if ( j != juega[i] )
	  {
	    //Si algun jugador no puede pagar, se debe indicar como moroso.
	    if ( juega[i]->pagar ( 0 - cantidad ) < 0)
	      {		
		cout << juega[i]->get_nombre() << " no puedes hacer frente al pago. Debes conseguir dinero o sers eliminado" << endl;
		j->pagar ( 0 - cantidad );
		j->set_estado ( j->get_estado() - cantidad );
	      }
	  }
    }
  return 0;
}

/**
 * Pagar el sueldo si pasa por la casilla de salida y si está definida dicha casilla.
 */
void tablero::pagarSalario ( jugador *j, int numCasillaFinal )
{
  casilla *c = j->get_estaEn ();
  int numCasillaInicial = c->get_numero();
  if ( (  dynamic_cast <salida*> ( contiene[ get_salida() ] ) ) //Existe casilla salida
       && ( ( numCasillaFinal <= numCasillaInicial ) || ( numCasillaInicial < get_salida() ) ) 
       && ( numCasillaFinal > get_salida() ) )
    {
      //Pagar el salario -> Acción de la casilla de salida.
      tablero::contiene[ get_salida() ]->accion ( j );
    }
  return;
}

/**
 * Mtodo que muestra la informacin de la banca
 */
void tablero::verBanca ()
{
  cout << get_banca();
  return;
}

/**
 * Mtodo que muestra la informacin de la casilla indicada por la variable num
 */
void tablero::verCasilla ( int num )
{
  if ( num < get_numCasillas() )
    cout << contiene[ num ];
  else
    cerr << "No existe la casilla indicada." << endl;
  return;
}

/**
 * Mtodo que inicia un proceso de negociacin entre dos jugadores.
 * Se debe indicar el jugador receptor de la oferta
 */
void tablero::negociar ()
{
  //Muestra una lista de jugadores con los que negociar
  cout << "Ejige el jugador al que deseas hacer la oferta" << endl;
  for ( int i = 0; i < get_numJugadores(); i++ )
    {
      if ( ! ( juega[ i ] == juega[ get_jugadorActual() ] ) )
	{
	  cout << "(" << i << ") " << juega[ i ]->get_nombre() << " [" << juega[i]->get_color() << "]" << endl;
	}
    }
  cout << "(C) Cancelar" << endl;
  int opcionI;
  string opcion;
  int intento = 0;
  //Elegir el jugador al que se desea hacer la oferta
  do 
    {
      if ( intento > 0 )
	cout << "[Valor no vlido] " << endl;
      opcionI = 10; 
      cout << "Elige: ";
      opcion = getchar();
      cin.clear();
      cin.ignore( 2000, '\n' );
      opcionI = atoi ( opcion.c_str() );
      intento ++;
    } while ( ( opcionI == get_jugadorActual () ) || ( opcionI == 0 && opcion != "0" && opcion != "c" && opcion != "C" ) || 
	      ( opcionI < 0 || opcionI > get_numJugadores ()  ) );
  if ( opcion == "C" || opcion == "c" )
    cout << "Cancelado" << endl;
  else
    {
      //Lanzar el proceso de negociacin
      juega [ get_jugadorActual() ]->negociar ( juega[ opcionI ] );
    }
  return;
}

/**
 * Metodo que realiza un recuento del numero de estaciones libres que hay en el tablero
 */
int tablero::estacionesLibres ()
{
  int libres = 0;
  //Recorre todas las casillas buscando estaciones que pertenezcan a la banca
  for ( int i = 0; i < contiene.size(); i++ )
    {
      if ( estacion *e = dynamic_cast <estacion*> ( contiene[i] ) )
	if ( e->get_esPropiedad () == get_banca () )
	  libres ++;
    }
  return libres;
}

/**
 * Metodo que realiza un recuento del numero de servicios ocupados que hay en el tablero
 */
int tablero::serviciosOcupados ()
{
  int ocupados = 0;
  //Recorre todas las casillas buscando servicios que no pertnezcan a la banca
  for ( int i = 0; i < contiene.size(); i++ )
    {
      if ( servicio *e = dynamic_cast <servicio*> ( contiene[i] ) )
	if ( e->get_esPropiedad () != get_banca () )
	  ocupados ++;
    }
  return ocupados;
}

/**
 * Devolver el numero total de propiedades, siendo propiedades las calles, servicios y estaciones
 */
int tablero::totalPropiedades ()
{
  int totales = 0;
  for ( int i = 0; i < contiene.size(); i++ )
    {
      if ( propiedad *p = dynamic_cast <propiedad *> ( contiene[i] ) )	
	totales ++;      
    }
  return totales;
}

/**
 * Mtodo que calcula y devuelve el tanto por ciento de las casillas que estn libres
 */
int tablero::propiedadesLibres ()
{
  int libres = 0;
  int totales = 0;
  //Recorre todas las propidades buscando las propiedades que hay y cuantas estan libres
  for ( int i = 0; i < contiene.size(); i++ )
    {
      if ( propiedad *p = dynamic_cast <propiedad *> ( contiene[i] ) )
	{
	  totales ++;
	  if ( !dynamic_cast <jugador *> ( p->get_esPropiedad() ) )	
	    libres ++;
	}
    }
  int porcentaje = ( libres * 100 / totales );
  return porcentaje;
}

/**
 * Mtodo que devuelve las casas disponibles en la banca
 */
int tablero::obtenerCasasDisponibles()
{
  return _banca->get_casasDisponibles();
}

/**
 * Mtodo que devuelve los hoteles disponibles en la banca
 */
int tablero::obtenerHotelesDisponibles()
{
  return _banca->get_hotelesDisponibles();
}

/**
 * Metodo que comprueba si el jugador actual es un jugador de IA (jugador_regas)
 */
bool tablero::jugadorClienteIA ()
{
  if ( dynamic_cast <jugador_reglas*> ( juega [ get_jugadorActual() ] ) )
    return true;
  return false;
}

/**
 * Mtodo que comprueba si alguno de los jugadores es un jugador de IA
 */
bool tablero::existeClienteIA ()
{
  bool existe = false;
  for ( int i = 0; i < juega.size(); i++ )
    if ( dynamic_cast <jugador_reglas*> ( juega [ i ] ) )
      existe = true;
  return existe;
}

/**
 * Mtodo que muestra la mejor opcion para hipotecar siguiendo el criterio de la utilidad esperada
 * tomando como limite de dinero la cantidad introducida. El jugador sobre el que se aplica es el
 * jugador actual del juego.
 */
void tablero::utilidadEsperadaHipotecar ( long cantidad )
{  
  juega[ get_jugadorActual () ] ->  calculoUtilidadEsperadaHipotecarImprimir ( cantidad );
  return;
}

/**
 * Mtodo que muestra la mejor opcion para vender edificios siguiendo el criterio de la utilidad esperada
 * tomando como limite de dinero la cantidad introducida. El jugador sobre el que se aplica es el
 * jugador actual del juego.
 */
void tablero::utilidadEsperadaVender ( long cantidad )
{  
  juega[ get_jugadorActual () ] ->  calculoUtilidadEsperadaVenderImprimir ( cantidad );
  return;
}

/**
 * Mtodo que muestra la mejor opcion para deshipotecar siguiendo el criterio de la utilidad esperada
 * tomando como limite de dinero la cantidad introducida. El jugador sobre el que se aplica es el
 * jugador actual del juego.
 */
void tablero::utilidadEsperadaDeshipotecar ( long cantidad )
{  
  juega[ get_jugadorActual () ] ->  calculoUtilidadEsperadaDeshipotecarImprimir ( cantidad );
  return;
}

/**
 * Mtodo que muestra la mejor opcion para edificar siguiendo el criterio de la utilidad esperada
 * tomando como limite de dinero la cantidad introducida. El jugador sobre el que se aplica es el
 * jugador actual del juego.
 */
void tablero::utilidadEsperadaConstruir ( long cantidad )
{  
  juega[ get_jugadorActual () ] ->  calculoUtilidadEsperadaConstruirImprimir ( cantidad );
  return;
}

/**
 * Mtodo que muestra el log indicado en el argumento tipoLog para el color que posee la ficha del colo
 * que indica el argumento colorJugador.
 */
void tablero::mostrarLog ( string colorJugador, string tipoLog )
{
  jugador *j = buscar_jugador_color ( colorJugador );
  if ( j == NULL )
    cerr << "Jugador no encontrado" << endl;
  else
    {
      if ( tipoLog == "dinero")
	j->verLogDinero ();
      else if ( tipoLog == "capital" )
	j->verLogCapital ();
      else if ( tipoLog == "propiedades" )
	j->verLogPropiedades ();
      else if ( tipoLog == "edificios" )
	j->verLogEdificios ();
      else if ( tipoLog == "construibles" )
	j->verLogConstruibles ();
      else
	cerr << "ERROR: Argumento no vlido" << endl;
    }
  return;
}

/**
 * Mtodo que guarda en fichero los logs de todos jugadores que participan en la partida
 */
void tablero::guardarLog ()
{
  for ( int i = 0; i != juega.size(); i++ )
    {
      juega[ i ]->guardarLogs ();
    }
}
