#include "tarjeta.h"

// Constructor por defecto
tarjeta::tarjeta ()
  : _numero ( 0 ),
    _texto ( "" ),
    _disponible ( true ),
    _tipoMazo ( "suerte" )
{
  // No se hace nada!
}

// constructor explcito
tarjeta::tarjeta ( int numero, string texto, bool disponible, string tipoMazo )
  : _numero ( numero ),
    _texto ( texto ),
    _disponible ( disponible ),
    _tipoMazo ( tipoMazo )
{
  // No se hace nada!
}
 
// Copy constructor
tarjeta::tarjeta (const tarjeta& right)
{
  _numero 	= right._numero;  
  _texto	= right._texto;
  _disponible	= right._disponible;
  _tipoMazo     = right._tipoMazo;
}

// Assignment operator
tarjeta& tarjeta::operator= (const tarjeta& right)
{  
  _numero 	= right._numero;  
  _texto	= right._texto;
  _disponible	= right._disponible;
  _tipoMazo     = right._tipoMazo;
  return ( *this );
}

// Operador de impresión
ostream& operator<< ( ostream &os, tarjeta *tjt )
{
  const int anchoDibujado = 40;
  string aux = " ";

  for ( int i = 1; i <= 40; i++ )
    aux += "_";
  aux += "\n";
  string texto = tjt -> get_texto();
  aux += centrarCadena ( texto, anchoDibujado + 2, texto.size() / anchoDibujado + 1, "|" );
  aux += "|";
  for ( int i = 1; i <= 40; i++ )
    aux += "_";
  aux += "|";
  os << aux << endl;
  return os;
}

// Destructor
tarjeta::~tarjeta ()
{
  // No se hace nada!
}

/**
 * Aade una accin con los valores dados como argumentos al vector de acciones de la tarjeta.
 */
void tarjeta::incluirAccion ( int pay, int advance, int advanceto, int payeach, 
			      int payhouse, int payhotel, int multiPago, bool nextutil, bool tojail,
			      bool nextrr, bool jailcard, bool cobrarSalida, 
			      int multiPagoDado, bool nextPropiedad )
{
  accion nuevaAccion ( pay, advance, advanceto, payeach, payhouse, payhotel, multiPago, nextutil, tojail,
		       nextrr, jailcard, cobrarSalida, multiPagoDado, nextPropiedad );
  
  _accion.push_back ( nuevaAccion );
}

/**
 * Retorna el nmero de acciones de la tarjeta.
 */
int tarjeta::get_numeroOpciones ()
{
  return _accion.size();
}

/**
 * Ejecuta la accin indicada por el jugador. El resultado afectar a dicho jugador.
 */
int tarjeta::ejecutar ( jugador *j )
{
  int opcion = 0;
  bool valor = false;
  //Si hay ms de una accin para elegir, el jugador debe decidir.
  if ( get_numeroOpciones() > 1 )    
    opcion = j->decidirAccionTarjeta ( _accion.size (), this );    
  //Se ejecuta la accion elegida por el jugador.
  //Si se trata de una tarjeta del tipo carcel, se incluye en la lista de tarjetas del
  //jugador y se elimina del mazo (metodo de la clase mazo)
  if ( _accion[ opcion ].ejecutar ( j ) == 1 )
    {
      j->incluir_tieneTarjeta ( this );
      set_disponible ( false );
      return 1;
    }
  return 0;
}

/**
 * Busca si alguna de las acciones dadas se corresponde con una tarjeta de carcel. Si es as,
 * se devuelve el nmero de accin a la que corresponde.
 */
int tarjeta::buscarAccionCarcel ()
{
  for ( int i = 0; i < _accion.size (); i ++ )
    {
      if ( _accion[ i ].get_tarjetaCarcel() )
	return i;
    }
  return ( -1 );
}

/**
 * Busca si alguna de las acciones dadas se corresponde con cobro de dinero por parte del jugador. 
 * Si es as, se devuelve el nmero de accin a la que corresponde.
 */
int tarjeta::buscarCobrar ()
{
  int max = 0;
  int accionCobrar = -1;
  for ( int i = 0; i < _accion.size (); i ++ )
    {
      if ( _accion[ i ].get_pagar() < max )
	{
	  max = _accion [ i ].get_pagar ();
	  accionCobrar = i;	  
	}
    }
  return accionCobrar;
}
