/***********************************************************************************
 *  CLASE TARJETA
 *  Clase que reperesenta una tarjeta del juego monopoly. Una tarjeta estar formada por
 *  un conjunto de acciones y pertenecer a un nico mazo o baraja.
 *
 *  Requiere:
 *     - clase string
 *     - clase vector
 *     - clase iostream
 *
 *  Autor: Javier Aragn Zabalegui (PFC para la uc3m: street Master's)
 *  Contacto: 100039831@alumnos.uc3m.es
 *  version 1.0
 **********************************************************************************/


#ifndef __TARJETA__
#define __TARJETA__


#include "accion.h"
#include "cadenas.h"

#include <string>
#include <vector>
#include <iostream>
using namespace std;


class tarjeta {

 private:

  //Atributos
  int _numero;
  string _texto;
  bool _disponible;
  string _tipoMazo;
  
  //Asociaciones
  vector <accion> _accion;

 public:

  // Constructor por defecto
  tarjeta ();
  
  // Constructor explcito
  tarjeta ( int numero, string texto, bool disponible, string tipoMazo );
  
  // Copy constructor
  tarjeta ( const tarjeta& right );

  // Operador de asignacin
  tarjeta& operator= (const tarjeta& right);

  friend ostream& operator<< ( ostream& os, tarjeta *tjt );

  // Destructor
  ~tarjeta ();

  // Mtodos GET
  int get_numero () const;
  string get_texto () const;
  bool get_disponible () const;
  string get_tipoMazo () const;

  // Mtodos SET
  void set_numero ( int numero );
  void set_texto ( string texto );
  void set_disponible ( bool disponible );
  void set_tipoMazo ( string tipoMazo );

  //Otros mtodos

  /**
   * Aade una accin con los valores dados como argumentos al vector de acciones de la tarjeta.
   */
  void incluirAccion ( int pay, int advance, int advanceto, int payeach, 
		       int payhouse, int payhotel, int multiPago, bool nextutil, bool tojail,
		       bool nextrr, bool jailcard, bool cobrarSalida, int multiPagoDado, 
		       bool nextPropiedad );

  /**
   * Retorna el nmero de acciones de la tarjeta.
   */
  int get_numeroOpciones ();

  /**
   * Ejecuta la accin indicada por el jugador. El resultado afectar a dicho jugador.
   */
  int ejecutar ( jugador* player );

  /**
   * Busca si alguna de las acciones dadas se corresponde con una tarjeta de carcel. Si es as,
   * se devuelve el nmero de accin a la que corresponde.
   */
  int buscarAccionCarcel ();

  /**
   * Busca si alguna de las acciones dadas se corresponde con cobro de dinero por parte del jugador. 
   * Si es as, se devuelve el nmero de accin a la que corresponde.
   */
  int buscarCobrar ();
};

inline int tarjeta::get_numero () const
{
  return _numero;
}

inline string tarjeta::get_texto () const
{
  return _texto;
}

inline bool tarjeta::get_disponible () const
{
  return _numero;
}

inline string tarjeta::get_tipoMazo () const
{
  return _tipoMazo;
}

inline void tarjeta::set_numero ( int numero )
{
  _numero = numero;
}

inline void tarjeta::set_texto( string texto )
{
  _texto = texto;
}

inline void tarjeta::set_disponible ( bool disponible )
{
  _disponible = disponible;
}

inline void tarjeta::set_tipoMazo ( string tipoMazo )
{
  _tipoMazo = tipoMazo;
}

#endif
